@@ -43,7 +43,6 @@ typedef struct resource_values_s {
 	MaritimeInfo info;
 } resource_values_t;
 
-static Map *map;
 static int quote_num;
 /* used to avoid multiple chat messages when more than one other player
  * must discard resources */
@@ -120,10 +119,10 @@ static void for_each_node(iterate_node_func_t * func, void *rock)
 {
 	int i, j, k;
 
-	for (i = 0; i < map->x_size; i++) {
-		for (j = 0; j < map->y_size; j++) {
+	for (i = 0; i < get_map()->x_size; i++) {
+		for (j = 0; j < get_map()->y_size; j++) {
 			for (k = 0; k < 6; k++) {
-				Node *n = map_node(map, i, j, k);
+				Node *n = map_node(get_map(), i, j, k);
 
 				if (n)
 					func(n, rock);
@@ -345,7 +344,7 @@ static void reevaluate_resources(resource_values_t * outval)
 	/*
 	 * Save the maritime info too so we know if we can do port trades
 	 */
-	map_maritime_info(map, &outval->info, my_player_num());
+	map_maritime_info(get_map(), &outval->info, my_player_num());
 }
 
 
@@ -469,10 +468,10 @@ static Node *best_settlement_spot(gboolean during_setup,
 	float bestscore = -1.0;
 	float score;
 
-	for (i = 0; i < map->x_size; i++) {
-		for (j = 0; j < map->y_size; j++) {
+	for (i = 0; i < get_map()->x_size; i++) {
+		for (j = 0; j < get_map()->y_size; j++) {
 			for (k = 0; k < 6; k++) {
-				Node *n = map_node(map, i, j, k);
+				Node *n = map_node(get_map(), i, j, k);
 				if (!n)
 					continue;
 				if (during_setup) {
@@ -507,10 +506,10 @@ static Node *best_city_spot(const resource_values_t * resval)
 	Node *best = NULL;
 	float bestscore = -1.0;
 
-	for (i = 0; i < map->x_size; i++) {
-		for (j = 0; j < map->y_size; j++) {
+	for (i = 0; i < get_map()->x_size; i++) {
+		for (j = 0; j < get_map()->y_size; j++) {
 			for (k = 0; k < 6; k++) {
-				Node *n = map_node(map, i, j, k);
+				Node *n = map_node(get_map(), i, j, k);
 				if (!n)
 					continue;
 				if ((n->owner == my_player_num())
@@ -577,7 +576,7 @@ static Edge *traverse_out(Node * n, node_seen_set_t * set, float *score,
 				    traverse_out(othernode, set,
 						 &cur_score, resval);
 
-		} else if ((e->owner == -1) && (is_edge_on_land(e))) {
+		} else if (can_road_be_built(e, my_player_num())) {
 
 			/* no owner, how good is the other node ? */
 			cur_e = e;
@@ -617,13 +616,19 @@ static Edge *best_road_to_road_spot(Node * n, float *score,
 		if (e) {
 			Node *othernode = other_node(e, n);
 
-			if (is_edge_on_land(e) && e->owner == -1) {
+			if (can_road_be_built(e, my_player_num())) {
 
 				for (j = 0; j < 3; j++) {
 					Edge *e2 = othernode->edges[j];
+					if (e2 == NULL)
+						continue;
 
+					/* We need to look further, temporarily mark this edge as having our road on it. */
+					e->owner = my_player_num();
+					e->type = BUILD_ROAD;
 
-					if ((e2) && (e2->owner == -1)) {
+					if (can_road_be_built
+					    (e2, my_player_num())) {
 						float score =
 						    score_node(other_node
 							       (e2,
@@ -635,6 +640,9 @@ static Edge *best_road_to_road_spot(Node * n, float *score,
 							best = e;
 						}
 					}
+					/* restore map to its real state */
+					e->owner = -1;
+					e->type = BUILD_NONE;
 				}
 			}
 
@@ -655,10 +663,10 @@ static Edge *best_road_to_road(const resource_values_t * resval)
 	Edge *best = NULL;
 	float bestscore = -1.0;
 
-	for (i = 0; i < map->x_size; i++) {
-		for (j = 0; j < map->y_size; j++) {
+	for (i = 0; i < get_map()->x_size; i++) {
+		for (j = 0; j < get_map()->y_size; j++) {
 			for (k = 0; k < 6; k++) {
-				Node *n = map_node(map, i, j, k);
+				Node *n = map_node(get_map(), i, j, k);
 				Edge *e;
 				float score;
 
@@ -697,10 +705,10 @@ static Edge *best_road_spot(const resource_values_t * resval)
 	 * xxx loops
 	 */
 
-	for (i = 0; i < map->x_size; i++) {
-		for (j = 0; j < map->y_size; j++) {
+	for (i = 0; i < get_map()->x_size; i++) {
+		for (j = 0; j < get_map()->y_size; j++) {
 			for (k = 0; k < 6; k++) {
-				Node *n = map_node(map, i, j, k);
+				Node *n = map_node(get_map(), i, j, k);
 
 				if ((n != NULL)
 				    && (n->owner == my_player_num())) {
@@ -849,10 +857,9 @@ static int which_resource(gint assets[NO_RESOURCE])
 /*
  * What resource do we want the most?
  *
+ * NOTE: If a resource is not available (players or bank), the
+ * resval->value[resource] should be zero.
  */
-
-/* Don't want what isn't available anymore... */
-
 static int resource_desire(gint assets[NO_RESOURCE],
 			   const resource_values_t * resval)
 {
@@ -872,6 +879,7 @@ static int resource_desire(gint assets[NO_RESOURCE],
 	}
 
 	/* desire the one we don't produce the most */
+	res = NO_RESOURCE;
 	for (i = 0; i < NO_RESOURCE; i++) {
 		if ((resval->value[i] > value) && (assets[i] < 2)) {
 			res = i;
@@ -895,6 +903,8 @@ static void findit_iterator(Node * n, void *rock)
 
 	/* if i own this node */
 	for (i = 0; i < 3; i++) {
+		if (n->edges[i] == NULL)
+			continue;
 		if (n->edges[i]->owner == my_player_num())
 			return;
 	}
@@ -927,8 +937,18 @@ static void greedy_setup_house(void)
 
 	reevaluate_resources(&resval);
 
+	if (stock_num_settlements() == 0) {
+		ai_panic(N_("No settlements in stock to use for setup"));
+		return;
+	}
+
 	node = best_settlement_spot(TRUE, &resval);
 
+	if (node == NULL) {
+		ai_panic(N_("There is no place to setup a settlement"));
+		return;
+	}
+
 	/*node_add(player, BUILD_SETTLEMENT, node->x, node->y, node->pos, FALSE); */
 	cb_build_settlement(node);
 }
@@ -948,6 +968,11 @@ static void greedy_setup_road(void)
 
 	reevaluate_resources(&resval);
 
+	if (stock_num_roads() == 0) {
+		ai_panic(N_("No roads in stock to use for setup"));
+		return;
+	}
+
 	node = void_settlement();
 
 	e = best_road_to_road_spot(node, &tmp, &resval);
@@ -960,6 +985,10 @@ static void greedy_setup_road(void)
 				break;
 			}
 		}
+		if (e == NULL) {
+			ai_panic(N_("There is no place to setup a road"));
+			return;
+		}
 	}
 
 	cb_build_road(e);
@@ -999,8 +1028,7 @@ static gboolean find_optimal_trade(gint assets[NO_RESOURCE],
 		 * to trade for what's missing.
 		 */
 		leftover_resources(assets, bt, need);
-		for (temp = 0; temp < G_N_ELEMENTS(build_preferences);
-		     temp++) {
+		for (temp = 0; temp < NO_RESOURCE; temp++) {
 			if (temp == res)
 				continue;
 			if (need[temp] > ports[temp]) {
@@ -1032,7 +1060,7 @@ static gboolean will_do_maritime_trade(gint assets[NO_RESOURCE],
 	Resource res, want, discard;
 	gint ports[NO_RESOURCE];
 
-	map_maritime_info(map, &info, my_player_num());
+	map_maritime_info(get_map(), &info, my_player_num());
 
 	for (res = 0; res < NO_RESOURCE; res++) {
 		if (info.specific_resource[res])
@@ -1386,9 +1414,9 @@ static void greedy_place_robber(void)
 	int victim_resources = -1;
 
 	ai_wait();
-	for (i = 0; i < map->x_size; i++) {
-		for (j = 0; j < map->y_size; j++) {
-			Hex *hex = map_hex(map, i, j);
+	for (i = 0; i < get_map()->x_size; i++) {
+		for (j = 0; j < get_map()->y_size; j++) {
+			Hex *hex = map_hex(get_map(), i, j);
 			float score = score_hex_hurt_opponents(hex);
 
 			if (score > bestscore) {
@@ -1532,7 +1560,7 @@ static void greedy_monopoly(void)
 	/* order resources by preference */
 	reevaluate_resources(&resval);
 
-	/* this could only infinite loop if no other players had any cards */
+	/* try to get something we need */
 	while (TRUE) {
 		r = resource_desire(assets, &resval);
 		if (r == NO_RESOURCE)
@@ -1918,7 +1946,8 @@ static void greedy_player_robbed(G_GNUC_UNUSED gint robber_num,
 }
 
 static void greedy_get_rolled_resources(gint player_num,
-					const gint * resources)
+					const gint * resources,
+					G_GNUC_UNUSED const gint * wanted)
 {
 	gint total = 0, i;
 	for (i = 0; i < NO_RESOURCE; ++i)
@@ -1960,8 +1989,6 @@ static void greedy_new_statistics(gint player_num, StatisticType type,
 
 void greedy_init(G_GNUC_UNUSED int argc, G_GNUC_UNUSED char **argv)
 {
-	map = get_map();
-
 	callbacks.setup = &greedy_setup;
 	callbacks.turn = &greedy_turn;
 	callbacks.robber = &greedy_place_robber;
