@@ -2,7 +2,7 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003,2006 Bas Wijnen <shevek@fmf.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,7 +70,9 @@ enum callback_mode {
 	MODE_TURN,		/* your turn */
 	MODE_ROBBER,		/* place robber */
 	MODE_MONOPOLY,		/* choose monopoly resource */
+	MODE_MONOPOLY_RESPONSE,	/* chosen monopoly resource, waiting */
 	MODE_PLENTY,		/* choose year of plenty resources */
+	MODE_PLENTY_RESPONSE,	/* chosen year of plenty resources, waiting */
 	MODE_ROAD_BUILD,	/* build two roads/ships/bridges */
 	MODE_DOMESTIC,		/* called for quotes */
 	MODE_QUOTE,		/* got a call for quotes */
@@ -109,7 +111,7 @@ struct callbacks {
 	 * call cb_discard. */
 	void (*discard_add) (gint player_num, gint discard_num);
 	/* a player discarded resources */
-	void (*discard_remove) (gint player_num, gint * resources);
+	void (*discard_remove) (gint player_num);
 	/* discard mode is finished. */
 	void (*discard_done) (void);
 	/* starting gold distribution */
@@ -151,12 +153,14 @@ struct callbacks {
 	void (*trade_player_end) (gint player_num);
 	/* while you're trading, someone else offers you a quote */
 	void (*trade_add_quote) (gint player_num, gint quote_num,
-				 gint * they_supply, gint * they_receive);
+				 const gint * they_supply,
+				 const gint * they_receive);
 	/* while you're trading, someone revokes a quote */
 	void (*trade_remove_quote) (gint player_num, gint quote_num);
 	/* you're trading, and a trade has just been performed. */
 	void (*trade_domestic) (gint partner_num, gint quote_num,
-				gint * we_supply, gint * we_receive);
+				const gint * we_supply,
+				const gint * we_receive);
 	/* you're trading, and a trade has just been performed. */
 	void (*trade_maritime) (gint ratio, Resource we_supply,
 				Resource we_receive);
@@ -164,7 +168,8 @@ struct callbacks {
 	void (*quote_player_end) (gint player_num);
 	/* while someone else is trading, a player makes a quote */
 	void (*quote_add) (gint player_num, gint quote_num,
-			   gint * they_supply, gint * they_receive);
+			   const gint * they_supply,
+			   const gint * they_receive);
 	/* while someone else is trading, a player revokes a quote */
 	void (*quote_remove) (gint player_num, gint quote_num);
 	/* someone else makes a call for quotes.  This is an initialization
@@ -179,12 +184,10 @@ struct callbacks {
 	void (*quote_monitor) (void);
 	/* while someone else is trading, a quote is accepted. */
 	void (*quote_trade) (gint player_num, gint partner_num,
-			     gint quote_num, gint * they_supply,
-			     gint * they_receive);
+			     gint quote_num, const gint * they_supply,
+			     const gint * they_receive);
 	/* the dice have been rolled */
 	void (*rolled_dice) (gint die1, gint die2, gint player_num);
-	/* the terminal should beep, if it likes to. */
-	void (*beep) (void);
 	/* An edge changed, it should be drawn */
 	void (*draw_edge) (Edge * edge);
 	/* A node changed, it should be drawn */
@@ -216,14 +219,22 @@ struct callbacks {
 	 * frontend should also be able to handle players not getting any
 	 * resources, because it may be called for all players in the future
 	 * The value of the resources has already been updated, and there has
-	 * been a call to resource_change when this is called. */
+	 * been a call to resource_change when this is called.
+	 * If resources is different from wanted, the player should have
+	 * received resources, but the bank was empty.  */
 	void (*get_rolled_resources) (gint player_num,
-				      const gint * resources);
+				      const gint * resources,
+				      const gint * wanted);
 	/* Something happened to someones stats.  As with resource_change,
 	 * the value must not be updated by the frontend, it has already been
 	 * done by the client. */
 	void (*new_statistics) (gint player_num, StatisticType type,
 				gint num);
+	/* Something happened to someones special points. As with
+	 * resource_change, the value must not be updated by the frontend,
+	 * it has already been done by the client. */
+	void (*new_points) (gint player_num, Points * points,
+			    gboolean added);
 	/* a viewer changed his/her name */
 	void (*viewer_name) (gint viewer_num, const gchar * name);
 	/* a player changed his/her name */
@@ -232,6 +243,8 @@ struct callbacks {
 	void (*player_quit) (gint player_num);
 	/* a viewer left the game */
 	void (*viewer_quit) (gint player_num);
+	/* respond to incoming chat messages */
+	void (*incoming_chat) (gint player_num, const gchar * chat);
 	/* something changed in the bank. */
 	void (*new_bank) (const gint * new_bank);
 	/* some communication error occurred, and it has already been logged */
@@ -239,6 +252,10 @@ struct callbacks {
 	/* mainloop.  This is initialized to run the glib main loop.  It can
 	 * be overridden */
 	void (*mainloop) (void);
+	/* exit the main loop.  The program will then quit.  This is
+	 * initialized to quit the main loop.  It should be overridden if
+	 * mainloop is. */
+	void (*quit) (void);
 };
 
 #if ENABLE_NLS
@@ -265,7 +282,8 @@ extern gboolean color_chat_enabled;
 /* these functions do things for the frontends, they should be used to make
  * changes to the board, etc.  The frontend should NEVER touch any game
  * structures directly (except for reading). */
-void cb_connect(const gchar * server, const gchar * port);
+void cb_connect(const gchar * server, const gchar * port,
+		const gchar * name, gboolean viewer);
 void cb_disconnect(void);
 void cb_roll(void);
 void cb_build_road(const Edge * edge);
@@ -278,21 +296,21 @@ void cb_buy_develop(void);
 void cb_play_develop(int card);
 void cb_undo(void);
 void cb_maritime(gint ratio, Resource supply, Resource receive);
-void cb_domestic(gint * supply, gint * receive);
+void cb_domestic(const gint * supply, const gint * receive);
 void cb_end_turn(void);
 void cb_place_robber(const Hex * hex, gint victim_num);
 void cb_choose_monopoly(gint resource);
 void cb_choose_plenty(gint * resources);
-void cb_trade_call(gint * we_supply, gint * we_receive);
-void cb_trade(gint player, gint quote, gint * supply, gint * receive);
+void cb_trade(gint player, gint quote, const gint * supply,
+	      const gint * receive);
 void cb_end_trade(void);
-void cb_quote(gint num, gint * supply, gint * receive);
+void cb_quote(gint num, const gint * supply, const gint * receive);
 void cb_delete_quote(gint num);
 void cb_end_quote(void);
 void cb_chat(const gchar * text);
 void cb_name_change(const gchar * name);
-void cb_discard(gint * resources);
-void cb_choose_gold(gint * resources);
+void cb_discard(const gint * resources);
+void cb_choose_gold(const gint * resources);
 
 /* check functions used by front ends and internally */
 /* these functions don't change anything in the program, they are used to get
@@ -304,12 +322,18 @@ gboolean can_play_any_develop(void);
 Player *player_get(gint num);
 gboolean player_is_viewer(gint num);
 Viewer *viewer_get(gint num);
-gchar *player_name(gint player_num, gboolean word_caps);
+const gchar *player_name(gint player_num, gboolean word_caps);
 gint player_get_score(gint player_num);
 gint my_player_num(void);
 const gchar *my_player_name(void);
+gboolean my_player_viewer(void);
 gint num_players(void);
 gint current_player(void);
+/** Find the player or viewer with name
+ *  @param name The name to search for
+ *  @return the player/viewer number or -1 if the name was not found
+ */
+gint find_player_by_name(const gchar * name);
 gint build_count_edges(void);
 gint build_count_settlements(void);
 gint build_count(BuildType type);
@@ -320,6 +344,7 @@ gint stock_num_settlements(void);
 gint stock_num_cities(void);
 gint stock_num_develop(void);
 gint resource_asset(Resource which);
+gint resource_count(const gint * resources);
 gint resource_total(void);
 void resource_format_type(gchar * buffer, const gint * resources);
 const gchar *resource_name(Resource which, gboolean capital);
