@@ -1,7 +1,7 @@
 /* Pioneers - Implementation of the excellent Settlers of Catan board game.
  *   Go buy a copy.
  *
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003,2006 Bas Wijnen <shevek@fmf.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,19 +29,25 @@
  * It is filled in by the front end. */
 struct callbacks callbacks;
 
+/* these variables must be remembered between connect and handshake */
+gchar *requested_name = NULL;
+gboolean requested_viewer;
+
 /* current callback mode */
 enum callback_mode callback_mode;
 
 /* is chat currently colourful? */
 gboolean color_chat_enabled;
 
-/* The name of the player */
-static const gchar *saved_player_name = NULL;
-
-void cb_connect(const gchar * server, const gchar * port)
+void cb_connect(const gchar * server, const gchar * port,
+		const gchar * name, gboolean viewer)
 {
 	/* connect to a server */
 	g_assert(callback_mode == MODE_INIT);
+	if (requested_name)
+		g_free(requested_name);
+	requested_name = g_strdup(name);
+	requested_viewer = viewer;
 	if (sm_connect(SM(), server, port)) {
 		if (sm_is_connected(SM())) {
 			sm_goto(SM(), mode_start);
@@ -53,18 +59,22 @@ void cb_connect(const gchar * server, const gchar * port)
 	}
 }
 
-void cb_disconnect()
+void cb_disconnect(void)
 {
 	sm_close(SM());
 	callback_mode = MODE_INIT;
 	callbacks.offline();
 }
 
-void cb_roll()
+void cb_roll(void)
 {
 	/* roll dice */
 	g_assert(callback_mode == MODE_TURN && !have_rolled_dice());
 	sm_send(SM(), "roll\n");
+	/* This should really be sm_push, but on return it should be
+	 * sm_pop_noenter; sm_goto_noenter; sm_push, and since sm_pop_noenter
+	 * doesn't exist, the combination is changed into
+	 * sm_goto here and sm_goto_noenter; sm_push later. */
 	sm_goto(SM(), mode_roll_response);
 }
 
@@ -129,12 +139,12 @@ void cb_build_city(const Node * node)
 	sm_push(SM(), mode_build_response);
 }
 
-void cb_buy_develop()
+void cb_buy_develop(void)
 {
 	/* buy development card */
 	g_assert(callback_mode == MODE_TURN && can_buy_develop());
 	sm_send(SM(), "buy-develop\n");
-	sm_goto(SM(), mode_buy_develop_response);
+	sm_push(SM(), mode_buy_develop_response);
 }
 
 void cb_play_develop(int card)
@@ -145,7 +155,7 @@ void cb_play_develop(int card)
 	sm_push(SM(), mode_play_develop_response);
 }
 
-void cb_undo()
+void cb_undo(void)
 {
 	/* undo a move */
 	g_assert(callback_mode == MODE_TURN
@@ -165,7 +175,7 @@ void cb_maritime(gint ratio, Resource supply, Resource receive)
 	sm_push(SM(), mode_trade_maritime_response);
 }
 
-void cb_domestic(gint * supply, gint * receive)
+void cb_domestic(const gint * supply, const gint * receive)
 {
 	/* call for quotes */
 	g_assert(callback_mode == MODE_TURN
@@ -179,7 +189,7 @@ void cb_domestic(gint * supply, gint * receive)
 	}
 }
 
-void cb_end_turn()
+void cb_end_turn(void)
 {
 	/* end turn or road building or setup */
 	g_assert(callback_mode == MODE_TURN
@@ -214,7 +224,8 @@ void cb_choose_plenty(gint * resources)
 	sm_push(SM(), mode_year_of_plenty_response);
 }
 
-void cb_trade(gint player, gint quote, gint * supply, gint * receive)
+void cb_trade(gint player, gint quote, const gint * supply,
+	      const gint * receive)
 {
 	/* accept a domestic trade */
 	g_assert(callback_mode == MODE_DOMESTIC);
@@ -224,7 +235,7 @@ void cb_trade(gint player, gint quote, gint * supply, gint * receive)
 	sm_push(SM(), mode_trade_domestic_response);
 }
 
-void cb_end_trade()
+void cb_end_trade(void)
 {
 	/* stop trading */
 	g_assert(callback_mode == MODE_DOMESTIC);
@@ -232,7 +243,7 @@ void cb_end_trade()
 	sm_push(SM(), mode_domestic_finish_response);
 }
 
-void cb_quote(gint num, gint * supply, gint * receive)
+void cb_quote(gint num, const gint * supply, const gint * receive)
 {
 	/* make a quote */
 	g_assert(callback_mode == MODE_QUOTE);
@@ -249,12 +260,12 @@ void cb_delete_quote(gint num)
 	sm_push(SM(), mode_quote_delete_response);
 }
 
-void cb_end_quote()
+void cb_end_quote(void)
 {
 	/* stop trading */
 	g_assert(callback_mode == MODE_QUOTE);
 	sm_send(SM(), "domestic-quote finish\n");
-	sm_goto(SM(), mode_quote_finish_response);
+	sm_push(SM(), mode_quote_finish_response);
 }
 
 void cb_chat(const gchar * text)
@@ -266,19 +277,12 @@ void cb_chat(const gchar * text)
 
 void cb_name_change(const gchar * name)
 {
-	saved_player_name = g_strdup(name);
 	/* change your name */
-	copy_player_name(name);
-	if (callback_mode != MODE_INIT)
-		sm_send(SM(), "name %s\n", name);
+	g_assert(callback_mode != MODE_INIT);
+	sm_send(SM(), "name %s\n", name);
 }
 
-const gchar *my_player_name(void)
-{
-	return saved_player_name;
-}
-
-void cb_discard(gint * resources)
+void cb_discard(const gint * resources)
 {
 	/* discard resources */
 	g_assert(callback_mode == MODE_DISCARD);
@@ -286,7 +290,7 @@ void cb_discard(gint * resources)
 	sm_send(SM(), "discard %R\n", resources);
 }
 
-void cb_choose_gold(gint * resources)
+void cb_choose_gold(const gint * resources)
 {
 	/* choose gold */
 	g_assert(callback_mode == MODE_GOLD);
@@ -311,12 +315,12 @@ const GameParams *get_game_params(void)
 	return game_params;
 }
 
-gint game_resources()
+gint game_resources(void)
 {
 	return game_params->resource_count;
 }
 
-gint game_victory_points()
+gint game_victory_points(void)
 {
 	return game_params->victory_points;
 }
@@ -404,7 +408,7 @@ gboolean turn_can_build_settlement(void)
 	    && map_can_place_settlement(map, my_player_num());
 }
 
-gboolean turn_can_build_city()
+gboolean turn_can_build_city(void)
 {
 	return have_rolled_dice()
 	    && stock_num_cities() > 0
@@ -412,7 +416,7 @@ gboolean turn_can_build_city()
 	    && map_can_upgrade_settlement(map, my_player_num());
 }
 
-gboolean turn_can_trade()
+gboolean turn_can_trade(void)
 {
 	/* We are not allowed to trade before we have rolled the dice,
 	 * or after we have done built a settlement / city, or after
@@ -568,7 +572,7 @@ int pirate_count_victims(const Hex * hex, gint * victim_list)
 	return num_victims;
 }
 
-Map *get_map()
+Map *get_map(void)
 {
 	return map;
 }
