@@ -2,7 +2,7 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003-2005 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003-2006 Bas Wijnen <shevek@fmf.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,9 +37,8 @@
 static enum callback_mode previous_mode;
 GameParams *game_params;
 Map *map;
-static gchar *saved_name;
-struct recovery_info_t {
-	gchar prevstate[40];
+static struct recovery_info_t {
+	gchar *prevstate;
 	gint turnnum;
 	gint playerturn;
 	gint numdiscards;
@@ -49,7 +48,7 @@ struct recovery_info_t {
 	gboolean bought_develop;
 	GList *build_list;
 	gboolean ship_moved;
-};
+} recovery_info;
 
 static gboolean global_unhandled(StateMachine * sm, gint event);
 static gboolean global_filter(StateMachine * sm, gint event);
@@ -73,14 +72,15 @@ static gboolean mode_domestic_trade(StateMachine * sm, gint event);
 static gboolean mode_domestic_quote(StateMachine * sm, gint event);
 static gboolean mode_domestic_monitor(StateMachine * sm, gint event);
 static gboolean mode_game_over(StateMachine * sm, gint event);
-static gboolean mode_choose_gold(StateMachine * sm, gint event);
 static gboolean mode_wait_resources(StateMachine * sm, gint event);
+static gboolean mode_recovery_wait_start_response(StateMachine * sm,
+						  gint event);
 static void recover_from_disconnect(StateMachine * sm,
 				    struct recovery_info_t *rinfo);
 
 /* Create and/or return the client state machine.
  */
-StateMachine *SM()
+StateMachine *SM(void)
 {
 	static StateMachine *state_machine;
 	if (state_machine == NULL) {
@@ -130,8 +130,7 @@ static void dummy_discard_add(G_GNUC_UNUSED gint player_num,
 			      G_GNUC_UNUSED gint discard_num)
 {;
 }
-static void dummy_discard_remove(G_GNUC_UNUSED gint player_num,
-				 G_GNUC_UNUSED gint * resources)
+static void dummy_discard_remove(G_GNUC_UNUSED gint player_num)
 {;
 }
 static void dummy_discard_done(void)
@@ -197,8 +196,8 @@ static void dummy_trade_player_end(G_GNUC_UNUSED gint player_num)
 }
 static void dummy_trade_add_quote(G_GNUC_UNUSED gint player_num,
 				  G_GNUC_UNUSED gint quote_num,
-				  G_GNUC_UNUSED gint * they_supply,
-				  G_GNUC_UNUSED gint * they_receive)
+				  G_GNUC_UNUSED const gint * they_supply,
+				  G_GNUC_UNUSED const gint * they_receive)
 {;
 }
 static void dummy_trade_remove_quote(G_GNUC_UNUSED gint player_num,
@@ -207,8 +206,8 @@ static void dummy_trade_remove_quote(G_GNUC_UNUSED gint player_num,
 }
 static void dummy_trade_domestic(G_GNUC_UNUSED gint partner_num,
 				 G_GNUC_UNUSED gint quote_num,
-				 G_GNUC_UNUSED gint * we_supply,
-				 G_GNUC_UNUSED gint * we_receive)
+				 G_GNUC_UNUSED const gint * we_supply,
+				 G_GNUC_UNUSED const gint * we_receive)
 {;
 }
 static void dummy_trade_maritime(G_GNUC_UNUSED gint ratio,
@@ -221,8 +220,8 @@ static void dummy_quote_player_end(G_GNUC_UNUSED gint player_num)
 }
 static void dummy_quote_add(G_GNUC_UNUSED gint player_num,
 			    G_GNUC_UNUSED gint quote_num,
-			    G_GNUC_UNUSED gint * they_supply,
-			    G_GNUC_UNUSED gint * they_receive)
+			    G_GNUC_UNUSED const gint * they_supply,
+			    G_GNUC_UNUSED const gint * they_receive)
 {;
 }
 static void dummy_quote_remove(G_GNUC_UNUSED gint player_num,
@@ -241,8 +240,8 @@ static void dummy_quote_monitor(void)
 static void dummy_quote_trade(G_GNUC_UNUSED gint player_num,
 			      G_GNUC_UNUSED gint partner_num,
 			      G_GNUC_UNUSED gint quote_num,
-			      G_GNUC_UNUSED gint * they_supply,
-			      G_GNUC_UNUSED gint * they_receive)
+			      G_GNUC_UNUSED const gint * they_supply,
+			      G_GNUC_UNUSED const gint * they_receive)
 {;
 }
 static void dummy_rolled_dice(G_GNUC_UNUSED gint die1,
@@ -250,9 +249,6 @@ static void dummy_rolled_dice(G_GNUC_UNUSED gint die1,
 			      G_GNUC_UNUSED gint player_num)
 {;
 }
-static void dummy_beep(void)
-{;
-}
 static void dummy_draw_edge(G_GNUC_UNUSED Edge * edge)
 {;
 }
@@ -291,7 +287,8 @@ static void dummy_player_robbed(G_GNUC_UNUSED gint robber_num,
 }
 static void dummy_get_rolled_resources(G_GNUC_UNUSED gint player_num,
 				       G_GNUC_UNUSED const gint *
-				       resources)
+				       resources,
+				       G_GNUC_UNUSED const gint * wanted)
 {;
 }
 static void dummy_new_statistics(G_GNUC_UNUSED gint player_num,
@@ -299,6 +296,11 @@ static void dummy_new_statistics(G_GNUC_UNUSED gint player_num,
 				 G_GNUC_UNUSED gint num)
 {;
 }
+static void dummy_new_points(G_GNUC_UNUSED gint player_num,
+			     G_GNUC_UNUSED Points * points,
+			     G_GNUC_UNUSED gboolean added)
+{
+}
 static void dummy_viewer_name(G_GNUC_UNUSED gint viewer_num,
 			      G_GNUC_UNUSED const gchar * name)
 {;
@@ -313,6 +315,10 @@ static void dummy_player_quit(G_GNUC_UNUSED gint player_num)
 static void dummy_viewer_quit(G_GNUC_UNUSED gint player_num)
 {;
 }
+static void dummy_incoming_chat(G_GNUC_UNUSED gint player_num,
+				G_GNUC_UNUSED const gchar * chat)
+{;
+}
 static void dummy_new_bank(G_GNUC_UNUSED const gint * new_bank)
 {;
 }
@@ -367,7 +373,6 @@ void client_init(void)
 	callbacks.quote_monitor = &dummy_quote_monitor;
 	callbacks.quote_trade = &dummy_quote_trade;
 	callbacks.rolled_dice = &dummy_rolled_dice;
-	callbacks.beep = &dummy_beep;
 	callbacks.draw_edge = &dummy_draw_edge;
 	callbacks.draw_node = &dummy_draw_node;
 	callbacks.bought_develop = &dummy_bought_develop;
@@ -380,13 +385,15 @@ void client_init(void)
 	callbacks.player_robbed = &dummy_player_robbed;
 	callbacks.get_rolled_resources = &dummy_get_rolled_resources;
 	callbacks.new_statistics = &dummy_new_statistics;
+	callbacks.new_points = &dummy_new_points;
 	callbacks.viewer_name = &dummy_viewer_name;
 	callbacks.player_name = &dummy_player_name;
 	callbacks.player_quit = &dummy_player_quit;
 	callbacks.viewer_quit = &dummy_viewer_quit;
+	callbacks.incoming_chat = &dummy_incoming_chat;
 	callbacks.new_bank = &dummy_new_bank;
 	callbacks.error = &dummy_error;
-	/* mainloop is not set here */
+	/* mainloop and quit are not set here */
 	resource_init();
 }
 
@@ -421,8 +428,8 @@ static gboolean global_filter(StateMachine * sm, gint event)
 	case SM_NET_CLOSE:
 		log_message(MSG_ERROR,
 			    _("We have been kicked out of the game.\n"));
-		sm_pop_all(sm);
-		sm_goto(sm, mode_offline);
+		waiting_for_network(FALSE);
+		sm_pop_all_and_goto(sm, mode_offline);
 		callbacks.network_status(_("Offline"));
 		return TRUE;
 	default:
@@ -436,39 +443,56 @@ static gboolean global_filter(StateMachine * sm, gint event)
  */
 static gboolean global_unhandled(StateMachine * sm, gint event)
 {
-	char str[512];
+	gchar *str;
 
 	switch (event) {
 	case SM_NET_CLOSE:
-		g_error("SM_NET_CLOSE not caught by global_filter.\n");
+		g_error("SM_NET_CLOSE not caught by global_filter");
 	case SM_RECV:
 		/* all errors start with ERR */
-		if (sm_recv(sm, "ERR %S", str, sizeof(str))) {
-			log_message(MSG_ERROR, "Error (%s): %s\n",
+		if (sm_recv(sm, "ERR %S", &str)) {
+			log_message(MSG_ERROR, _("Error (%s): %s\n"),
 				    sm_current_name(sm), str);
 			callbacks.error(str);
+			g_free(str);
 			return TRUE;
 		}
 		/* notices which are not errors should appear in the message
 		 * window */
-		if (sm_recv(sm, "NOTE %S", str, sizeof(str))) {
-			log_message(MSG_ERROR, "Notice: %s\n", str);
+		if (sm_recv(sm, "NOTE %S", &str)) {
+			log_message(MSG_ERROR, _("Notice: %s\n"), _(str));
+			g_free(str);
+			return TRUE;
+		}
+		/* A notice with 1 argument */
+		if (sm_recv(sm, "NOTE1 %S", &str)) {
+			gchar *message;
+			gchar **parts;
+
+			parts = g_strsplit(str, "|", 2);
+			message = g_strdup_printf(_(parts[1]), parts[0]);
+			log_message(MSG_ERROR, _("Notice: %s\n"), message);
+			g_strfreev(parts);
+			g_free(message);
+			g_free(str);
 			return TRUE;
 		}
 		/* protocol extensions which may be ignored have this prefix
 		 * before the next protocol changing version of the game is
 		 * released.  Notify the client about it anyway. */
-		if (sm_recv(sm, "extension %S", str, sizeof(str))) {
+		if (sm_recv(sm, "extension %S", &str)) {
 			log_message(MSG_INFO,
 				    "Ignoring extension used by server: %s\n",
 				    str);
+			g_free(str);
 			return TRUE;
 		}
 		/* we're receiving strange things */
-		if (sm_recv(sm, "%S", str, sizeof(str))) {
+		if (sm_recv(sm, "%S", &str)) {
 			log_message(MSG_ERROR,
 				    "Unknown message in %s: %s\n",
 				    sm_current_name(sm), str);
+			g_free(str);
 			return TRUE;
 		}
 		/* this is never reached: everything matches "%S" */
@@ -482,21 +506,6 @@ static gboolean global_unhandled(StateMachine * sm, gint event)
 	return FALSE;
 }
 
-/*----------------------------------------------------------------------
- * Hooks for GUI events that can happen at almost any time
- */
-void copy_player_name(const gchar * name)
-{
-	char *tmp = g_strdup(name);
-	tmp = g_strstrip(tmp);
-	if (*tmp != '\0') {
-		if (saved_name != NULL)
-			g_free(saved_name);
-		saved_name = g_strdup(tmp);
-	}
-	g_free(tmp);
-}
-
 /*----------------------------------------------------------------------
  * Server notifcations about player name changes and chat messages.
  * These can happen in any state (maybe this should be moved to
@@ -505,15 +514,16 @@ void copy_player_name(const gchar * name)
 static gboolean check_chat_or_name(StateMachine * sm)
 {
 	gint player_num;
-	char str[512];
+	gchar *str;
 
-	if (sm_recv
-	    (sm, "player %d chat %S", &player_num, str, sizeof(str))) {
-		chat_parser(player_num, str);
+	if (sm_recv(sm, "player %d chat %S", &player_num, &str)) {
+		callbacks.incoming_chat(player_num, str);
+		g_free(str);
 		return TRUE;
 	}
-	if (sm_recv(sm, "player %d is %S", &player_num, str, sizeof(str))) {
+	if (sm_recv(sm, "player %d is %S", &player_num, &str)) {
 		player_change_name(player_num, str);
+		g_free(str);
 		return TRUE;
 	}
 	return FALSE;
@@ -530,12 +540,11 @@ static gboolean check_other_players(StateMachine * sm)
 	DevelType devel_type;
 	Resource resource_type, supply_type, receive_type;
 	gint player_num, victim_num, card_idx, backwards;
-	gint turn_num, discard_num, gold_num, num, ratio, die1, die2, x, y,
-	    pos;
+	gint turn_num, discard_num, num, ratio, die1, die2, x, y, pos;
 	gint id;
-	gint resource_list[NO_RESOURCE];
+	gint resource_list[NO_RESOURCE], wanted_list[NO_RESOURCE];
 	gint sx, sy, spos, dx, dy, dpos;
-	gchar str[512];
+	gchar *str;
 
 	if (check_chat_or_name(sm))
 		return TRUE;
@@ -564,10 +573,46 @@ static gboolean check_other_players(StateMachine * sm)
 		player_build_remove(player_num, build_type, x, y, pos);
 		return TRUE;
 	}
-	if (sm_recv(sm, "receives %R", resource_list)) {
-		player_resource_action(player_num, _("%s receives %s.\n"),
+	if (sm_recv(sm, "receives %R %R", resource_list, wanted_list)) {
+		gint i;
+		for (i = 0; i < NO_RESOURCE; ++i) {
+			if (resource_list[i] == wanted_list[i])
+				continue;
+			if (resource_list[i] == 0) {
+				log_message(MSG_RESOURCE,
+					    _
+					    ("%s does not receive any %s, because the bank is empty.\n"),
+					    player_name(player_num, TRUE),
+					    resource_name(i, FALSE));
+			} else {
+				gint j, list[NO_RESOURCE];
+				gchar *buff;
+				for (j = 0; j < NO_RESOURCE; ++j)
+					list[j] = 0;
+				list[i] = resource_list[i];
+				resource_list[i] = 0;
+				buff = resource_format_num(list);
+				log_message(MSG_RESOURCE,
+					    _
+					    ("%s only receives %s, because the bank didn't have any more.\n"),
+					    player_name(player_num, TRUE),
+					    buff);
+				g_free(buff);
+				resource_apply_list(player_num, list, 1);
+			}
+		}
+		if (resource_count(resource_list) != 0)
+			player_resource_action(player_num,
+					       _("%s receives %s.\n"),
+					       resource_list, 1);
+		callbacks.get_rolled_resources(player_num, resource_list,
+					       wanted_list);
+		return TRUE;
+	}
+	if (sm_recv(sm, "plenty %R", resource_list)) {
+		/* Year of Plenty */
+		player_resource_action(player_num, _("%s takes %s.\n"),
 				       resource_list, 1);
-		callbacks.get_rolled_resources(player_num, resource_list);
 		return TRUE;
 	}
 	if (sm_recv(sm, "spent %R", resource_list)) {
@@ -609,12 +654,7 @@ static gboolean check_other_players(StateMachine * sm)
 	if (sm_recv(sm, "discarded %R", resource_list)) {
 		player_resource_action(player_num, _("%s discarded %s.\n"),
 				       resource_list, -1);
-		callbacks.discard_remove(player_num, resource_list);
-		return TRUE;
-	}
-	if (sm_recv(sm, "prepare-gold %d", &gold_num)) {
-		sm_goto(sm, mode_choose_gold);
-		callbacks.gold_add(player_num, gold_num);
+		callbacks.discard_remove(player_num);
 		return TRUE;
 	}
 	if (sm_recv(sm, "is-robber")) {
@@ -648,8 +688,9 @@ static gboolean check_other_players(StateMachine * sm)
 		player_longest_road(player_num);
 		return TRUE;
 	}
-	if (sm_recv(sm, "get-point %d %d %S", &id, &num, str, sizeof(str))) {
+	if (sm_recv(sm, "get-point %d %d %S", &id, &num, &str)) {
 		player_get_point(player_num, id, str, num);
+		g_free(str);
 		return TRUE;
 	}
 	if (sm_recv(sm, "lose-point %d", &id)) {
@@ -764,7 +805,7 @@ gboolean mode_connecting(StateMachine * sm, gint event)
 gboolean mode_start(StateMachine * sm, gint event)
 {
 	gint player_num, total_num;
-	gchar version[512];
+	gchar *version;
 
 	sm_state_name(sm, "mode_start");
 
@@ -781,16 +822,26 @@ gboolean mode_start(StateMachine * sm, gint event)
 		return TRUE;
 	}
 	if (sm_recv(sm, "status report")) {
-		if (saved_name != NULL) {
-			sm_send(sm, "status reconnect %s\n", saved_name);
-			return TRUE;
+		if (requested_viewer) {
+			if (requested_name[0] != '\0') {
+				sm_send(sm, "status viewer %s\n",
+					requested_name);
+			} else {
+				sm_send(sm, "status newviewer\n");
+			}
 		} else {
-			sm_send(sm, "status newplayer\n");
-			return TRUE;
+			if (requested_name[0] != '\0') {
+				sm_send(sm, "status reconnect %s\n",
+					requested_name);
+			} else {
+				sm_send(sm, "status newplayer\n");
+			}
 		}
+		return TRUE;
 	}
 	if (sm_recv(sm, "player %d of %d, welcome to pioneers server %S",
-		    &player_num, &total_num, version, sizeof(version))) {
+		    &player_num, &total_num, &version)) {
+		g_free(version);
 		player_set_my_num(player_num);
 		player_set_total_num(total_num);
 		sm_send(sm, "players\n");
@@ -799,12 +850,12 @@ gboolean mode_start(StateMachine * sm, gint event)
 		return TRUE;
 	}
 	if (sm_recv(sm, "ERR sorry, version conflict")) {
-		sm_pop_all(sm);
-		sm_goto(sm, mode_offline);
+		sm_pop_all_and_goto(sm, mode_offline);
 		callbacks.network_status(_("Offline"));
 		callbacks.instructions(_("Version mismatch"));
 		log_message(MSG_ERROR,
-			    "Connect Error: Version mismatch! Please make sure client and server are up to date.\n");
+			    _("Version mismatch. Please make sure client "
+			      "and server are up to date.\n"));
 		return TRUE;
 	}
 	return check_chat_or_name(sm);
@@ -843,7 +894,7 @@ static gboolean mode_player_list(StateMachine * sm, gint event)
  */
 static gboolean mode_load_game(StateMachine * sm, gint event)
 {
-	gchar str[512];
+	gchar *str;
 
 	sm_state_name(sm, "mode_load_game");
 	if (event != SM_RECV)
@@ -859,14 +910,28 @@ static gboolean mode_load_game(StateMachine * sm, gint event)
 		map = game_params->map;
 		stock_init();
 		develop_init();
+		/* initialize global recovery info struct */
+		recovery_info.prevstate = NULL;
+		recovery_info.turnnum = -1;
+		recovery_info.playerturn = -1;
+		recovery_info.numdiscards = -1;
+		recovery_info.rolled_dice = FALSE;
+		recovery_info.die1 = -1;
+		recovery_info.die2 = -1;
+		recovery_info.played_develop = FALSE;
+		recovery_info.bought_develop = FALSE;
+		recovery_info.build_list = NULL;
+		recovery_info.ship_moved = FALSE;
+
 		sm_send(sm, "gameinfo\n");
 		sm_goto(sm, mode_load_gameinfo);
 		return TRUE;
 	}
 	if (check_other_players(sm))
 		return TRUE;
-	if (sm_recv(sm, "%S", str, sizeof(str))) {
+	if (sm_recv(sm, "%S", &str)) {
 		params_load_line(game_params, str);
+		g_free(str);
 		return TRUE;
 	}
 	return FALSE;
@@ -876,11 +941,7 @@ static gboolean mode_load_game(StateMachine * sm, gint event)
  */
 static gboolean mode_load_gameinfo(StateMachine * sm, gint event)
 {
-	gchar str[512];
 	gint x, y, pos, owner;
-	static struct recovery_info_t rinfo = { "", -1, -1, -1, FALSE,
-		-1, -1, FALSE, FALSE, NULL, FALSE
-	};
 	static gboolean disconnected = FALSE;
 	static gboolean have_bank = FALSE;
 	static gint devcardidx = -1;
@@ -890,6 +951,8 @@ static gboolean mode_load_gameinfo(StateMachine * sm, gint event)
 	gint opnum, opnassets, opncards, opnsoldiers;
 	gboolean pchapel, puniv, pgov, plibr, pmarket, plongestroad,
 	    plargestarmy;
+	gint point_id, point_points;
+	gchar *point_name;
 	DevelType devcard;
 	gint devcardturnbought;
 	BuildType btype;
@@ -918,7 +981,7 @@ static gboolean mode_load_gameinfo(StateMachine * sm, gint event)
 		callback_mode = MODE_WAIT_TURN;	/* allow chatting */
 		callbacks.start_game();
 		if (disconnected) {
-			recover_from_disconnect(sm, &rinfo);
+			sm_goto(sm, mode_recovery_wait_start_response);
 		} else {
 			sm_send(sm, "start\n");
 			sm_goto(sm, mode_start_response);
@@ -936,37 +999,40 @@ static gboolean mode_load_gameinfo(StateMachine * sm, gint event)
 			stock_use_develop();
 		return TRUE;
 	}
-	if (sm_recv(sm, "turn num %d", &rinfo.turnnum)) {
+	if (sm_recv(sm, "turn num %d", &recovery_info.turnnum)) {
 		return TRUE;
 	}
-	if (sm_recv(sm, "player turn: %d", &rinfo.playerturn)) {
+	if (sm_recv(sm, "player turn: %d", &recovery_info.playerturn)) {
 		return TRUE;
 	}
-	if (sm_recv(sm, "dice rolled: %d %d", &rinfo.die1, &rinfo.die2)) {
-		rinfo.rolled_dice = TRUE;
+	if (sm_recv
+	    (sm, "dice rolled: %d %d", &recovery_info.die1,
+	     &recovery_info.die2)) {
+		recovery_info.rolled_dice = TRUE;
 		return TRUE;
 	}
-	if (sm_recv(sm, "dice value: %d %d", &rinfo.die1, &rinfo.die2)) {
+	if (sm_recv
+	    (sm, "dice value: %d %d", &recovery_info.die1,
+	     &recovery_info.die2)) {
 		return TRUE;
 	}
 	if (sm_recv(sm, "played develop")) {
-		rinfo.played_develop = TRUE;
+		recovery_info.played_develop = TRUE;
 		return TRUE;
 	}
 	if (sm_recv(sm, "moved ship")) {
-		rinfo.ship_moved = TRUE;
+		recovery_info.ship_moved = TRUE;
 		return TRUE;
 	}
 	if (sm_recv(sm, "bought develop")) {
-		rinfo.bought_develop = TRUE;
+		recovery_info.bought_develop = TRUE;
 		return TRUE;
 	}
 	if (sm_recv(sm, "player disconnected")) {
 		disconnected = TRUE;
 		return TRUE;
 	}
-	if (sm_recv(sm, "state %S", str, sizeof(str))) {
-		strcpy(rinfo.prevstate, str);
+	if (sm_recv(sm, "state %S", &recovery_info.prevstate)) {
 		return TRUE;
 	}
 	if (sm_recv(sm, "playerinfo: resources: %R", resources)) {
@@ -1036,6 +1102,15 @@ static gboolean mode_load_gameinfo(StateMachine * sm, gint event)
 		}
 		return TRUE;
 	}
+	if (sm_recv
+	    (sm, "get-point %d %d %d %S", &opnum, &point_id, &point_points,
+	     &point_name)) {
+		Points *points =
+		    points_new(point_id, point_name, point_points);
+		player_modify_points(opnum, points, TRUE);	/* Added */
+		return TRUE;
+	}
+
 	if (sm_recv
 	    (sm, "otherplayerinfo: %d %d %d %d %d %d %d %d %d %d %d",
 	     &opnum, &opnassets, &opncards, &opnsoldiers, &pchapel, &puniv,
@@ -1080,7 +1155,8 @@ static gboolean mode_load_gameinfo(StateMachine * sm, gint event)
 		rec->x = x;
 		rec->y = y;
 		rec->pos = pos;
-		rinfo.build_list = g_list_append(rinfo.build_list, rec);
+		recovery_info.build_list =
+		    g_list_append(recovery_info.build_list, rec);
 		return TRUE;
 	}
 	if (sm_recv(sm, "RO%d,%d", &x, &y)) {
@@ -1156,8 +1232,6 @@ gboolean mode_build_response(StateMachine * sm, gint event)
 		}
 		if (check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -1184,8 +1258,6 @@ gboolean mode_move_response(StateMachine * sm, gint event)
 		}
 		if (check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -1214,9 +1286,6 @@ gboolean mode_done_response(StateMachine * sm, gint event)
 		}
 		if (check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		/* pop back to parent if "done" didn't work */
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -1224,17 +1293,16 @@ gboolean mode_done_response(StateMachine * sm, gint event)
 
 static char *setup_msg(void)
 {
-	static char msg[1024];
-	char *msg_end;
+	gchar *msg;
 	const gchar *parts[3];
 	int num_parts;
 	int idx;
 
 	if (is_setup_double())
-		strcpy(msg,
-		       _("Build two settlements, each with a connecting"));
+		msg = g_strdup(_("Build two settlements, "
+				 "each with a connecting"));
 	else
-		strcpy(msg, _("Build a settlement with a connecting"));
+		msg = g_strdup(_("Build a settlement with a connecting"));
 	num_parts = 0;
 	if (setup_can_build_road())
 		parts[num_parts++] = _("road");
@@ -1243,21 +1311,24 @@ static char *setup_msg(void)
 	if (setup_can_build_ship())
 		parts[num_parts++] = _("ship");
 
-	msg_end = msg + strlen(msg);
 	for (idx = 0; idx < num_parts; idx++) {
+		gchar *old;
 		if (idx > 0) {
 			if (idx == num_parts - 1) {
-				strcpy(msg_end, _(" or"));
-				msg_end += 3;
+				old = msg;
+				msg =
+				    g_strdup_printf("%s%s", msg, _(" or"));
+				g_free(old);
 			} else {
-				*msg_end++ = ',';
+				old = msg;
+				msg = g_strdup_printf("%s,", msg);
+				g_free(old);
 			}
 		}
-		*msg_end++ = ' ';
-		strcpy(msg_end, parts[idx]);
-		msg_end += strlen(msg_end);
+		old = msg;
+		msg = g_strdup_printf("%s %s", msg, parts[idx]);
+		g_free(old);
 	}
-	*msg_end = '\0';
 
 	return msg;
 }
@@ -1303,7 +1374,8 @@ static gboolean mode_idle(StateMachine * sm, gint event)
 	switch (event) {
 	case SM_ENTER:
 		callback_mode = MODE_WAIT_TURN;
-		callbacks.instructions(_("Waiting for your turn"));
+		if (!player_is_viewer(my_player_num()))
+			callbacks.instructions(_("Waiting for your turn"));
 		break;
 	case SM_RECV:
 		if (sm_recv(sm, "setup %d", &backwards)) {
@@ -1384,9 +1456,6 @@ gboolean mode_robber_response(StateMachine * sm, gint event)
 		}
 		if (check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		/* pop to parent if move failed */
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -1502,6 +1571,7 @@ gboolean mode_monopoly_response(StateMachine * sm, gint event)
 	sm_state_name(sm, "mode_monopoly_response");
 	switch (event) {
 	case SM_ENTER:
+		callback_mode = MODE_MONOPOLY_RESPONSE;
 		waiting_for_network(TRUE);
 		break;
 	case SM_RECV:
@@ -1513,9 +1583,6 @@ gboolean mode_monopoly_response(StateMachine * sm, gint event)
 		}
 		if (check_other_players(sm))
 			return TRUE;
-		/* pop to parent if it didn't work */
-		sm_pop(sm);
-		waiting_for_network(FALSE);
 		break;
 	}
 	return FALSE;
@@ -1550,6 +1617,7 @@ gboolean mode_year_of_plenty_response(StateMachine * sm, gint event)
 	sm_state_name(sm, "mode_year_of_plenty_response");
 	switch (event) {
 	case SM_ENTER:
+		callback_mode = MODE_PLENTY_RESPONSE;
 		waiting_for_network(TRUE);
 		break;
 	case SM_RECV:
@@ -1561,9 +1629,6 @@ gboolean mode_year_of_plenty_response(StateMachine * sm, gint event)
 		}
 		if (check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		/* if it fails, go back to parent */
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -1613,6 +1678,11 @@ gboolean mode_play_develop_response(StateMachine * sm, gint event)
 			waiting_for_network(FALSE);
 			develop_played(my_player_num(), card_idx,
 				       card_type);
+			/* This mode should be popped off after the response
+			 * has been handled.  However, for the development
+			 * card, a new mode must be pushed immediately.  Due
+			 * to the lack of sm_pop_noenter this is combined as
+			 * sm_goto */
 			switch (card_type) {
 			case DEVEL_ROAD_BUILDING:
 				sm_goto(sm, mode_road_building);
@@ -1634,8 +1704,6 @@ gboolean mode_play_develop_response(StateMachine * sm, gint event)
 		}
 		if (check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -1716,8 +1784,6 @@ gboolean mode_roll_response(StateMachine * sm, gint event)
 		}
 		if (check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_goto(sm, mode_turn);
 		break;
 	}
 	return FALSE;
@@ -1760,14 +1826,12 @@ gboolean mode_buy_develop_response(StateMachine * sm, gint event)
 	case SM_RECV:
 		if (sm_recv(sm, "bought-develop %D", &card_type)) {
 			develop_bought_card(card_type);
-			sm_goto(sm, mode_turn_rolled);
+			sm_pop(sm);
 			waiting_for_network(FALSE);
 			return TRUE;
 		}
 		if (check_other_players(sm))
 			return TRUE;
-		sm_goto(sm, mode_turn_rolled);
-		waiting_for_network(FALSE);
 		break;
 	}
 	return FALSE;
@@ -1803,8 +1867,6 @@ gboolean mode_undo_response(StateMachine * sm, gint event)
 		}
 		if (check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -1878,13 +1940,12 @@ gboolean mode_trade_call_response(StateMachine * sm, gint event)
 		if (sm_recv(sm, "domestic-trade call supply %R receive %R",
 			    we_supply, we_receive)) {
 			waiting_for_network(FALSE);
+			/* pop response state + push trade state == goto */
 			sm_goto(sm, mode_domestic_trade);
 			return TRUE;
 		}
 		if (check_trading(sm) || check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -1908,12 +1969,12 @@ gboolean mode_trade_maritime_response(StateMachine * sm, gint event)
 	case SM_RECV:
 		/* Handle out-of-resource-cards */
 		if (sm_recv(sm, "ERR no-cards %r", &no_receive)) {
-			gchar buf_receive[128];
+			gchar *buf_receive;
 
-			resource_cards(0, no_receive, buf_receive,
-				       sizeof(buf_receive));
+			buf_receive = resource_cards(0, no_receive);
 			log_message(MSG_TRADE, _("Sorry, %s available.\n"),
 				    buf_receive);
+			g_free(buf_receive);
 			waiting_for_network(FALSE);
 			sm_pop(sm);
 			return TRUE;
@@ -1930,8 +1991,6 @@ gboolean mode_trade_maritime_response(StateMachine * sm, gint event)
 		}
 		if (check_trading(sm) || check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -1958,8 +2017,6 @@ gboolean mode_trade_call_again_response(StateMachine * sm, gint event)
 		}
 		if (check_trading(sm) || check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -1996,8 +2053,6 @@ gboolean mode_trade_domestic_response(StateMachine * sm, gint event)
 		}
 		if (check_trading(sm) || check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -2022,9 +2077,6 @@ gboolean mode_domestic_finish_response(StateMachine * sm, gint event)
 		}
 		if (check_trading(sm) || check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		/* pop back to parent if finish fails */
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -2054,7 +2106,8 @@ static gboolean mode_domestic_trade(StateMachine * sm, gint event)
  * Quote processing - all quoting done inside a nested state machine.
  */
 
-static gboolean check_quoting(StateMachine * sm)
+static gboolean check_quoting(StateMachine * sm, gint exitdepth,
+			      gboolean monitor)
 {
 	gint player_num, partner_num, quote_num;
 	gint they_supply[NO_RESOURCE];
@@ -2079,6 +2132,8 @@ static gboolean check_quoting(StateMachine * sm)
 	}
 	if (sm_recv(sm, "domestic-trade call supply %R receive %R",
 		    they_supply, they_receive)) {
+		if (monitor)
+			sm_pop(sm);
 		callbacks.quote(player_num, they_supply, they_receive);
 		return TRUE;
 	}
@@ -2096,7 +2151,7 @@ static gboolean check_quoting(StateMachine * sm)
 		callback_mode = previous_mode;
 		callbacks.quote_end();
 		sm_send(sm, "domestic-quote exit\n");
-		sm_pop(sm);
+		sm_multipop(sm, exitdepth);
 		return TRUE;
 	}
 
@@ -2116,14 +2171,13 @@ gboolean mode_quote_finish_response(StateMachine * sm, gint event)
 	case SM_RECV:
 		if (sm_recv(sm, "domestic-quote finish")) {
 			waiting_for_network(FALSE);
+			/* pop response + push monitor == goto */
 			sm_goto(sm, mode_domestic_monitor);
+			callbacks.quote_monitor();
 			return TRUE;
 		}
-		if (check_quoting(sm) || check_other_players(sm))
+		if (check_quoting(sm, 2, FALSE) || check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_goto(sm, mode_domestic_quote);
-		callbacks.quote_monitor();
 		break;
 	}
 	return FALSE;
@@ -2152,10 +2206,8 @@ gboolean mode_quote_submit_response(StateMachine * sm, gint event)
 			sm_pop(sm);
 			return TRUE;
 		}
-		if (check_quoting(sm) || check_other_players(sm))
+		if (check_quoting(sm, 2, FALSE) || check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -2179,10 +2231,8 @@ gboolean mode_quote_delete_response(StateMachine * sm, gint event)
 			sm_pop(sm);
 			return TRUE;
 		}
-		if (check_quoting(sm) || check_other_players(sm))
+		if (check_quoting(sm, 2, FALSE) || check_other_players(sm))
 			return TRUE;
-		waiting_for_network(FALSE);
-		sm_pop(sm);
 		break;
 	}
 	return FALSE;
@@ -2202,7 +2252,7 @@ static gboolean mode_domestic_quote(StateMachine * sm, gint event)
 		}
 		break;
 	case SM_RECV:
-		if (check_quoting(sm) || check_other_players(sm))
+		if (check_quoting(sm, 1, FALSE) || check_other_players(sm))
 			return TRUE;
 		break;
 	default:
@@ -2218,7 +2268,7 @@ static gboolean mode_domestic_monitor(StateMachine * sm, gint event)
 	sm_state_name(sm, "mode_domestic_monitor");
 	switch (event) {
 	case SM_RECV:
-		if (check_quoting(sm) || check_other_players(sm))
+		if (check_quoting(sm, 2, TRUE) || check_other_players(sm))
 			return TRUE;
 		break;
 	default:
@@ -2259,7 +2309,7 @@ static gboolean mode_recovery_wait_start_response(StateMachine * sm,
 		break;
 	case SM_RECV:
 		if (sm_recv(sm, "OK")) {
-			sm_pop(sm);
+			recover_from_disconnect(sm, &recovery_info);
 			return TRUE;
 		}
 		break;
@@ -2275,6 +2325,7 @@ static void recover_from_disconnect(StateMachine * sm,
 	StateFunc modeturn;
 	GList *next;
 
+	callbacks.start_game();
 	if (rinfo->turnnum > 0)
 		turn_begin(rinfo->playerturn, rinfo->turnnum);
 	if (rinfo->rolled_dice) {
@@ -2295,18 +2346,42 @@ static void recover_from_disconnect(StateMachine * sm,
 						 rinfo->bought_develop);
 	}
 
+	/* setup_begin must be called before the build list is created,
+	 * because it contains a call to build_clear() */
+	if (strcmp(rinfo->prevstate, "SETUP") == 0 ||
+	    strcmp(rinfo->prevstate, "RSETUP") == 0 ||
+	    strcmp(rinfo->prevstate, "SETUPDOUBLE") == 0) {
+		if (strcmp(rinfo->prevstate, "SETUPDOUBLE") == 0) {
+			setup_begin_double(my_player_num());
+		} else {
+			setup_begin(my_player_num());
+		}
+	}
+
+	/* The build list must be created before the state is entered,
+	 * because when the state is entered the frontend is called and
+	 * it will want to have the build list present. */
+	if (rinfo->build_list) {
+		for (next = rinfo->build_list; next != NULL;
+		     next = g_list_next(next)) {
+			BuildRec *build = (BuildRec *) next->data;
+			build_add(build->type, build->x, build->y,
+				  build->pos, FALSE);
+		}
+		rinfo->build_list = buildrec_free(rinfo->build_list);
+	}
+
 	if (strcmp(rinfo->prevstate, "PREGAME") == 0) {
 		sm_goto(sm, mode_idle);
 	} else if (strcmp(rinfo->prevstate, "IDLE") == 0) {
 		sm_goto(sm, mode_idle);
 	} else if (strcmp(rinfo->prevstate, "SETUP") == 0 ||
+		   strcmp(rinfo->prevstate, "RSETUP") == 0 ||
 		   strcmp(rinfo->prevstate, "SETUPDOUBLE") == 0) {
-		if (strcmp(rinfo->prevstate, "SETUP") == 0) {
-			setup_begin(my_player_num());
-		} else {
-			setup_begin_double(my_player_num());
-		}
 		sm_goto_noenter(sm, mode_idle);
+		if (strcmp(rinfo->prevstate, "SETUP") != 0) {
+			sm_push_noenter(sm, mode_wait_resources);
+		}
 		sm_push(sm, mode_setup);
 	} else if (strcmp(rinfo->prevstate, "TURN") == 0) {
 		sm_goto_noenter(sm, mode_idle);
@@ -2336,32 +2411,25 @@ static void recover_from_disconnect(StateMachine * sm,
 		sm_goto_noenter(sm, mode_idle);
 		sm_push_noenter(sm, modeturn);
 		callback_mode = MODE_TURN;
-		sm_push(sm, mode_choose_gold);
+		sm_push(sm, mode_wait_resources);
 	} else if (strcmp(rinfo->prevstate, "ROADBUILDING") == 0) {
 		sm_goto_noenter(sm, mode_idle);
 		sm_push_noenter(sm, modeturn);
 		callback_mode = MODE_TURN;
 		sm_push(sm, mode_road_building);
-	}
-
-	if (rinfo->build_list) {
-		for (next = rinfo->build_list; next != NULL;
-		     next = g_list_next(next)) {
-			BuildRec *build = (BuildRec *) next->data;
-			build_add(build->type, build->x, build->y,
-				  build->pos, FALSE);
-		}
-		rinfo->build_list = buildrec_free(rinfo->build_list);
-	}
-
-	/* tell the server we have finished reconnecting and wait for
-	   its ok response */
-	sm_push(sm, mode_recovery_wait_start_response);
+	} else
+		g_warning("Not entering any state after reconnect, "
+			  "please report this as a bug.  "
+			  "Should enter state \"%s\"", rinfo->prevstate);
+	g_free(rinfo->prevstate);
 }
 
 /*----------------------------------------------------------------------
- * Nested state machine for handling resource card choice.  We enter
- * gold-choose mode whenever any player has to choose resources.
+ * Nested state machine for handling resource card distribution.  We enter
+ * here whenever resources might be distributed.
+ *
+ * This also includes choosing gold.  Gold-choose mode is entered the first
+ * time prepare-gold is received.
  * 
  * When in gold-choose mode, as in discard mode, a section of the GUI
  * changes to list all players who must choose resources.  This is
@@ -2370,24 +2438,22 @@ static void recover_from_disconnect(StateMachine * sm,
  * Only the top player in the list can actually choose, the rest is waiting
  * for their turn.
  */
-static gboolean mode_choose_gold(StateMachine * sm, gint event)
+static gboolean mode_wait_resources(StateMachine * sm, gint event)
 {
 	gint resource_list[NO_RESOURCE], bank[NO_RESOURCE];
 	gint player_num, gold_num;
 
-	sm_state_name(sm, "mode_choose_gold");
+	sm_state_name(sm, "mode_wait_resources");
 	switch (event) {
-	case SM_ENTER:
-		if (callback_mode != MODE_GOLD
-		    && callback_mode != MODE_GOLD_WAIT) {
-			previous_mode = callback_mode;
-			callback_mode = MODE_GOLD_WAIT;
-		}
-		callbacks.gold();
-		break;
 	case SM_RECV:
 		if (sm_recv(sm, "player %d prepare-gold %d", &player_num,
 			    &gold_num)) {
+			if (callback_mode != MODE_GOLD
+			    && callback_mode != MODE_GOLD_WAIT) {
+				previous_mode = callback_mode;
+				callback_mode = MODE_GOLD_WAIT;
+				callbacks.gold();
+			}
 			callbacks.gold_add(player_num, gold_num);
 			return TRUE;
 		}
@@ -2405,8 +2471,11 @@ static gboolean mode_choose_gold(StateMachine * sm, gint event)
 			return TRUE;
 		}
 		if (sm_recv(sm, "done-resources")) {
-			callback_mode = previous_mode;
-			callbacks.gold_done();
+			if (callback_mode == MODE_GOLD
+			    || callback_mode == MODE_GOLD_WAIT) {
+				callback_mode = previous_mode;
+				callbacks.gold_done();
+			}
 			sm_pop(sm);
 			return TRUE;
 		}
@@ -2419,32 +2488,9 @@ static gboolean mode_choose_gold(StateMachine * sm, gint event)
 	return FALSE;
 }
 
-static gboolean mode_wait_resources(StateMachine * sm, gint event)
-{
-	sm_state_name(sm, "mode_wait_resources");
-	switch (event) {
-	case SM_RECV:
-		if (sm_recv(sm, "done-resources")) {
-			sm_pop(sm);
-			return TRUE;
-		}
-		if (check_other_players(sm))
-			return TRUE;
-		break;
-	}
-	return FALSE;
-}
-
 gboolean can_trade_domestic(void)
 {
-	gint idx;
-	/* Check if we can do domestic trading */
-	if (!game_params->domestic_trade)
-		return FALSE;
-	for (idx = 0; idx < NO_RESOURCE; idx++)
-		if (resource_asset(idx) > 0)
-			return TRUE;
-	return FALSE;
+	return game_params->domestic_trade;
 }
 
 gboolean can_trade_maritime(void)
