@@ -1,7 +1,7 @@
 /* Pioneers - Implementation of the excellent Settlers of Catan board game.
  *   Go buy a copy.
  *
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003,2006 Bas Wijnen <shevek@fmf.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@
 /* variables */
 extern Map *map;
 extern GameParams *game_params;
+extern gchar *requested_name;
+extern gboolean requested_viewer;
 
 /********* client.c ***********/
 /* client initialization */
@@ -38,9 +40,6 @@ void client_start(int argc, char **argv);	/* after frontend initialization */
 /* access the state machine (a client has only one state machine) */
 StateMachine *SM(void);
 
-/* handle a player name change */
-void copy_player_name(const gchar * name);
-
 /* state machine modes */
 gboolean mode_connecting(StateMachine * sm, gint event);
 gboolean mode_start(StateMachine * sm, gint event);
@@ -63,15 +62,13 @@ gboolean mode_quote_finish_response(StateMachine * sm, gint event);
 gboolean mode_quote_submit_response(StateMachine * sm, gint event);
 gboolean mode_quote_delete_response(StateMachine * sm, gint event);
 
-/******** chat.c ************/
-/* parse an incoming chat message */
-void chat_parser(gint player_num, char *chat_str);
-
 /******* player.c **********/
 void player_reset(void);
 void player_set_my_num(gint player_num);
 void player_modify_statistic(gint player_num, StatisticType type,
 			     gint num);
+void player_modify_points(gint player_num, Points * points,
+			  gboolean added);
 void player_change_name(gint player_num, const gchar * name);
 void player_has_quit(gint player_num);
 void player_largest_army(gint player_num);
@@ -91,11 +88,12 @@ void player_build_remove(gint player_num, BuildType type, gint x, gint y,
 void player_build_move(gint player_num, gint sx, gint sy, gint spos,
 		       gint dx, gint dy, gint dpos, gint isundo);
 void player_resource_action(gint player_num, const gchar * action,
-			    gint * resource_list, gint mult);
+			    const gint * resource_list, gint mult);
 void player_get_point(gint player_num, gint id, const gchar * str,
 		      gint num);
 void player_lose_point(gint player_num, gint id);
 void player_take_point(gint player_num, gint id, gint old_owner);
+gint find_viewer_by_name(const gchar * name);
 
 /********* build.c **********/
 void build_clear(void);
@@ -143,14 +141,12 @@ void stock_use_develop(void);
 
 /********** resource.c **********/
 void resource_init(void);
-void resource_apply_list(gint player_num, gint * resources,
+void resource_apply_list(gint player_num, const gint * resources,
 			 gint multiplier);
-void resource_cards(gint num, Resource which, gchar * buffer,
-		    gint buf_size);
-gint resource_count(gint * resources);
-void resource_format_num(gchar * desc, guint desc_size, gint * resources);
+gchar *resource_cards(gint num, Resource which);
+gchar *resource_format_num(const gint * resources);
 void resource_log_list(gint player_num, const gchar * action,
-		       gint * resources);
+		       const gint * resources);
 void resource_modify(Resource type, gint num);
 void set_bank(const gint * new_bank);
 void modify_bank(const gint * bank_change);
