@@ -53,7 +53,7 @@ static struct {
 
 static DevelDeck *develop_deck;	/* our deck of development cards */
 
-void develop_init()
+void develop_init(void)
 {
 	int idx;
 	if (develop_deck != NULL)
@@ -175,38 +175,37 @@ void develop_played(gint player_num, gint card_idx, DevelType type)
 void monopoly_player(gint player_num, gint victim_num, gint num,
 		     Resource type)
 {
-	gchar buf[128];
+	gchar *buf;
 	gchar *tmp;
 
 	player_modify_statistic(player_num, STAT_RESOURCES, num);
 	player_modify_statistic(victim_num, STAT_RESOURCES, -num);
 
-	resource_cards(num, type, buf, sizeof(buf));
+	buf = resource_cards(num, type);
 	if (player_num == my_player_num()) {
 		/* I get the cards!
 		 */
 		log_message(MSG_STEAL, _("You get %s from %s.\n"),
 			    buf, player_name(victim_num, FALSE));
 		resource_modify(type, num);
-		return;
-	}
-	if (victim_num == my_player_num()) {
+	} else if (victim_num == my_player_num()) {
 		/* I lose the cards!
 		 */
 		log_message(MSG_STEAL, _("%s took %s from you.\n"),
 			    player_name(player_num, TRUE), buf);
 		resource_modify(type, -num);
-		return;
+	} else {
+		/* I am a bystander
+		 */
+		tmp = g_strdup(player_name(player_num, TRUE));
+		log_message(MSG_STEAL, _("%s took %s from %s.\n"),
+			    tmp, buf, player_name(victim_num, FALSE));
+		g_free(tmp);
 	}
-	/* I am a bystander
-	 */
-	tmp = g_strdup(player_name(player_num, TRUE));
-	log_message(MSG_STEAL, _("%s took %s from %s.\n"),
-		    tmp, buf, player_name(victim_num, FALSE));
-	g_free(tmp);
+	g_free(buf);
 }
 
-void develop_begin_turn()
+void develop_begin_turn(void)
 {
 	played_develop = FALSE;
 	bought_develop = FALSE;
@@ -218,10 +217,16 @@ gboolean can_play_develop(gint card)
 	    || !deck_card_playable(develop_deck, played_develop, card,
 				   turn_num()))
 		return FALSE;
+	if (deck_card_type(develop_deck, card) == DEVEL_ROAD_BUILDING
+	    && !road_building_can_build_road()
+	    && !road_building_can_build_ship()
+	    && !road_building_can_build_bridge())
+		return FALSE;
+
 	return TRUE;
 }
 
-gboolean can_play_any_develop()
+gboolean can_play_any_develop(void)
 {
 	int i;
 	for (i = 0; i < develop_deck->num_cards; ++i)
@@ -230,13 +235,13 @@ gboolean can_play_any_develop()
 	return FALSE;
 }
 
-gboolean can_buy_develop()
+gboolean can_buy_develop(void)
 {
 	return have_rolled_dice()
 	    && stock_num_develop() > 0 && can_afford(cost_development());
 }
 
-gboolean have_bought_develop()
+gboolean have_bought_develop(void)
 {
 	return bought_develop;
 }
