@@ -2,7 +2,7 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003,2006 Bas Wijnen <shevek@fmf.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 #include "config.h"
 #include <stdio.h>
+#include <string.h>
 
 #include "game.h"
 #include "map.h"
@@ -38,7 +39,7 @@ static gint num_total_players = 4;	/* total number of players in the game */
 
 /* this function is called when the game starts, to clean up from the
  * previous game. */
-void player_reset()
+void player_reset(void)
 {
 	gint i, idx;
 
@@ -53,10 +54,10 @@ void player_reset()
 		}
 		while (players[i].points != NULL) {
 			Points *points = players[i].points->data;
-			g_free(points->name);
-			g_free(points);
 			players[i].points =
 			    g_list_remove(players[i].points, points);
+			points_free(points);
+			g_free(points);
 		}
 		for (idx = 0; idx < G_N_ELEMENTS(players[i].statistics);
 		     ++idx)
@@ -87,7 +88,22 @@ Viewer *viewer_get(gint num)
 	return NULL;
 }
 
-gchar *player_name(gint player_num, gboolean word_caps)
+/** Find a viewer with the given name.
+ *  @param name The name to find
+ *  @return -1 if the name is not found
+ */
+gint find_viewer_by_name(const gchar * name)
+{
+	GList *list;
+	for (list = viewers; list != NULL; list = g_list_next(list)) {
+		Viewer *viewer = list->data;
+		if (!strcmp(viewer->name, name))
+			return viewer->num;
+	}
+	return -1;
+}
+
+const gchar *player_name(gint player_num, gboolean word_caps)
 {
 	static gchar buff[256];
 	if (player_num >= num_total_players) {
@@ -116,21 +132,28 @@ gchar *player_name(gint player_num, gboolean word_caps)
 gint player_get_score(gint player_num)
 {
 	Player *player = player_get(player_num);
+	GList *list;
 	gint i, score;
 
 	for (i = 0, score = 0; i < G_N_ELEMENTS(player->statistics); i++) {
 		score += stat_get_vp_value(i) * player->statistics[i];
 	}
 
+	list = player->points;
+	while (list) {
+		Points *points = list->data;
+		score += points->points;
+		list = g_list_next(list);
+	}
 	return score;
 }
 
-gint my_player_num()
+gint my_player_num(void)
 {
 	return my_player_id;
 }
 
-gint num_players()
+gint num_players(void)
 {
 	return num_total_players;
 }
@@ -147,6 +170,15 @@ void player_modify_statistic(gint player_num, StatisticType type, gint num)
 	callbacks.new_statistics(player_num, type, num);
 }
 
+void player_modify_points(gint player_num, Points * points, gboolean added)
+{
+	Player *player = player_get(player_num);
+	/* if !added -> is already removed */
+	if (added)
+		player->points = g_list_append(player->points, points);
+	callbacks.new_points(player_num, points, added);
+}
+
 void player_change_name(gint player_num, const gchar * name)
 {
 	Player *player;
@@ -186,7 +218,7 @@ void player_change_name(gint player_num, const gchar * name)
 	if (old_name == NULL)
 		log_message(MSG_INFO, _("Player %d is now %s.\n"),
 			    player_num, name);
-	else
+	else if (strcmp(old_name, name))
 		log_message(MSG_INFO, _("%s is now %s.\n"), old_name,
 			    name);
 	if (old_name != NULL)
@@ -272,10 +304,11 @@ void player_set_current(gint player_num)
 {
 	turn_player = player_num;
 	if (player_num != my_player_num()) {
-		char buffer[100];
-		snprintf(buffer, sizeof(buffer), _("Waiting for %s."),
-			 player_name(player_num, FALSE));
+		gchar *buffer;
+		buffer = g_strdup_printf(_("Waiting for %s."),
+					 player_name(player_num, FALSE));
 		callbacks.instructions(buffer);
+		g_free(buffer);
 		return;
 	}
 	build_new_turn();
@@ -288,8 +321,6 @@ void player_set_total_num(gint num)
 
 void player_stole_from(gint player_num, gint victim_num, Resource resource)
 {
-	gchar buf[128];
-
 	player_modify_statistic(player_num, STAT_RESOURCES, 1);
 	player_modify_statistic(victim_num, STAT_RESOURCES, -1);
 
@@ -306,7 +337,9 @@ void player_stole_from(gint player_num, gint victim_num, Resource resource)
 			    player_name(player_num, TRUE),
 			    player_name(victim_num, FALSE));
 	} else {
-		resource_cards(1, resource, buf, sizeof(buf));
+		gchar *buf;
+
+		buf = resource_cards(1, resource);
 		if (player_num == my_player_num()) {
 			/* We stole a card :-) */
 			log_message(MSG_STEAL,
@@ -320,6 +353,7 @@ void player_stole_from(gint player_num, gint victim_num, Resource resource)
 				    player_name(player_num, TRUE), buf);
 			resource_modify(resource, -1);
 		}
+		g_free(buf);
 	}
 	callbacks.player_robbed(player_num, victim_num, resource);
 }
@@ -327,8 +361,8 @@ void player_stole_from(gint player_num, gint victim_num, Resource resource)
 void player_domestic_trade(gint player_num, gint partner_num,
 			   gint * supply, gint * receive)
 {
-	gchar supply_desc[512];
-	gchar receive_desc[512];
+	gchar *supply_desc;
+	gchar *receive_desc;
 	gint diff;
 	gint idx;
 
@@ -353,38 +387,38 @@ void player_domestic_trade(gint player_num, gint partner_num,
 				    player_name(player_num, TRUE),
 				    player_name(partner_num, FALSE));
 		} else {
-			resource_format_num(receive_desc,
-					    sizeof(receive_desc), receive);
+			receive_desc = resource_format_num(receive);
 			log_message(MSG_TRADE,
 				    _("%s gave %s %s for free.\n"),
 				    player_name(player_num, TRUE),
 				    player_name(partner_num, FALSE),
 				    receive_desc);
+			g_free(receive_desc);
 		}
 	} else if (!resource_count(receive)) {
-		resource_format_num(supply_desc, sizeof(supply_desc),
-				    supply);
+		supply_desc = resource_format_num(supply);
 		log_message(MSG_TRADE, _("%s gave %s %s for free.\n"),
 			    player_name(partner_num, TRUE),
 			    player_name(player_num, FALSE), supply_desc);
+		g_free(supply_desc);
 	} else {
-		resource_format_num(supply_desc, sizeof(supply_desc),
-				    supply);
-		resource_format_num(receive_desc, sizeof(receive_desc),
-				    receive);
+		supply_desc = resource_format_num(supply);
+		receive_desc = resource_format_num(receive);
 		log_message(MSG_TRADE,
 			    _("%s gave %s %s in exchange for %s.\n"),
 			    player_name(player_num, TRUE),
 			    player_name(partner_num, FALSE), receive_desc,
 			    supply_desc);
+		g_free(supply_desc);
+		g_free(receive_desc);
 	}
 }
 
 void player_maritime_trade(gint player_num,
 			   gint ratio, Resource supply, Resource receive)
 {
-	gchar buf_give[128];
-	gchar buf_receive[128];
+	gchar *buf_give;
+	gchar *buf_receive;
 	gint resources[NO_RESOURCE];
 	gint idx;
 
@@ -400,10 +434,12 @@ void player_maritime_trade(gint player_num,
 		resource_modify(receive, 1);
 	}
 
-	resource_cards(ratio, supply, buf_give, sizeof(buf_give));
-	resource_cards(1, receive, buf_receive, sizeof(buf_receive));
+	buf_give = resource_cards(ratio, supply);
+	buf_receive = resource_cards(1, receive);
 	log_message(MSG_TRADE, _("%s exchanged %s for %s.\n"),
 		    player_name(player_num, TRUE), buf_give, buf_receive);
+	g_free(buf_give);
+	g_free(buf_receive);
 }
 
 void player_build_add(gint player_num,
@@ -492,7 +528,7 @@ void player_build_add(gint player_num,
 	case BUILD_MOVE_SHIP:
 		/* This clause here to remove a compiler warning.
 		   It should not be possible to reach this code. */
-		g_error("Bug: unreachable code reached.\n");
+		g_error("Bug: unreachable code reached");
 		break;
 	}
 }
@@ -568,7 +604,7 @@ void player_build_remove(gint player_num,
 	case BUILD_MOVE_SHIP:
 		/* This clause here to remove a compiler warning.
 		   It should not be possible to reach this case. */
-		g_error("Bug: unreachable code reached.\n");
+		g_error("Bug: unreachable code reached");
 		break;
 	}
 }
@@ -599,7 +635,7 @@ void player_build_move(gint player_num, gint sx, gint sy, gint spos,
 }
 
 void player_resource_action(gint player_num, const gchar * action,
-			    gint * resource_list, gint mult)
+			    const gint * resource_list, gint mult)
 {
 	resource_log_list(player_num, action, resource_list);
 	resource_apply_list(player_num, resource_list, mult);
@@ -610,13 +646,9 @@ void player_get_point(gint player_num, gint id, const gchar * str,
 		      gint num)
 {
 	Player *player = player_get(player_num);
-	/* create the memory and fill it */
-	Points *point = g_malloc0(sizeof(*point));
-	point->id = id;
-	point->name = g_strdup(str);
-	point->points = num;
-	/* put the point in the list */
-	player->points = g_list_append(player->points, point);
+	/* create the point */
+	Points *point = points_new(id, str, num);
+	player_modify_points(player_num, point, TRUE);
 	/* tell the user that someone got something */
 	log_message(MSG_INFO, _("%s received %s.\n"), player->name, str);
 }
@@ -640,11 +672,12 @@ void player_lose_point(gint player_num, gint id)
 		return;
 	}
 	player->points = g_list_remove(player->points, point);
+	player_modify_points(player_num, point, FALSE);
 	/* tell the user the point is lost */
 	log_message(MSG_INFO, _("%s lost %s.\n"), player->name,
 		    point->name);
 	/* free the memory */
-	g_free(point->name);
+	points_free(point);
 	g_free(point);
 }
 
@@ -670,12 +703,41 @@ void player_take_point(gint player_num, gint id, gint old_owner)
 	/* move the point in memory */
 	victim->points = g_list_remove(victim->points, point);
 	player->points = g_list_append(player->points, point);
-	/* tell the user someone lost something to someone else */
+	/* tell the user someone (1) lost something (2) to someone else (3) */
 	log_message(MSG_INFO, _("%s lost %s to %s.\n"), victim->name,
 		    point->name, player->name);
 }
 
-gint current_player()
+gint current_player(void)
 {
 	return turn_player;
 }
+
+const gchar *my_player_name(void)
+{
+	return player_name(my_player_num(), TRUE);
+}
+
+gboolean my_player_viewer(void)
+{
+	return player_is_viewer(my_player_num());
+}
+
+gint find_player_by_name(const gchar * name)
+{
+	gint i;
+	GList *list;
+
+	for (i = 0; i < num_total_players; i++) {
+		Player *player = player_get(i);
+		if (player->name && !strcmp(player->name, name))
+			return i;
+	}
+
+	for (list = viewers; list != NULL; list = g_list_next(list)) {
+		Viewer *viewer = list->data;
+		if (!strcmp(viewer->name, name))
+			return viewer->num;
+	}
+	return -1;
+}
