@@ -43,6 +43,7 @@ static void frontend_network_wait(gboolean is_waiting)
 static void frontend_init_game(void)
 {
 	player_clear_summary();
+	chat_clear_names();
 	develop_reset();
 	histogram_reset();
 	gui_reset();
@@ -53,6 +54,7 @@ static void frontend_start_game(void)
 	gui_set_game_params(get_game_params());
 	set_num_players(num_players());
 	identity_reset();
+	gui_set_show_no_setup_nodes(!player_is_viewer(my_player_num()));
 	frontend_gui_update();
 }
 
@@ -82,6 +84,7 @@ static void frontend_update_stock(void)
 
 static void frontend_player_turn(gint player)
 {
+	gui_set_show_no_setup_nodes(FALSE);
 	player_show_current(player);
 }
 
@@ -98,8 +101,8 @@ static void frontend_robber_moved(G_GNUC_UNUSED Hex * old,
 
 static void frontend_new_bank(G_GNUC_UNUSED const gint * new_bank)
 {
-#ifdef DEBUG
-	debug("New bank: %d %d %d %d %d\n", new_bank[0], new_bank[1],
+#ifdef DEBUG_BANK
+	debug("New bank: %d %d %d %d %d", new_bank[0], new_bank[1],
 	      new_bank[2], new_bank[3], new_bank[4]);
 #endif
 }
@@ -145,7 +148,6 @@ void frontend_set_callbacks(void)
 	callbacks.rolled_dice = &frontend_rolled_dice;
 	callbacks.gold_choose = &frontend_gold_choose;
 	callbacks.gold_done = &frontend_gold_done;
-	callbacks.beep = &frontend_beep;
 	callbacks.draw_edge = &frontend_draw_edge;
 	callbacks.draw_node = &frontend_draw_node;
 	callbacks.bought_develop = &frontend_bought_develop;
@@ -156,9 +158,11 @@ void frontend_set_callbacks(void)
 	callbacks.robber = &frontend_robber;
 	callbacks.robber_moved = &frontend_robber_moved;
 	callbacks.new_statistics = &frontend_new_statistics;
+	callbacks.new_points = &frontend_new_points;
 	callbacks.viewer_name = &frontend_viewer_name;
 	callbacks.player_name = &frontend_player_name;
 	callbacks.player_quit = &frontend_player_quit;
 	callbacks.viewer_quit = &frontend_viewer_quit;
+	callbacks.incoming_chat = &chat_parser;
 	callbacks.new_bank = &frontend_new_bank;
 }
