@@ -2,7 +2,7 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003,2006 Bas Wijnen <shevek@fmf.nl>
  * Copyright (C) 2004 Roland Clobus <rclobus@bigfoot.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -36,11 +36,14 @@
 const int PRIVATE_GAME_HISTORY_SIZE = 10;
 
 static gchar *connect_name;	/* Name of the player */
+static gboolean connect_viewer;	/* Prefer to be a viewer */
+static gboolean connect_viewer_allowed;	/* Viewer allowed */
 static gchar *connect_server;	/* Name of the server */
 static gchar *connect_port;	/* Port of the server */
 
 static GtkWidget *connect_dlg;	/* Dialog for starting a new game */
 static GtkWidget *name_entry;	/* Name of the player */
+static GtkWidget *viewer_toggle;	/* Prefer to be a viewer */
 static GtkWidget *meta_server_entry;	/* Name of the metaserver */
 
 
@@ -131,8 +134,8 @@ static gchar server_version[STRARG_LEN];
 static gchar server_max[INTARG_LEN];
 static gchar server_curr[INTARG_LEN];
 static gchar server_vpoints[STRARG_LEN];
-static gchar server_sevenrule[STRARG_LEN];
-static gchar server_terrain[STRARG_LEN];
+static gchar *server_sevenrule = NULL;
+static gchar *server_terrain = NULL;
 static gchar server_title[STRARG_LEN];
 
 static void query_meta_server(const gchar * server, const gchar * port);
@@ -141,22 +144,59 @@ static void show_waiting_box(const gchar * message, const gchar * server,
 static void close_waiting_box(void);
 
 static void connect_set_field(gchar ** field, const gchar * value);
-static void connect_close_all(gboolean user_pressed_ok);
+static void connect_close_all(gboolean user_pressed_ok,
+			      gboolean can_be_viewer);
 static void set_meta_serverinfo(void);
 static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 				   GtkWindow * parent);
 
+static void connect_set_field(gchar ** field, const gchar * value)
+{
+	gchar *temp = g_strdup(value);
+	if (*field)
+		g_free(*field);
+	*field = g_strdup(g_strstrip(temp));
+	g_free(temp);
+}
+
 /* Public functions */
 const gchar *connect_get_name(void)
 {
 	return connect_name;
 }
 
+void connect_set_name(const gchar * name)
+{
+	connect_set_field(&connect_name, name);
+	if (name_entry != NULL)
+		gtk_entry_set_text(GTK_ENTRY(name_entry), connect_name);
+}
+
+gboolean connect_get_viewer(void)
+{
+	return connect_viewer && connect_viewer_allowed;
+}
+
+void connect_set_viewer(gboolean viewer)
+{
+	connect_viewer = viewer;
+	connect_viewer_allowed = TRUE;
+	if (viewer_toggle != NULL)
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+					     (viewer_toggle),
+					     connect_viewer);
+}
+
 const gchar *connect_get_server(void)
 {
 	return connect_server;
 }
 
+void connect_set_server(const gchar * server)
+{
+	connect_set_field(&connect_server, server);
+}
+
 static const gchar *connect_get_meta_server(void)
 {
 	const gchar *text;
@@ -169,31 +209,41 @@ static const gchar *connect_get_meta_server(void)
 	return text;
 }
 
+void connect_set_meta_server(const gchar * meta_server)
+{
+	connect_set_field(&metaserver_info.server, meta_server);
+	if (meta_server_entry != NULL)
+		gtk_entry_set_text(GTK_ENTRY(meta_server_entry),
+				   metaserver_info.server);
+}
+
 const gchar *connect_get_port(void)
 {
 	return connect_port;
 }
 
-static void connect_set_field(gchar ** field, const gchar * value)
+void connect_set_port(const gchar * port)
 {
-	gchar *temp = g_strdup(value);
-	if (*field)
-		g_free(*field);
-	*field = g_strdup(g_strstrip(temp));
-	g_free(temp);
+	connect_set_field(&connect_port, port);
 }
 
-static void connect_close_all(gboolean user_pressed_ok)
+static void connect_close_all(gboolean user_pressed_ok,
+			      gboolean can_be_viewer)
 {
+	connect_viewer_allowed = can_be_viewer;
 	if (user_pressed_ok) {
+		connect_set_field(&connect_name,
+				  gtk_entry_get_text(GTK_ENTRY
+						     (name_entry)));
+		connect_viewer =
+		    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
+						 (viewer_toggle));
 		/* Save connect dialogue entries */
 		config_set_string("connect/meta-server",
 				  connect_get_meta_server());
 		config_set_string("connect/name", connect_name);
+		config_set_int("connect/viewer", connect_viewer);
 
-		connect_set_field(&connect_name,
-				  gtk_entry_get_text(GTK_ENTRY
-						     (name_entry)));
 		frontend_gui_register_destroy(connect_dlg,
 					      GUI_CONNECT_TRY);
 	} else {
@@ -320,7 +370,7 @@ static void meta_create_notify(NetEvent event,
 				/* The meta server is now busy creating the new game.
 				 * UGLY FIX: Wait for some time */
 				g_usleep(500000);
-				connect_close_all(TRUE);
+				connect_close_all(TRUE, FALSE);
 			} else
 				log_message(MSG_ERROR,
 					    _
@@ -377,6 +427,8 @@ static void server_end(void)
 static void meta_notify(NetEvent event, G_GNUC_UNUSED void *user_data,
 			char *line)
 {
+	gchar argument[STRARG_LEN];
+
 	switch (event) {
 	case NET_CONNECT:
 		break;
@@ -522,13 +574,49 @@ static void meta_notify(NetEvent event, G_GNUC_UNUSED void *user_data,
 				 (line, "vpoints=", server_vpoints))
 				break;
 			else if (check_str_info
-				 (line, "sevenrule=", server_sevenrule))
+				 (line, "sevenrule=", argument)) {
+				if (server_sevenrule)
+					g_free(server_sevenrule);
+				if (!strcmp(argument, "normal")) {
+					server_sevenrule =
+					    g_strdup(_("Normal"));
+				} else
+				    if (!strcmp
+					(argument, "reroll first 2")) {
+					server_sevenrule =
+					    g_strdup(_
+						     ("Reroll on 1st 2 turns"));
+				} else if (!strcmp(argument, "reroll all")) {
+					server_sevenrule =
+					    g_strdup(_("Reroll all 7s"));
+				} else {
+					g_warning("Unknown seven rule: %s",
+						  argument);
+					server_sevenrule =
+					    g_strdup(argument);
+				}
 				break;
-			else if (check_str_info
-				 (line, "terrain=", server_terrain))
+			} else if (check_str_info
+				   (line, "terrain=", argument)) {
+				if (server_terrain)
+					g_free(server_terrain);
+				if (!strcmp(argument, "default"))
+					server_terrain =
+					    g_strdup(_("Default"));
+				else if (!strcmp(argument, "random"))
+					server_terrain =
+					    g_strdup(_("Random"));
+				else {
+					g_warning
+					    ("Unknown terrain type: %s",
+					     argument);
+					server_terrain =
+					    g_strdup(argument);
+				}
 				break;
-			else if (check_str_info
-				 (line, "title=", server_title))
+			} else
+			    if (check_str_info
+				(line, "title=", server_title))
 				break;
 			/* meta-protocol 0 compat */
 			else if (check_str_info
@@ -662,11 +750,11 @@ static void create_server_dlg_cb(GtkDialog * dlg, gint arg1,
 	case GTK_RESPONSE_OK:
 		log_message(MSG_INFO, _("Requesting new game server\n"));
 
-		cfg_terrain = game_settings_get_terrain(gs),
-		    cfg_num_players = game_settings_get_players(gs),
-		    cfg_victory_points =
-		    game_settings_get_victory_points(gs), cfg_sevens_rule =
-		    game_settings_get_sevens_rule(gs), cfg_ai_players =
+		cfg_terrain = game_settings_get_terrain(gs);
+		cfg_num_players = game_settings_get_players(gs);
+		cfg_victory_points = game_settings_get_victory_points(gs);
+		cfg_sevens_rule = game_settings_get_sevens_rule(gs);
+		cfg_ai_players =
 		    gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON
 						     (aiplayers_spin));
 		cfg_gametype = select_game_get_active(sg);
@@ -800,7 +888,7 @@ static void meta_dlg_cb(GtkDialog * dlg, gint arg1,
 			g_free(host);
 			g_free(port);
 			gtk_tree_path_free(path);
-			connect_close_all(TRUE);
+			connect_close_all(TRUE, TRUE);
 		}
 		break;
 	case GTK_RESPONSE_CANCEL:	/* Cancel */
@@ -816,7 +904,8 @@ static void set_meta_serverinfo(void)
 
 	meta_tmp =
 	    g_strdup(gtk_entry_get_text(GTK_ENTRY(meta_server_entry)));
-	if (!meta_tmp || !strlen(meta_tmp)) {
+	if (meta_tmp[0] == '\0') {
+		g_free(meta_tmp);
 		meta_tmp = get_meta_server_name(TRUE);
 		gtk_entry_set_text(GTK_ENTRY(meta_server_entry), meta_tmp);
 	}
@@ -1052,7 +1141,7 @@ static void connect_dlg_cb(G_GNUC_UNUSED GtkDialog * dlg,
 			   G_GNUC_UNUSED gint arg1,
 			   G_GNUC_UNUSED gpointer userdata)
 {
-	connect_close_all(FALSE);
+	connect_close_all(FALSE, FALSE);
 }
 
 void connect_create_dlg(void)
@@ -1065,9 +1154,6 @@ void connect_create_dlg(void)
 	GtkWidget *sep;
 	GtkTooltips *tooltips;
 	gchar *fullname;
-	gchar *saved_meta_server;
-	gchar *saved_name;
-	gboolean default_returned;
 
 	if (connect_dlg) {
 		gtk_window_present(GTK_WINDOW(connect_dlg));
@@ -1076,21 +1162,6 @@ void connect_create_dlg(void)
 
 	tooltips = gtk_tooltips_new();
 
-	/* initialize meta server value */
-	default_returned = FALSE;
-	saved_meta_server =
-	    config_get_string("connect/meta-server", &default_returned);
-	if (default_returned
-	    || !strncmp(saved_meta_server, "gnocatan.debian.net",
-			strlen(saved_meta_server) + 1)) {
-		g_free(saved_meta_server);
-		saved_meta_server = get_meta_server_name(TRUE);
-	}
-
-	/* initialize name value */
-	default_returned = FALSE;
-	saved_name = g_strdup(my_player_name());
-
 	connect_dlg = gtk_dialog_new_with_buttons(_("Start a new game"),
 						  GTK_WINDOW(app_window),
 						  0,
@@ -1109,7 +1180,7 @@ void connect_create_dlg(void)
 	dlg_vbox = GTK_DIALOG(connect_dlg)->vbox;
 	gtk_widget_show(dlg_vbox);
 
-	table = gtk_table_new(4, 2, FALSE);
+	table = gtk_table_new(4, 3, FALSE);
 	gtk_widget_show(table);
 	gtk_box_pack_start(GTK_BOX(dlg_vbox), table, FALSE, TRUE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(table), 5);
@@ -1125,17 +1196,26 @@ void connect_create_dlg(void)
 	name_entry = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(name_entry), MAX_NAME_LENGTH);
 	gtk_widget_show(name_entry);
-	gtk_entry_set_text(GTK_ENTRY(name_entry), saved_name);
-	connect_set_field(&connect_name, saved_name);
+	gtk_entry_set_text(GTK_ENTRY(name_entry), connect_name);
 
 	gtk_table_attach(GTK_TABLE(table), name_entry, 1, 2, 0, 1,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_tooltips_set_tip(tooltips, name_entry,
 			     _("Enter your name"), NULL);
 
+	viewer_toggle = gtk_check_button_new_with_label(_("Viewer"));
+	gtk_widget_show(viewer_toggle);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(viewer_toggle),
+				     connect_viewer);
+
+	gtk_table_attach(GTK_TABLE(table), viewer_toggle, 2, 3, 0, 1, 0,
+			 GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_tooltips_set_tip(tooltips, viewer_toggle,
+			     _("Do you want to be a viewer?"), NULL);
+
 	sep = gtk_hseparator_new();
 	gtk_widget_show(sep);
-	gtk_table_attach(GTK_TABLE(table), sep, 0, 2, 1, 2,
+	gtk_table_attach(GTK_TABLE(table), sep, 0, 3, 1, 2,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 6);
 
 	lbl = gtk_label_new(_("Meta Server"));
@@ -1147,18 +1227,18 @@ void connect_create_dlg(void)
 
 	meta_server_entry = gtk_entry_new();
 	gtk_widget_show(meta_server_entry);
-	gtk_table_attach(GTK_TABLE(table), meta_server_entry, 1, 2, 2, 3,
+	gtk_table_attach(GTK_TABLE(table), meta_server_entry, 1, 3, 2, 3,
 			 (GtkAttachOptions) GTK_EXPAND | GTK_FILL,
 			 (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_entry_set_text(GTK_ENTRY(meta_server_entry),
-			   saved_meta_server);
+			   metaserver_info.server);
 	gtk_tooltips_set_tip(tooltips, meta_server_entry,
 			     _("Leave empty for the default meta server"),
 			     NULL);
 
 	hbox = gtk_hbox_new(FALSE, 3);
 	gtk_widget_show(hbox);
-	gtk_table_attach(GTK_TABLE(table), hbox, 0, 2, 3, 4,
+	gtk_table_attach(GTK_TABLE(table), hbox, 0, 3, 3, 4,
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 3);
 
 	btn = gtk_button_new_with_label(_("Join public game"));
@@ -1196,19 +1276,16 @@ void connect_create_dlg(void)
 	gtk_widget_show(connect_dlg);
 
 	gtk_widget_grab_focus(name_entry);
-
-	g_free(saved_meta_server);
-	g_free(saved_name);
 }
 
 /* ------------ Join a private game dialog ------------------- */
 
 static void update_recent_servers_list(void)
 {
-	gchar keyname1[150], keyname2[150];
-	gchar temp_name[150] = "", temp_port[150] = "";
-	gchar cur_name[150], cur_port[150];
-	gchar conn_name[150], conn_port[150];
+	gchar keyname1[50], keyname2[50];
+	gchar *temp_name, *temp_port;
+	gchar *cur_name, *cur_port;
+	gchar *conn_name, *conn_port;
 	gboolean default_used;
 	gboolean done;
 	gint i;
@@ -1216,21 +1293,21 @@ static void update_recent_servers_list(void)
 	done = FALSE;
 	i = 0;
 
-	strcpy(conn_name, connect_get_server());
-	strcpy(conn_port, connect_get_port());
+	conn_name = g_strdup(connect_get_server());
+	conn_port = g_strdup(connect_get_port());
 
-	strcpy(temp_name, conn_name);
-	strcpy(temp_port, conn_port);
+	temp_name = g_strdup(conn_name);
+	temp_port = g_strdup(conn_port);
 
 	do {
 		sprintf(keyname1, "favorites/server%dname=", i);
 		sprintf(keyname2, "favorites/server%dport=", i);
-		strcpy(cur_name,
-		       config_get_string(keyname1, &default_used));
-		strcpy(cur_port,
-		       config_get_string(keyname2, &default_used));
+		cur_name =
+		    g_strdup(config_get_string(keyname1, &default_used));
+		cur_port =
+		    g_strdup(config_get_string(keyname2, &default_used));
 
-		if (strlen(temp_name)) {
+		if (temp_name) {
 			sprintf(keyname1, "favorites/server%dname", i);
 			sprintf(keyname2, "favorites/server%dport", i);
 			config_set_string(keyname1, temp_name);
@@ -1245,18 +1322,28 @@ static void update_recent_servers_list(void)
 
 		if (!strcmp(cur_name, conn_name)
 		    && !strcmp(cur_port, conn_port)) {
-			strcpy(temp_name, "");
-			strcpy(temp_port, "");
+			temp_name = NULL;
+			temp_port = NULL;
 		} else {
-			strcpy(temp_name, cur_name);
-			strcpy(temp_port, cur_port);
+			g_free(temp_name);
+			g_free(temp_port);
+			temp_name = g_strdup(cur_name);
+			temp_port = g_strdup(cur_port);
 		}
 
 		i++;
 		if (i > PRIVATE_GAME_HISTORY_SIZE) {
 			done = TRUE;
 		}
+		g_free(cur_name);
+		g_free(cur_port);
 	} while (!done);
+	g_free(cur_name);
+	g_free(cur_port);
+	g_free(temp_name);
+	g_free(temp_port);
+	g_free(conn_name);
+	g_free(conn_port);
 }
 
 static void host_list_select_cb(GtkWidget * widget, gpointer user_data)
@@ -1292,7 +1379,7 @@ static void connect_private_dlg_cb(GtkDialog * dlg, gint arg1,
 		update_recent_servers_list();
 		config_set_string("connect/server", connect_server);
 		config_set_string("connect/port", connect_port);
-		connect_close_all(TRUE);
+		connect_close_all(TRUE, TRUE);
 		break;
 	case GTK_RESPONSE_CANCEL:
 	default:		/* For the compiler */
@@ -1313,10 +1400,8 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 	GtkWidget *host_list;
 	GPtrArray *host_entries;
 
-	gchar *saved_server;
-	gchar *saved_port;
 	gint i;
-	gchar *host_name, *host_port, *host_name_port, temp_str[150];
+	gchar *host_name, *host_port, *host_name_port, temp_str[50];
 	gboolean default_returned;
 
 	if (connect_private_dlg) {
@@ -1326,16 +1411,6 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 
 	tooltips = gtk_tooltips_new();
 
-	/* initialize server value */
-	saved_server =
-	    config_get_string("connect/server=" PIONEERS_DEFAULT_GAME_HOST,
-			      &default_returned);
-
-	/* initialize port value */
-	saved_port =
-	    config_get_string("connect/port=" PIONEERS_DEFAULT_GAME_PORT,
-			      &default_returned);
-
 	connect_private_dlg =
 	    gtk_dialog_new_with_buttons(_("Join a private game"),
 					GTK_WINDOW(parent), 0,
@@ -1376,10 +1451,10 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 	gtk_table_attach(GTK_TABLE(table), host_entry, 1, 2, 0, 1,
 			 GTK_EXPAND | GTK_FILL,
 			 GTK_EXPAND | GTK_FILL, 0, 0);
-	gtk_entry_set_text(GTK_ENTRY(host_entry), saved_server);
+	gtk_entry_set_text(GTK_ENTRY(host_entry), connect_server);
 	gtk_tooltips_set_tip(tooltips, host_entry,
 			     _("Name of the host of the game"), NULL);
-	connect_set_field(&connect_server, saved_server);
+	connect_set_field(&connect_server, connect_server);
 
 	lbl = gtk_label_new(_("Server Port"));
 	gtk_widget_show(lbl);
@@ -1395,10 +1470,10 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 	port_entry = gtk_entry_new();
 	gtk_widget_show(port_entry);
 	gtk_box_pack_start(GTK_BOX(hbox), port_entry, FALSE, TRUE, 0);
-	gtk_entry_set_text(GTK_ENTRY(port_entry), saved_port);
+	gtk_entry_set_text(GTK_ENTRY(port_entry), connect_port);
 	gtk_tooltips_set_tip(tooltips, port_entry,
 			     _("Port of the host of the game"), NULL);
-	connect_set_field(&connect_port, saved_port);
+	connect_set_field(&connect_port, connect_port);
 
 	host_list = gtk_combo_box_new_text();
 	host_entries = g_ptr_array_new();
@@ -1449,7 +1524,4 @@ static void connect_private_dialog(G_GNUC_UNUSED GtkWidget * widget,
 	gtk_entry_set_activates_default(GTK_ENTRY(port_entry), TRUE);
 
 	gtk_widget_show(connect_private_dlg);
-
-	g_free(saved_server);
-	g_free(saved_port);
 }
