@@ -1,7 +1,7 @@
 /* Pioneers - Implementation of the excellent Settlers of Catan board game.
  *   Go buy a copy.
  *
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003,2006 Bas Wijnen <shevek@fmf.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -102,14 +102,16 @@ void frontend_gui_route_event(GuiEvent event);
 void frontend_init(int argc, char **argv);
 void frontend_new_statistics(gint player_num, StatisticType type,
 			     gint num);
+void frontend_new_points(gint player_num, Points * points, gboolean added);
 void frontend_viewer_name(gint viewer_num, const gchar * name);
 void frontend_player_name(gint player_num, const gchar * name);
 void frontend_player_quit(gint player_num);
 void frontend_viewer_quit(gint player_num);
+void frontend_disconnect(void);
 void frontend_offline(void);
 void frontend_discard(void);
 void frontend_discard_add(gint player_num, gint discard_num);
-void frontend_discard_remove(gint player_num, gint * list);
+void frontend_discard_remove(gint player_num);
 void frontend_discard_done(void);
 void frontend_gold(void);
 void frontend_gold_add(gint player_num, gint gold_num);
@@ -125,17 +127,18 @@ void frontend_plenty(const gint * bank);
 void frontend_turn(void);
 void frontend_trade_player_end(gint player_num);
 void frontend_trade_add_quote(gint player_num, gint quote_num,
-			      gint * they_supply, gint * they_receive);
+			      const gint * they_supply,
+			      const gint * they_receive);
 void frontend_trade_remove_quote(int player_num, int quote_num);
 void frontend_quote_player_end(gint player_num);
 void frontend_quote_add(gint player_num, gint quote_num,
-			gint * they_supply, gint * they_receive);
+			const gint * they_supply,
+			const gint * they_receive);
 void frontend_quote_remove(gint player_num, gint quote_num);
 void frontend_quote_start(void);
 void frontend_quote_end(void);
 void frontend_quote_monitor(void);
 void frontend_rolled_dice(gint die1, gint die2, gint player_num);
-void frontend_beep(void);
 void frontend_bought_develop(DevelType type);
 void frontend_played_develop(gint player_num, gint card_idx,
 			     DevelType type);
@@ -147,29 +150,36 @@ void frontend_game_over(gint player, gint points);
 const gchar *connect_get_server(void);
 const gchar *connect_get_port(void);
 const gchar *connect_get_name(void);
+gboolean connect_get_viewer(void);
+void connect_set_server(const gchar * server);
+void connect_set_port(const gchar * port);
+void connect_set_name(const gchar * name);
+void connect_set_viewer(gboolean viewer);
+void connect_set_meta_server(const gchar * meta_server);
 void connect_create_dlg(void);
 
 /* trade.c */
 GtkWidget *trade_build_page(void);
 gboolean can_call_for_quotes(void);
 gboolean trade_valid_selection(void);
-gint *trade_we_supply(void);
-gint *trade_we_receive(void);
-QuoteInfo *trade_current_quote(void);
+const gint *trade_we_supply(void);
+const gint *trade_we_receive(void);
+const QuoteInfo *trade_current_quote(void);
 void trade_finish(void);
-void trade_add_quote(int player_num, int quote_num, gint * they_supply,
-		     gint * they_receive);
+void trade_add_quote(int player_num, int quote_num,
+		     const gint * they_supply, const gint * they_receive);
 void trade_delete_quote(int player_num, int quote_num);
 void trade_player_finish(gint player_num);
 void trade_begin(void);
-void trade_format_quote(QuoteInfo * quote, gchar * buffer);
+void trade_format_quote(const QuoteInfo * quote, gchar * buffer);
 void trade_new_trade(void);
 void trade_perform_maritime(gint ratio, Resource supply, Resource receive);
 void trade_perform_domestic(gint player_num, gint partner_num,
-			    gint quote_num, gint * they_supply,
-			    gint * they_receive);
+			    gint quote_num, const gint * they_supply,
+			    const gint * they_receive);
 void frontend_trade_domestic(gint partner_num, gint quote_num,
-			     gint * we_supply, gint * we_receive);
+			     const gint * we_supply,
+			     const gint * we_receive);
 void frontend_trade_maritime(gint ratio, Resource we_supply,
 			     Resource we_receive);
 
@@ -179,20 +189,21 @@ gboolean can_submit_quote(void);
 gboolean can_delete_quote(void);
 gboolean can_reject_quote(void);
 gint quote_next_num(void);
-gint *quote_we_supply(void);
-gint *quote_we_receive(void);
-QuoteInfo *quote_current_quote(void);
-void quote_begin_again(gint player_num, gint * they_supply,
-		       gint * they_receive);
-void quote_begin(gint player_num, gint * they_supply, gint * they_receive);
+const gint *quote_we_supply(void);
+const gint *quote_we_receive(void);
+const QuoteInfo *quote_current_quote(void);
+void quote_begin_again(gint player_num, const gint * they_supply,
+		       const gint * they_receive);
+void quote_begin(gint player_num, const gint * they_supply,
+		 const gint * they_receive);
 void quote_add_quote(gint player_num, gint quote_num,
-		     gint * they_supply, gint * they_receive);
+		     const gint * they_supply, const gint * they_receive);
 void quote_delete_quote(gint player_num, gint quote_num);
 void quote_player_finish(gint player_num);
 void quote_finish(void);
 void frontend_quote_trade(gint player_num, gint partner_num,
-			  gint quote_num, gint * they_supply,
-			  gint * they_receive);
+			  gint quote_num, const gint * they_supply,
+			  const gint * they_receive);
 
 /* legend.c */
 GtkWidget *legend_create_dlg(void);
@@ -206,16 +217,16 @@ void develop_reset(void);
 /* discard.c */
 GtkWidget *discard_build_page(void);
 gboolean can_discard(void);
-gint *discard_get_list(void);
+void discard_get_list(gint * discards);
 void discard_begin(void);
 void discard_player_must(gint player_num, gint discard_num);
-void discard_player_did(gint player_num, gint * resources);
+void discard_player_did(gint player_num);
 void discard_end(void);
 
 /* gold.c */
 GtkWidget *gold_build_page(void);
 gboolean can_choose_gold(void);
-gint *choose_gold_get_list(gint * choice);
+void choose_gold_get_list(gint * choice);
 void gold_choose_begin(void);
 void gold_choose_player_prepare(gint player_num, gint gold_num);
 void gold_choose_player_must(gint gold_num, const gint * bank);
@@ -230,6 +241,13 @@ void identity_reset(void);
 
 /* resource.c */
 GtkWidget *resource_build_panel(void);
+/** Draw the resources on the image.
+ *  @param image     The image to draw the resources on
+ *  @param resources The resources
+ *  @param max_width If > 0, use this for the maximum width
+ */
+void resource_format_type_image(GtkImage * image, const gint * resources,
+				gint max_width);
 
 /* player.c */
 GtkWidget *player_build_summary(void);
@@ -256,6 +274,16 @@ GtkWidget *chat_build_panel(void);
 void chat_set_grab_focus_on_update(gboolean grab);
 /** Set the focus to the chat widget */
 void chat_set_focus(void);
+/** A player/viewer has changed his name */
+void chat_player_name(gint player_num, const gchar * name);
+/** A player has quit */
+void chat_player_quit(gint player_num);
+/** A viewer has quit */
+void chat_viewer_quit(gint viewer_num);
+/** Clear all names */
+void chat_clear_names(void);
+/** Parse the chat for commands */
+void chat_parser(gint player_num, const gchar * chat_str);
 
 /* name.c */
 void name_create_dlg(void);
@@ -273,6 +301,7 @@ void monopoly_create_dlg(void);
 void plenty_resources(gint * plenty);
 void plenty_destroy_dlg(void);
 void plenty_create_dlg(const gint * bank);
+gboolean plenty_can_activate(void);
 
 /* gameover.c */
 GtkWidget *gameover_create_dlg(gint player_num, gint num_points);
@@ -288,4 +317,10 @@ GtkWidget *gameover_create_dlg(gint player_num, gint num_points);
 #define PIONEERS_PIXMAP_DEVELOP "pioneers/develop.png"
 #define PIONEERS_PIXMAP_FINISH "pioneers/finish.png"
 
+#define PIONEERS_PIXMAP_BRICK "pioneers/brick.png"
+#define PIONEERS_PIXMAP_GRAIN "pioneers/grain.png"
+#define PIONEERS_PIXMAP_LUMBER "pioneers/lumber.png"
+#define PIONEERS_PIXMAP_ORE "pioneers/ore.png"
+#define PIONEERS_PIXMAP_WOOL "pioneers/wool.png"
+
 #endif
