@@ -41,8 +41,8 @@
 static GtkWidget *preferences_dlg;
 GtkWidget *app_window;		/* main application window */
 
-#define MAP_WIDTH 550		/* default map width */
-#define MAP_HEIGHT 400		/* default map height */
+#define MAP_WIDTH 350		/* default map width */
+#define MAP_HEIGHT 200		/* default map height */
 
 #define PIONEERS_ICON_FILE	"pioneers.png"
 
@@ -75,6 +75,8 @@ static GtkUIManager *ui_manager = NULL;	/* The manager of the GtkActions */
 static GtkWidget *toolbar = NULL;	/* The toolbar */
 
 static gboolean toolbar_show_accelerators = TRUE;
+static gboolean color_messages_enabled = TRUE;
+static gboolean legend_page_enabled = TRUE;
 
 static GList *rules_callback_list = NULL;
 
@@ -93,6 +95,21 @@ static const gchar *pioneers_pixmaps[] = {
 	PIONEERS_PIXMAP_FINISH
 };
 
+static const gchar *resources_pixmaps[] = {
+	PIONEERS_PIXMAP_BRICK,
+	PIONEERS_PIXMAP_GRAIN,
+	PIONEERS_PIXMAP_ORE,
+	PIONEERS_PIXMAP_WOOL,
+	PIONEERS_PIXMAP_LUMBER
+};
+
+static struct {
+	GdkPixmap *p;
+	GdkBitmap *b;
+	GdkGC *gcp, *gcb;
+} resource_pixmap[NO_RESOURCE];
+static gint resource_pixmap_res = 0;
+
 static void gui_set_toolbar_visible(void);
 static void gui_toolbar_show_accelerators(gboolean show_accelerators);
 
@@ -175,9 +192,9 @@ static void showhide_toolbar_cb(void);
 static void preferences_cb(void);
 
 static void help_about_cb(void);
-static void help_legend_cb(void);
-static void help_histogram_cb(void);
-static void help_settings_cb(void);
+static void game_legend_cb(void);
+static void game_histogram_cb(void);
+static void game_settings_cb(void);
 #ifdef HAVE_HELP
 static void help_manual_cb(void);
 #endif
@@ -195,6 +212,12 @@ static GtkActionEntry entries[] = {
 #endif
 	{"PlayerName", NULL, N_("_Player name"), "<control>P",
 	 N_("Change your player name"), playername_cb},
+	{"Legend", GTK_STOCK_DIALOG_INFO, N_("_Legend"), NULL,
+	 N_("Terrain legend and building costs"), game_legend_cb},
+	{"GameSettings", GTK_STOCK_DIALOG_INFO, N_("_Game Settings"), NULL,
+	 N_("Settings for the current game"), game_settings_cb},
+	{"DiceHistogram", GTK_STOCK_DIALOG_INFO, N_("_Dice Histogram"),
+	 NULL, N_("Histogram of dice rolls"), game_histogram_cb},
 	{"GameQuit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
 	 N_("Quit the program"), game_quit_cb},
 	{"ActionsMenu", NULL, N_("_Actions"), NULL, NULL, NULL},
@@ -225,12 +248,6 @@ static GtkActionEntry entries[] = {
 	 N_("Configure the application"), preferences_cb},
 
 	{"HelpMenu", NULL, N_("_Help"), NULL, NULL, NULL},
-	{"Legend", GTK_STOCK_DIALOG_INFO, N_("_Legend"), NULL,
-	 N_("Terrain legend and building costs"), help_legend_cb},
-	{"GameSettings", GTK_STOCK_DIALOG_INFO, N_("_Game Settings"), NULL,
-	 N_("Settings for the current game"), help_settings_cb},
-	{"DiceHistogram", GTK_STOCK_DIALOG_INFO, N_("_Dice Histogram"),
-	 NULL, N_("Histogram of dice rolls"), help_histogram_cb},
 	{"HelpAbout", NULL, N_("_About Pioneers"), NULL,
 	 N_("Information about Pioneers"), help_about_cb},
 #ifdef HAVE_HELP
@@ -258,6 +275,10 @@ static const char *ui_description =
 "      <separator/>"
 "      <menuitem action='PlayerName'/>"
 "      <separator/>"
+"      <menuitem action='Legend'/>"
+"      <menuitem action='GameSettings'/>"
+"      <menuitem action='DiceHistogram'/>"
+"      <separator/>"
 "      <menuitem action='GameQuit'/>"
 "    </menu>"
 "    <menu action='ActionsMenu'>"
@@ -279,10 +300,6 @@ static const char *ui_description =
 "      <menuitem action='Preferences'/>"
 "    </menu>"
 "    <menu action='HelpMenu'>"
-"      <menuitem action='Legend'/>"
-"      <menuitem action='GameSettings'/>"
-"      <menuitem action='DiceHistogram'/>"
-"      <separator/>"
 "      <menuitem action='HelpAbout'/>"
 #ifdef HAVE_HELP
 "      <menuitem action='HelpManual'/>"
@@ -332,13 +349,13 @@ void gui_set_instructions(const gchar * text)
 
 void gui_set_vp_target_value(gint vp)
 {
-	gchar vp_text[30];
+	gchar *vp_text;
 
 	/* Victory points target in statusbar */
-	g_snprintf(vp_text, sizeof(vp_text), _("Points Needed to Win: %i"),
-		   vp);
+	vp_text = g_strdup_printf(_("Points Needed to Win: %i"), vp);
 
 	gtk_label_set_text(GTK_LABEL(vp_target_status), vp_text);
+	g_free(vp_text);
 }
 
 void gui_set_net_status(const gchar * text)
@@ -346,20 +363,21 @@ void gui_set_net_status(const gchar * text)
 	gtk_label_set_text(GTK_LABEL(net_status), text);
 }
 
-void gui_cursor_none()
+void gui_cursor_none(void)
 {
 	MapElement dummyElement;
 	dummyElement.pointer = NULL;
-	guimap_cursor_set(gmap, NO_CURSOR, -1, NULL, NULL, &dummyElement,
-			  FALSE);
+	guimap_cursor_set(gmap, NO_CURSOR, -1, NULL, NULL, NULL,
+			  &dummyElement, FALSE);
 }
 
 void gui_cursor_set(CursorType type,
 		    CheckFunc check_func, SelectFunc select_func,
-		    const MapElement * user_data)
+		    CancelFunc cancel_func, const MapElement * user_data)
 {
 	guimap_cursor_set(gmap, type, my_player_num(),
-			  check_func, select_func, user_data, FALSE);
+			  check_func, select_func, cancel_func, user_data,
+			  FALSE);
 }
 
 void gui_draw_hex(const Hex * hex)
@@ -436,6 +454,7 @@ static GtkWidget *build_messages_panel(void)
 	messages_txt = gtk_text_view_new();
 	gtk_widget_show(messages_txt);
 	gtk_container_add(GTK_CONTAINER(scroll_win), messages_txt);
+	gtk_text_view_set_editable(GTK_TEXT_VIEW(messages_txt), FALSE);
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(messages_txt),
 				    GTK_WRAP_WORD);
 
@@ -452,8 +471,10 @@ void gui_show_trade_page(gboolean show)
 		gtk_widget_show(trade_page);
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(map_notebook),
 					      TRADE_PAGE);
-	} else
+	} else {
+		gtk_notebook_prev_page(GTK_NOTEBOOK(map_notebook));
 		gtk_widget_hide(trade_page);
+	}
 }
 
 void gui_show_quote_page(gboolean show)
@@ -464,8 +485,10 @@ void gui_show_quote_page(gboolean show)
 		gtk_widget_show(quote_page);
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(map_notebook),
 					      QUOTE_PAGE);
-	} else
+	} else {
+		gtk_notebook_prev_page(GTK_NOTEBOOK(map_notebook));
 		gtk_widget_hide(quote_page);
+	}
 }
 
 static void gui_theme_changed(void)
@@ -493,8 +516,11 @@ void gui_show_splash_page(gboolean show)
 		gtk_widget_show(splash_page);
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(map_notebook),
 					      SPLASH_PAGE);
-	} else
+	} else {
 		gtk_widget_hide(splash_page);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(map_notebook),
+					      MAP_PAGE);
+	}
 }
 
 static GtkWidget *splash_build_page(void)
@@ -571,22 +597,22 @@ static GtkWidget *build_map_panel(void)
 	return map_notebook;
 }
 
-void gui_discard_show()
+void gui_discard_show(void)
 {
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(develop_notebook), 1);
 }
 
-void gui_discard_hide()
+void gui_discard_hide(void)
 {
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(develop_notebook), 0);
 }
 
-void gui_gold_show()
+void gui_gold_show(void)
 {
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(develop_notebook), 2);
 }
 
-void gui_gold_hide()
+void gui_gold_hide(void)
 {
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(develop_notebook), 0);
 }
@@ -601,7 +627,7 @@ void gui_prompt_show(const gchar * message)
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(develop_notebook), 3);
 }
 
-void gui_prompt_hide()
+void gui_prompt_hide(void)
 {
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(develop_notebook), 0);
 }
@@ -729,6 +755,14 @@ static void summary_color_cb(GtkToggleButton * widget,
 	set_color_summary(color_summary);
 }
 
+static void announce_player_cb(GtkToggleButton * widget,
+			       G_GNUC_UNUSED gpointer user_data)
+{
+	gboolean announce_player = gtk_toggle_button_get_active(widget);
+	config_set_int("settings/announce_player", announce_player);
+	set_announce_player(announce_player);
+}
+
 static void showhide_toolbar_cb(void)
 {
 	gui_set_toolbar_visible();
@@ -748,7 +782,7 @@ static void preferences_cb(void)
 	GtkWidget *layout;
 	GtkTooltips *tooltips;
 
-	gint row;
+	guint row;
 	gint color_summary;
 	GList *theme_elt;
 	int i;
@@ -892,6 +926,23 @@ static void preferences_cb(void)
 			     _("Show keyboard shortcuts in the toolbar"),
 			     NULL);
 	row++;
+
+	/* Label for the option to announce when players/viewer enter */
+	widget =
+	    gtk_check_button_new_with_label(_("Announce new players"));
+	gtk_widget_show(widget);
+	gtk_table_attach_defaults(GTK_TABLE(layout), widget,
+				  0, 2, row, row + 1);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
+				     get_announce_player());
+	g_signal_connect(G_OBJECT(widget), "toggled",
+			 G_CALLBACK(announce_player_cb), NULL);
+	/* Tooltip for the option to use sound */
+	gtk_tooltips_set_tip(tooltips, widget,
+			     _
+			     ("Make a sound when a new player or viewer enters the game"),
+			     NULL);
+	row++;
 }
 
 static void help_about_cb(void)
@@ -902,17 +953,17 @@ static void help_about_cb(void)
 	aboutbox_display(_("The Pioneers Game"), authors);
 }
 
-static void help_legend_cb(void)
+static void game_legend_cb(void)
 {
 	legend_create_dlg();
 }
 
-static void help_histogram_cb(void)
+static void game_histogram_cb(void)
 {
 	histogram_create_dlg();
 }
 
-static void help_settings_cb(void)
+static void game_settings_cb(void)
 {
 	settings_create_dlg();
 }
@@ -920,7 +971,13 @@ static void help_settings_cb(void)
 #ifdef HAVE_HELP
 static void help_manual_cb(void)
 {
-	gnome_help_display("pioneers", NULL, NULL);
+	GError *error = NULL;
+	gnome_help_display("pioneers", NULL, &error);
+	if (error) {
+		log_message(MSG_ERROR, "%s: %s\n", _("Show the manual"),
+			    error->message);
+		g_error_free(error);
+	}
 }
 #endif
 
@@ -1149,6 +1206,8 @@ void gui_set_game_params(const GameParams * params)
 	legend_page = legend_create_content();
 	gtk_notebook_insert_page(GTK_NOTEBOOK(map_notebook),
 				 legend_page, label, LEGEND_PAGE);
+	if (!legend_page_enabled)
+		gui_show_legend_page(FALSE);
 	g_object_unref(label);
 }
 
@@ -1220,7 +1279,7 @@ static void register_pixmaps(void)
 			}
 		} else {
 			/* Missing pixmap */
-			g_warning(_("Pixmap not found: %s\n"), filename);
+			g_warning("Pixmap not found: %s", filename);
 		}
 
 		gtk_icon_factory_add(factory, pioneers_pixmaps[idx], icon);
@@ -1230,9 +1289,54 @@ static void register_pixmaps(void)
 
 	gtk_icon_factory_add_default(factory);
 	g_object_unref(factory);
+
+	for (idx = 0; idx < NO_RESOURCE; idx++) {
+		gchar *filename;
+
+		/* determine full path to pixmap file */
+		filename = g_build_filename(DATADIR, "pixmaps",
+					    resources_pixmaps[idx], NULL);
+		if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
+			GdkPixbuf *pixbuf;
+			GError *error = NULL;
+			pixbuf =
+			    gdk_pixbuf_new_from_file(filename, &error);
+			if (error != NULL) {
+				g_warning("Error loading pixmap %s\n",
+					  filename);
+				g_error_free(error);
+			} else {
+				gdk_pixbuf_render_pixmap_and_mask(pixbuf,
+								  &resource_pixmap
+								  [idx].p,
+								  &resource_pixmap
+								  [idx].b,
+								  128);
+				resource_pixmap[idx].gcb =
+				    gdk_gc_new(resource_pixmap[idx].b);
+				gdk_gc_set_function(resource_pixmap[idx].
+						    gcb, GDK_OR);
+				resource_pixmap[idx].gcp =
+				    gdk_gc_new(resource_pixmap[idx].p);
+				gdk_gc_set_clip_mask(resource_pixmap[idx].
+						     gcp,
+						     resource_pixmap[idx].
+						     b);
+				if (!resource_pixmap_res)
+					resource_pixmap_res =
+					    gdk_pixbuf_get_width(pixbuf);
+				g_object_unref(pixbuf);
+			}
+		} else {
+			/* Missing pixmap */
+			g_warning("Pixmap not found: %s", filename);
+		}
+
+		g_free(filename);
+	}
 }
 
-GtkWidget *gui_build_interface()
+GtkWidget *gui_build_interface(void)
 {
 	GtkWidget *vbox;
 	GtkWidget *menubar;
@@ -1282,7 +1386,7 @@ GtkWidget *gui_build_interface()
 		gtk_window_set_default_icon_from_file(icon_file, NULL);
 	} else {
 		/* Missing pixmap, main icon file */
-		g_warning(_("Pixmap not found: %s\n"), icon_file);
+		g_warning("Pixmap not found: %s", icon_file);
 	}
 	g_free(icon_file);
 
@@ -1296,6 +1400,9 @@ GtkWidget *gui_build_interface()
 	set_color_summary(config_get_int_with_default
 			  ("settings/color_summary", TRUE));
 
+	set_announce_player(config_get_int_with_default
+			    ("settings/announce_player", TRUE));
+
 	legend_page_enabled =
 	    config_get_int_with_default("settings/legend_page", FALSE);
 
@@ -1366,3 +1473,25 @@ GtkWidget *gui_build_interface()
 
 	return app_window;
 }
+
+void gui_get_resource_pixmap(gint idx, GdkPixmap ** p, GdkBitmap ** b,
+			     GdkGC ** gcp, GdkGC ** gcb)
+{
+	g_assert(idx < NO_RESOURCE);
+	*p = resource_pixmap[idx].p;
+	*b = resource_pixmap[idx].b;
+	if (gcp)
+		*gcp = resource_pixmap[idx].gcp;
+	if (gcb)
+		*gcb = resource_pixmap[idx].gcb;
+}
+
+gint gui_get_resource_pixmap_res()
+{
+	return resource_pixmap_res;
+}
+
+void gui_set_show_no_setup_nodes(gboolean show)
+{
+	guimap_set_show_no_setup_nodes(gmap, show);
+}
