@@ -25,8 +25,6 @@
 #include "cost.h"
 #include "histogram.h"
 
-static gboolean already_rejected = FALSE; /** @todo UGLY HACK, remove later */
-
 /* local functions */
 static void frontend_state_turn(GuiEvent event);
 static void build_road_cb(MapElement edge, MapElement extra);
@@ -36,13 +34,14 @@ static void move_ship_cb(MapElement edge, MapElement extra);
 static void build_settlement_cb(MapElement node, MapElement extra);
 static void build_city_cb(MapElement node, MapElement extra);
 
+static void dummy_state(G_GNUC_UNUSED GuiEvent event)
+{
+}
 
 /* for gold and discard, remember the previous gui state */
-static GuiState previous_state;
+static GuiState previous_state = dummy_state;
 
-static gboolean gold_busy = FALSE, discard_busy = FALSE, robber_busy =
-    FALSE;
-static gboolean have_turn = FALSE;
+static gboolean discard_busy = FALSE, robber_busy = FALSE;
 
 static void frontend_state_idle(G_GNUC_UNUSED GuiEvent event)
 {
@@ -87,6 +86,12 @@ void build_bridge_cb(MapElement edge, G_GNUC_UNUSED MapElement extra)
 	cb_build_bridge(edge.edge);
 }
 
+static void cancel_move_ship_cb(void)
+{
+	callbacks.instructions(_("Ship movement canceled."));
+	gui_prompt_hide();
+}
+
 void move_ship_cb(MapElement edge, G_GNUC_UNUSED MapElement extra)
 {
 	MapElement ship_from;
@@ -94,7 +99,7 @@ void move_ship_cb(MapElement edge, G_GNUC_UNUSED MapElement extra)
 	callbacks.instructions(_("Select a new location for the ship."));
 	gui_prompt_show(_("Select a new location for the ship."));
 	gui_cursor_set(SHIP_CURSOR, can_ship_be_moved_to, do_move_ship_cb,
-		       &ship_from);
+		       cancel_move_ship_cb, &ship_from);
 }
 
 void build_settlement_cb(MapElement node, G_GNUC_UNUSED MapElement extra)
@@ -111,7 +116,7 @@ void build_city_cb(MapElement node, G_GNUC_UNUSED MapElement extra)
 static void frontend_state_trade(GuiEvent event)
 {
 	static gboolean trading = FALSE;
-	QuoteInfo *quote;
+	const QuoteInfo *quote;
 	switch (event) {
 	case GUI_UPDATE:
 		frontend_gui_check(GUI_TRADE_CALL, can_call_for_quotes());
@@ -119,6 +124,7 @@ static void frontend_state_trade(GuiEvent event)
 				   trade_valid_selection());
 		frontend_gui_check(GUI_TRADE_FINISH, TRUE);
 		frontend_gui_check(GUI_TRADE, TRUE);
+		gui_cursor_none();	/* Finish single click build */
 		break;
 	case GUI_TRADE_CALL:
 		trading = TRUE;
@@ -156,7 +162,8 @@ static void frontend_state_trade(GuiEvent event)
 }
 
 void frontend_trade_add_quote(int player_num, int quote_num,
-			      gint * they_supply, gint * they_receive)
+			      const gint * they_supply,
+			      const gint * they_receive)
 {
 	trade_add_quote(player_num, quote_num, they_supply, they_receive);
 	frontend_gui_update();
@@ -180,7 +187,7 @@ static void frontend_state_quote(GuiEvent event)
 	case GUI_UPDATE:
 		frontend_gui_check(GUI_QUOTE_SUBMIT, can_submit_quote());
 		frontend_gui_check(GUI_QUOTE_DELETE, can_delete_quote());
-		frontend_gui_check(GUI_QUOTE_REJECT, !already_rejected);
+		frontend_gui_check(GUI_QUOTE_REJECT, can_reject_quote());
 		break;
 	case GUI_QUOTE_SUBMIT:
 		cb_quote(quote_next_num(), quote_we_supply(),
@@ -190,9 +197,8 @@ static void frontend_state_quote(GuiEvent event)
 		cb_delete_quote(quote_current_quote()->var.d.quote_num);
 		return;
 	case GUI_QUOTE_REJECT:
-		if (!already_rejected)
-			cb_end_quote();
-		already_rejected = TRUE;
+		quote_player_finish(my_player_num());
+		cb_end_quote();
 		return;
 	default:
 		break;
@@ -202,7 +208,6 @@ static void frontend_state_quote(GuiEvent event)
 void frontend_quote(gint player_num, gint * they_supply,
 		    gint * they_receive)
 {
-	already_rejected = FALSE;
 	if (get_gui_state() == frontend_state_quote) {
 		quote_begin_again(player_num, they_supply, they_receive);
 	} else {
@@ -212,8 +217,9 @@ void frontend_quote(gint player_num, gint * they_supply,
 	frontend_gui_update();
 }
 
-void frontend_quote_add(int player_num, int quote_num, gint * they_supply,
-			gint * they_receive)
+void frontend_quote_add(int player_num, int quote_num,
+			const gint * they_supply,
+			const gint * they_receive)
 {
 	quote_add_quote(player_num, quote_num, they_supply, they_receive);
 	frontend_gui_update();
@@ -231,20 +237,19 @@ void frontend_quote_player_end(gint player_num)
 	frontend_gui_update();
 }
 
-void frontend_quote_end()
+void frontend_quote_end(void)
 {
 	quote_finish();
 	set_gui_state(frontend_state_idle);
 }
 
-void frontend_quote_start()
+void frontend_quote_start(void)
 {
 	/*set_gui_state (frontend_state_quote); */
 }
 
-void frontend_quote_monitor()
+void frontend_quote_monitor(void)
 {
-	set_gui_state(frontend_state_idle);
 }
 
 static gboolean check_road(MapElement element, gint owner,
@@ -315,7 +320,11 @@ static void frontend_state_turn(GuiEvent event)
 						(), check_settlement,
 						build_settlement_cb,
 						turn_can_build_city(),
-						check_city, build_city_cb);
+						check_city, build_city_cb,
+						turn_can_move_ship(),
+						check_ship_move,
+						move_ship_cb,
+						cancel_move_ship_cb);
 		break;
 	case GUI_ROLL:
 		cb_roll();
@@ -325,27 +334,27 @@ static void frontend_state_turn(GuiEvent event)
 		return;
 	case GUI_ROAD:
 		gui_cursor_set(ROAD_CURSOR, check_road, build_road_cb,
-			       NULL);
+			       NULL, NULL);
 		return;
 	case GUI_SHIP:
 		gui_cursor_set(SHIP_CURSOR, check_ship, build_ship_cb,
-			       NULL);
+			       NULL, NULL);
 		return;
 	case GUI_MOVE_SHIP:
 		gui_cursor_set(SHIP_CURSOR, check_ship_move, move_ship_cb,
-			       NULL);
+			       NULL, NULL);
 		return;
 	case GUI_BRIDGE:
 		gui_cursor_set(BRIDGE_CURSOR, check_bridge,
-			       build_bridge_cb, NULL);
+			       build_bridge_cb, NULL, NULL);
 		return;
 	case GUI_SETTLEMENT:
 		gui_cursor_set(SETTLEMENT_CURSOR, check_settlement,
-			       build_settlement_cb, NULL);
+			       build_settlement_cb, NULL, NULL);
 		return;
 	case GUI_CITY:
 		gui_cursor_set(CITY_CURSOR, check_city, build_city_cb,
-			       NULL);
+			       NULL, NULL);
 		return;
 	case GUI_TRADE:
 		trade_begin();
@@ -358,7 +367,6 @@ static void frontend_state_turn(GuiEvent event)
 		cb_buy_develop();
 		return;
 	case GUI_FINISH:
-		have_turn = FALSE;
 		cb_end_turn();
 		gui_cursor_none();	/* Finish single click build */
 		set_gui_state(frontend_state_idle);
@@ -368,17 +376,17 @@ static void frontend_state_turn(GuiEvent event)
 	}
 }
 
-void frontend_turn()
+void frontend_turn(void)
 {
 	/* if it already is our turn, just update the gui (maybe something
 	 * happened), but don't beep */
-	if (have_turn) {
+	if (get_gui_state() == frontend_state_turn
+	    || get_gui_state() == frontend_state_trade) {
 		/* this is in the if, because it gets called from set_gui_state
 		 * anyway. */
 		frontend_gui_update();
 		return;
 	}
-	have_turn = TRUE;
 	set_gui_state(frontend_state_turn);
 	gdk_beep();
 }
@@ -403,22 +411,22 @@ static void frontend_state_roadbuilding(GuiEvent event)
 		     check_ship, build_ship_cb,
 		     road_building_can_build_bridge(), check_bridge,
 		     build_bridge_cb, FALSE, NULL, NULL, FALSE, NULL,
-		     NULL);
+		     NULL, FALSE, NULL, NULL, NULL);
 		break;
 	case GUI_UNDO:
 		cb_undo();
 		return;
 	case GUI_ROAD:
 		gui_cursor_set(ROAD_CURSOR, check_road, build_road_cb,
-			       NULL);
+			       NULL, NULL);
 		return;
 	case GUI_SHIP:
 		gui_cursor_set(SHIP_CURSOR, check_ship, build_ship_cb,
-			       NULL);
+			       NULL, NULL);
 		return;
 	case GUI_BRIDGE:
 		gui_cursor_set(BRIDGE_CURSOR, check_bridge,
-			       build_bridge_cb, NULL);
+			       build_bridge_cb, NULL, NULL);
 		return;
 	case GUI_FINISH:
 		cb_end_turn();
@@ -456,7 +464,7 @@ static void frontend_state_monopoly(GuiEvent event)
 	}
 }
 
-void frontend_monopoly()
+void frontend_monopoly(void)
 {
 	monopoly_create_dlg();
 	set_gui_state(frontend_state_monopoly);
@@ -468,7 +476,7 @@ static void frontend_state_plenty(GuiEvent event)
 	gint plenty[NO_RESOURCE];
 	switch (event) {
 	case GUI_UPDATE:
-		frontend_gui_check(GUI_PLENTY, TRUE);
+		frontend_gui_check(GUI_PLENTY, plenty_can_activate());
 		break;
 	case GUI_PLENTY:
 		plenty_resources(plenty);
@@ -491,26 +499,31 @@ void frontend_plenty(const gint * bank)
 /* discard */
 static void frontend_state_discard(GuiEvent event)
 {
+	gint discards[NO_RESOURCE];
+
 	switch (event) {
 	case GUI_UPDATE:
 		frontend_gui_check(GUI_DISCARD, can_discard());
 		break;
 	case GUI_DISCARD:
-		cb_discard(discard_get_list());
+		discard_get_list(discards);
+		cb_discard(discards);
 		return;
 	default:
 		break;
 	}
 }
 
-void frontend_discard()
+void frontend_discard(void)
 {
 	/* set state to idle until we must discard (or discard ends) */
 	if (!discard_busy) {
 		discard_busy = TRUE;
 		discard_begin();
+		g_assert(previous_state == dummy_state);
 		previous_state = get_gui_state();
 		set_gui_state(frontend_state_idle);
+		gui_cursor_none();	/* Clear possible cursor */
 	}
 }
 
@@ -524,21 +537,24 @@ void frontend_discard_add(gint player_num, gint discard_num)
 	frontend_gui_update();
 }
 
-void frontend_discard_remove(gint player_num, gint * list)
+void frontend_discard_remove(gint player_num)
 {
 	if (discard_busy) {
-		discard_player_did(player_num, list);
+		discard_player_did(player_num);
 		if (player_num == my_player_num())
 			set_gui_state(frontend_state_idle);
 	}
 	frontend_gui_update();
 }
 
-void frontend_discard_done()
+void frontend_discard_done(void)
 {
 	discard_busy = FALSE;
 	discard_end();
-	set_gui_state(previous_state);
+	if (previous_state != dummy_state) {
+		set_gui_state(previous_state);
+		previous_state = dummy_state;
+	}
 }
 
 /* gold */
@@ -550,20 +566,22 @@ static void frontend_state_gold(GuiEvent event)
 		frontend_gui_check(GUI_CHOOSE_GOLD, can_choose_gold());
 		break;
 	case GUI_CHOOSE_GOLD:
-		cb_choose_gold(choose_gold_get_list(gold));
+		choose_gold_get_list(gold);
+		cb_choose_gold(gold);
 		return;
 	default:
 		break;
 	}
 }
 
-void frontend_gold()
+void frontend_gold(void)
 {
-	if (!gold_busy) {
-		gold_busy = TRUE;
+	if (get_gui_state() != frontend_state_gold) {
 		gold_choose_begin();
+		g_assert(previous_state == dummy_state);
 		previous_state = get_gui_state();
 		set_gui_state(frontend_state_gold);
+		gui_cursor_none();	/* Clear possible cursor */
 	}
 }
 
@@ -585,17 +603,22 @@ void frontend_gold_remove(gint player_num, gint * resources)
 	frontend_gui_update();
 }
 
-void frontend_gold_done()
+void frontend_gold_done(void)
 {
-	gold_busy = FALSE;
 	gold_choose_end();
-	set_gui_state(previous_state);
+	if (previous_state != dummy_state) {
+		set_gui_state(previous_state);
+		previous_state = dummy_state;
+	}
 }
 
 void frontend_game_over(gint player, gint points)
 {
 	gui_cursor_none();	/* Clear possible (robber) cursor */
-	have_turn = FALSE;
+	if (robber_busy) {
+		robber_busy = FALSE;
+		gui_prompt_hide();
+	}
 	gameover_create_dlg(player, points);
 	set_gui_state(frontend_state_idle);
 }
@@ -608,17 +631,14 @@ void frontend_rolled_dice(gint die1, gint die2, gint player_num)
 	frontend_gui_update();
 }
 
-void frontend_beep()
-{
-	gdk_beep();
-	frontend_gui_update();
-}
-
 static void place_robber(const Hex * hex, gint victim)
 {
 	cb_place_robber(hex, victim);
 	robber_busy = FALSE;
-	set_gui_state(previous_state);
+	if (previous_state != dummy_state) {
+		set_gui_state(previous_state);
+		previous_state = dummy_state;
+	}
 }
 
 static void rob_building(MapElement node, MapElement hex)
@@ -693,7 +713,8 @@ static void place_robber_or_pirate_cb(MapElement hex,
 			break;
 		default:
 			gui_cursor_set(STEAL_SHIP_CURSOR,
-				       can_edge_be_robbed, rob_edge, &hex);
+				       can_edge_be_robbed, rob_edge, NULL,
+				       &hex);
 			gui_set_instructions(_
 					     ("Select the ship to steal from."));
 			gui_prompt_show(_
@@ -712,7 +733,7 @@ static void place_robber_or_pirate_cb(MapElement hex,
 		default:
 			gui_cursor_set(STEAL_BUILDING_CURSOR,
 				       can_building_be_robbed,
-				       rob_building, &hex);
+				       rob_building, NULL, &hex);
 			gui_set_instructions(_
 					     ("Select the building to steal from."));
 			gui_prompt_show(_
@@ -729,16 +750,17 @@ static gboolean check_move_robber_or_pirate(MapElement element,
 	return can_robber_or_pirate_be_moved(element.hex);
 }
 
-void frontend_robber()
+void frontend_robber(void)
 {
 	if (robber_busy)
 		return;
 	robber_busy = TRUE;
+	g_assert(previous_state == dummy_state);
 	previous_state = get_gui_state();
 	set_gui_state(frontend_state_idle);
 	gui_cursor_set(ROBBER_CURSOR,
 		       check_move_robber_or_pirate,
-		       place_robber_or_pirate_cb, NULL);
+		       place_robber_or_pirate_cb, NULL, NULL);
 	gui_prompt_show(_("Place the robber"));
 	frontend_gui_update();
 }
@@ -794,6 +816,7 @@ static void frontend_mode_setup(GuiEvent event)
 						setup_can_build_settlement
 						(), check_settlement_setup,
 						build_settlement_cb, FALSE,
+						NULL, NULL, FALSE, NULL,
 						NULL, NULL);
 		break;
 	case GUI_UNDO:
@@ -806,20 +829,23 @@ static void frontend_mode_setup(GuiEvent event)
 		return;
 	case GUI_ROAD:
 		gui_cursor_set(ROAD_CURSOR,
-			       check_road_setup, build_road_cb, NULL);
+			       check_road_setup, build_road_cb, NULL,
+			       NULL);
 		return;
 	case GUI_SHIP:
 		gui_cursor_set(SHIP_CURSOR,
-			       check_ship_setup, build_ship_cb, NULL);
+			       check_ship_setup, build_ship_cb, NULL,
+			       NULL);
 		return;
 	case GUI_BRIDGE:
 		gui_cursor_set(BRIDGE_CURSOR,
-			       check_bridge_setup, build_bridge_cb, NULL);
+			       check_bridge_setup, build_bridge_cb, NULL,
+			       NULL);
 		return;
 	case GUI_SETTLEMENT:
 		gui_cursor_set(SETTLEMENT_CURSOR,
 			       check_settlement_setup,
-			       build_settlement_cb, NULL);
+			       build_settlement_cb, NULL, NULL);
 		return;
 	case GUI_FINISH:
 		cb_end_turn();
