@@ -2,7 +2,7 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003,2006 Bas Wijnen <shevek@fmf.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ static void change_name_cb(GtkDialog * dlg, int response_id,
 	gtk_widget_destroy(GTK_WIDGET(dlg));
 }
 
-void name_create_dlg()
+void name_create_dlg(void)
 {
 	GtkWidget *dlg_vbox;
 	GtkWidget *hbox;
