@@ -2,7 +2,7 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003, 2006 Bas Wijnen <shevek@fmf.nl>
  * Copyright (C) 2004 Roland Clobus <rclobus@bigfoot.com>
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include "frontend.h"
 #include "log.h"
 #include "gtkbugs.h"
+#include "common_gtk.h"
 
 static void player_show_connected_at_iter(gint player_num,
 					  gboolean connected,
@@ -67,32 +68,36 @@ enum {
 	SUMMARY_COLUMN_TEXT_COLOUR, /**< Colour of the description */
 	SUMMARY_COLUMN_SCORE, /**< Score of the items (as string) */
 	SUMMARY_COLUMN_STATISTIC, /**< enum Statistic value+1, or 0 if not in the enum */
+	SUMMARY_COLUMN_POINTS_ID, /**< Id of points, or -1 */
 	SUMMARY_COLUMN_LAST
 };
 
 static Player players[MAX_PLAYERS];
 static GtkListStore *summary_store; /**< the player summary data */
 static GtkWidget *summary_widget; /**< the player summary widget */
-/** The summary line is found here */
-static GtkTreeIter summary_found_iter;
-/** Has the summary line been found ? */
-enum {
-	STORE_MATCH_EXACT,
-	STORE_MATCH_INSERT_BEFORE,
-	STORE_NO_MATCH
-} summary_found_flag;
 static gboolean summary_color_enabled = TRUE;
+static gboolean announce_player = FALSE;
 
 /** Structure to find combination of player and statistic */
 struct Player_statistic {
+	enum TFindResult result;
+	GtkTreeIter iter;
 	gint player_num;
 	gint statistic;
 };
 
+/** Structure to find combination of player and points */
+struct Player_point {
+	enum TFindResult result;
+	GtkTreeIter iter;
+	gint player_num;
+	gint point_id;
+};
+
 static GtkWidget *turn_area;	/** turn indicator in status bar */
 static const gint turn_area_icon_width = 30;
 
-void player_init()
+void player_init(void)
 {
 	gint idx;
 	GdkColormap *cmap;
@@ -130,7 +135,9 @@ GdkPixbuf *player_create_icon(GtkWidget * widget, gint player_num,
 
 	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
 
-	pixmap = gdk_pixmap_new(widget->window, width, height, -1);
+	pixmap =
+	    gdk_pixmap_new(widget->window, width, height,
+			   gdk_visual_get_system()->depth);
 
 	gc = gdk_gc_new(pixmap);
 	gdk_gc_set_foreground(gc, &black);
@@ -142,9 +149,9 @@ GdkPixbuf *player_create_icon(GtkWidget * widget, gint player_num,
 	if (!connected) {
 		gdk_gc_set_foreground(gc, &black);
 		gdk_draw_rectangle(pixmap, gc, FALSE,
-				   3, 3, width - 6, height - 6);
+				   3, 3, width - 7, height - 7);
 		gdk_draw_rectangle(pixmap, gc, FALSE,
-				   6, 6, width - 12, height - 12);
+				   6, 6, width - 13, height - 13);
 	}
 
 	/* Store the pixmap in player->user_data. 
@@ -164,28 +171,6 @@ GdkPixbuf *player_create_icon(GtkWidget * widget, gint player_num,
 	return temp;
 }
 
-/** Locate a line suitable for a player */
-static gboolean summary_locate_player(GtkTreeModel * model,
-				      G_GNUC_UNUSED GtkTreePath * path,
-				      GtkTreeIter * iter,
-				      gpointer user_data)
-{
-	int wanted = GPOINTER_TO_INT(user_data);
-	int current;
-	gtk_tree_model_get(model, iter, SUMMARY_COLUMN_PLAYER_NUM,
-			   &current, -1);
-	if (current > wanted) {
-		summary_found_flag = STORE_MATCH_INSERT_BEFORE;
-		summary_found_iter = *iter;
-		return TRUE;
-	} else if (current == wanted) {
-		summary_found_flag = STORE_MATCH_EXACT;
-		summary_found_iter = *iter;
-		return TRUE;
-	}
-	return FALSE;
-}
-
 /** Locate a line suitable for the statistic */
 static gboolean summary_locate_statistic(GtkTreeModel * model,
 					 G_GNUC_UNUSED GtkTreePath * path,
@@ -194,24 +179,63 @@ static gboolean summary_locate_statistic(GtkTreeModel * model,
 {
 	struct Player_statistic *ps =
 	    (struct Player_statistic *) user_data;
-	int current_player;
-	int current_statistic;
+	gint current_player;
+	gint current_statistic;
 	gtk_tree_model_get(model, iter,
 			   SUMMARY_COLUMN_PLAYER_NUM, &current_player,
 			   SUMMARY_COLUMN_STATISTIC, &current_statistic,
 			   -1);
 	if (current_player > ps->player_num) {
-		summary_found_flag = STORE_MATCH_INSERT_BEFORE;
-		summary_found_iter = *iter;
+		ps->result = FIND_MATCH_INSERT_BEFORE;
+		ps->iter = *iter;
 		return TRUE;
 	} else if (current_player == ps->player_num) {
 		if (current_statistic > ps->statistic) {
-			summary_found_flag = STORE_MATCH_INSERT_BEFORE;
-			summary_found_iter = *iter;
+			ps->result = FIND_MATCH_INSERT_BEFORE;
+			ps->iter = *iter;
 			return TRUE;
 		} else if (current_statistic == ps->statistic) {
-			summary_found_flag = STORE_MATCH_EXACT;
-			summary_found_iter = *iter;
+			ps->result = FIND_MATCH_EXACT;
+			ps->iter = *iter;
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
+/** Locate a line suitable for the statistic */
+static gboolean summary_locate_point(GtkTreeModel * model,
+				     G_GNUC_UNUSED GtkTreePath * path,
+				     GtkTreeIter * iter,
+				     gpointer user_data)
+{
+	struct Player_point *pp = (struct Player_point *) user_data;
+	gint current_player;
+	gint current_point_id;
+	gint current_statistic;
+
+	gtk_tree_model_get(model, iter,
+			   SUMMARY_COLUMN_PLAYER_NUM, &current_player,
+			   SUMMARY_COLUMN_STATISTIC, &current_statistic,
+			   SUMMARY_COLUMN_POINTS_ID, &current_point_id,
+			   -1);
+	if (current_player > pp->player_num) {
+		pp->result = FIND_MATCH_INSERT_BEFORE;
+		pp->iter = *iter;
+		return TRUE;
+	} else if (current_player == pp->player_num) {
+		if (current_statistic >= STAT_SOLDIERS) {
+			pp->result = FIND_MATCH_INSERT_BEFORE;
+			pp->iter = *iter;
+			return TRUE;
+		}
+		if (current_point_id > pp->point_id) {
+			pp->result = FIND_MATCH_INSERT_BEFORE;
+			pp->iter = *iter;
+			return TRUE;
+		} else if (current_point_id == pp->point_id) {
+			pp->result = FIND_MATCH_EXACT;
+			pp->iter = *iter;
 			return TRUE;
 		}
 	}
@@ -222,44 +246,50 @@ static gboolean summary_locate_statistic(GtkTreeModel * model,
 static void refresh_victory_point_total(int player_num)
 {
 	gchar points[16];
+	GtkTreeIter iter;
+	enum TFindResult found;
 
 	if (player_num < 0 || player_num >= G_N_ELEMENTS(players))
 		return;
 
-	snprintf(points, sizeof(points), "%d",
-		 player_get_score(player_num));
+	found =
+	    find_integer_in_tree(GTK_TREE_MODEL(summary_store), &iter,
+				 SUMMARY_COLUMN_PLAYER_NUM, player_num);
 
-	summary_found_flag = STORE_NO_MATCH;
-	gtk_tree_model_foreach(GTK_TREE_MODEL(summary_store),
-			       summary_locate_player,
-			       GINT_TO_POINTER(player_num));
-	if (summary_found_flag == STORE_MATCH_EXACT)
-		gtk_list_store_set(summary_store, &summary_found_iter,
+	if (found == FIND_MATCH_EXACT) {
+		snprintf(points, sizeof(points), "%d",
+			 player_get_score(player_num));
+		gtk_list_store_set(summary_store, &iter,
 				   SUMMARY_COLUMN_SCORE, points, -1);
-
+	}
 }
 
-/** Locate a line suitable for a player */
-static gboolean summary_apply_colors(GtkTreeModel * model,
-				     G_GNUC_UNUSED GtkTreePath * path,
-				     GtkTreeIter * iter,
-				     G_GNUC_UNUSED gpointer user_data)
+/** Apply colors to the summary */
+static gboolean summary_apply_colors_cb(GtkTreeModel * model,
+					G_GNUC_UNUSED GtkTreePath * path,
+					GtkTreeIter * iter,
+					G_GNUC_UNUSED gpointer user_data)
 {
 	gint current_statistic;
+	gint point_id;
 
 	gtk_tree_model_get(model, iter,
 			   SUMMARY_COLUMN_STATISTIC, &current_statistic,
-			   -1);
+			   SUMMARY_COLUMN_POINTS_ID, &point_id, -1);
 	if (current_statistic > 0)
 		gtk_list_store_set(summary_store, iter,
 				   SUMMARY_COLUMN_TEXT_COLOUR,
 				   summary_color_enabled ?
 				   statistics[current_statistic -
 					      1].textcolor : &black, -1);
+	else if (point_id >= 0)
+		gtk_list_store_set(summary_store, iter,
+				   SUMMARY_COLUMN_TEXT_COLOUR,
+				   summary_color_enabled ?
+				   &ps_largest : &black, -1);
 	return FALSE;
 }
 
-
 void set_color_summary(gboolean flag)
 {
 	if (flag != summary_color_enabled) {
@@ -267,16 +297,26 @@ void set_color_summary(gboolean flag)
 		if (summary_store)
 			gtk_tree_model_foreach(GTK_TREE_MODEL
 					       (summary_store),
-					       summary_apply_colors, NULL);
+					       summary_apply_colors_cb,
+					       NULL);
 	}
 }
 
+gboolean get_announce_player(void)
+{
+	return announce_player;
+}
+
+void set_announce_player(gboolean announce)
+{
+	announce_player = announce;
+}
+
 void frontend_new_statistics(gint player_num, StatisticType type,
 			     G_GNUC_UNUSED gint num)
 {
 	Player *player = player_get(player_num);
 	gint value;
-	gchar desc[128];
 	gchar points[16];
 	GtkTreeIter iter;
 	struct Player_statistic ps;
@@ -285,47 +325,50 @@ void frontend_new_statistics(gint player_num, StatisticType type,
 	if (stat_get_vp_value(type) > 0)
 		refresh_victory_point_total(player_num);
 
-	summary_found_flag = STORE_NO_MATCH;
+	ps.result = FIND_NO_MATCH;
 	ps.player_num = player_num;
 	ps.statistic = type + 1;
 	gtk_tree_model_foreach(GTK_TREE_MODEL(summary_store),
 			       summary_locate_statistic, &ps);
 
 	if (value == 0) {
-		if (summary_found_flag == STORE_MATCH_EXACT)
-			gtk_list_store_remove(summary_store,
-					      &summary_found_iter);
+		if (ps.result == FIND_MATCH_EXACT)
+			gtk_list_store_remove(summary_store, &ps.iter);
 	} else {
+		gchar *desc;
 		if (value == 1) {
 			if (statistics[type].plural != NULL)
-				sprintf(desc, "%d %s", value,
-					gettext(statistics[type].
-						singular));
+				desc = g_strdup_printf("%d %s", value,
+						       gettext(statistics
+							       [type].
+							       singular));
 			else
-				strcpy(desc,
-				       gettext(statistics[type].singular));
+				desc = g_strdup(gettext
+						(statistics[type].
+						 singular));
 		} else
-			sprintf(desc, "%d %s", value,
-				gettext(statistics[type].plural));
+			desc = g_strdup_printf("%d %s", value,
+					       gettext(statistics[type].
+						       plural));
 		if (stat_get_vp_value(type) > 0)
 			sprintf(points, "%d",
 				value * stat_get_vp_value(type));
 		else
 			strcpy(points, "");
 
-		switch (summary_found_flag) {
-		case STORE_NO_MATCH:
+		switch (ps.result) {
+		case FIND_NO_MATCH:
 			gtk_list_store_append(summary_store, &iter);
 			break;
-		case STORE_MATCH_INSERT_BEFORE:
+		case FIND_MATCH_INSERT_BEFORE:
 			gtk_list_store_insert_before(summary_store, &iter,
-						     &summary_found_iter);
+						     &ps.iter);
 			break;
-		case STORE_MATCH_EXACT:
-			iter = summary_found_iter;
+		case FIND_MATCH_EXACT:
+			iter = ps.iter;
 			break;
 		default:
-			g_assert(FALSE);
+			g_error("unknown case in frontend_new_statistics");
 		};
 		gtk_list_store_set(summary_store, &iter,
 				   SUMMARY_COLUMN_PLAYER_NUM, player_num,
@@ -334,39 +377,91 @@ void frontend_new_statistics(gint player_num, StatisticType type,
 				   summary_color_enabled ?
 				   statistics[type].textcolor : &black,
 				   SUMMARY_COLUMN_STATISTIC, type + 1,
+				   SUMMARY_COLUMN_POINTS_ID, -1,
 				   SUMMARY_COLUMN_SCORE, points, -1);
-
-
+		g_free(desc);
 	}
 	frontend_gui_update();
 }
 
+void frontend_new_points(gint player_num, Points * points, gboolean added)
+{
+	GtkTreeIter iter;
+	struct Player_point pp;
+	gchar score[16];
+
+	refresh_victory_point_total(player_num);
+
+	pp.result = FIND_NO_MATCH;
+	pp.player_num = player_num;
+	pp.point_id = points->id;
+	gtk_tree_model_foreach(GTK_TREE_MODEL(summary_store),
+			       summary_locate_point, &pp);
+
+	if (!added) {
+		if (pp.result != FIND_MATCH_EXACT)
+			g_error("cannot remove point");
+		gtk_list_store_remove(summary_store, &pp.iter);
+		frontend_gui_update();
+		return;
+	}
+
+	switch (pp.result) {
+	case FIND_NO_MATCH:
+		gtk_list_store_append(summary_store, &iter);
+		break;
+	case FIND_MATCH_INSERT_BEFORE:
+		gtk_list_store_insert_before(summary_store, &iter,
+					     &pp.iter);
+		break;
+	case FIND_MATCH_EXACT:
+		iter = pp.iter;
+		break;
+	default:
+		g_error("unknown case in frontend_new_points");
+	};
+	snprintf(score, sizeof(score), "%d", points->points);
+	gtk_list_store_set(summary_store, &iter,
+			   SUMMARY_COLUMN_PLAYER_NUM, player_num,
+			   SUMMARY_COLUMN_TEXT, _(points->name),
+			   SUMMARY_COLUMN_TEXT_COLOUR,
+			   summary_color_enabled ? &ps_largest : &black,
+			   SUMMARY_COLUMN_STATISTIC, 0,
+			   SUMMARY_COLUMN_POINTS_ID, points->id,
+			   SUMMARY_COLUMN_SCORE, score, -1);
+	frontend_gui_update();
+}
+
 static void player_create_find_player(gint player_num, GtkTreeIter * iter)
 {
+	GtkTreeIter found_iter;
+	enum TFindResult result;
+
 	/* Search for a place to add information about the player/viewer */
-	summary_found_flag = STORE_NO_MATCH;
-	gtk_tree_model_foreach(GTK_TREE_MODEL(summary_store),
-			       summary_locate_player,
-			       GINT_TO_POINTER(player_num));
-	switch (summary_found_flag) {
-	case STORE_NO_MATCH:
+	result =
+	    find_integer_in_tree(GTK_TREE_MODEL(summary_store),
+				 &found_iter, SUMMARY_COLUMN_PLAYER_NUM,
+				 player_num);
+
+	switch (result) {
+	case FIND_NO_MATCH:
 		gtk_list_store_append(summary_store, iter);
 		gtk_list_store_set(summary_store, iter,
 				   SUMMARY_COLUMN_PLAYER_NUM, player_num,
-				   -1);
+				   SUMMARY_COLUMN_POINTS_ID, -1, -1);
 		break;
-	case STORE_MATCH_INSERT_BEFORE:
+	case FIND_MATCH_INSERT_BEFORE:
 		gtk_list_store_insert_before(summary_store, iter,
-					     &summary_found_iter);
+					     &found_iter);
 		gtk_list_store_set(summary_store, iter,
 				   SUMMARY_COLUMN_PLAYER_NUM, player_num,
-				   -1);
+				   SUMMARY_COLUMN_POINTS_ID, -1, -1);
 		break;
-	case STORE_MATCH_EXACT:
-		*iter = summary_found_iter;
+	case FIND_MATCH_EXACT:
+		*iter = found_iter;
 		break;
 	default:
-		g_assert(FALSE);
+		g_error("unknown case in player_create_find_player");
 	};
 }
 
@@ -379,6 +474,10 @@ void frontend_player_name(gint player_num, const gchar * name)
 			   SUMMARY_COLUMN_TEXT, name, -1);
 
 	player_show_connected_at_iter(player_num, TRUE, &iter);
+	if (announce_player && callback_mode != MODE_INIT)
+		gdk_beep();
+
+	chat_player_name(player_num, name);
 }
 
 void frontend_viewer_name(gint viewer_num, const gchar * name)
@@ -388,6 +487,10 @@ void frontend_viewer_name(gint viewer_num, const gchar * name)
 	player_create_find_player(viewer_num, &iter);
 	gtk_list_store_set(summary_store, &iter,
 			   SUMMARY_COLUMN_TEXT, name, -1);
+	if (announce_player && callback_mode != MODE_INIT)
+		gdk_beep();
+
+	chat_player_name(viewer_num, name);
 }
 
 void frontend_player_quit(gint player_num)
@@ -396,6 +499,8 @@ void frontend_player_quit(gint player_num)
 
 	player_create_find_player(player_num, &iter);
 	player_show_connected_at_iter(player_num, FALSE, &iter);
+
+	chat_player_quit(player_num);
 }
 
 void frontend_viewer_quit(gint viewer_num)
@@ -404,6 +509,8 @@ void frontend_viewer_quit(gint viewer_num)
 
 	player_create_find_player(viewer_num, &iter);
 	gtk_list_store_remove(summary_store, &iter);
+
+	chat_viewer_quit(viewer_num);
 }
 
 static void player_show_connected_at_iter(gint player_num,
@@ -423,18 +530,21 @@ static void player_show_connected_at_iter(gint player_num,
  */
 static void player_show_summary(gint player_num)
 {
+	GtkTreeIter found_iter;
+	enum TFindResult result;
 	gboolean scroll_to_end = FALSE;
 
-	summary_found_flag = STORE_NO_MATCH;
-	gtk_tree_model_foreach(GTK_TREE_MODEL(summary_store),
-			       summary_locate_player,
-			       GINT_TO_POINTER(player_num + 1));
-	if (summary_found_flag == STORE_NO_MATCH) {
+	result =
+	    find_integer_in_tree(GTK_TREE_MODEL(summary_store),
+				 &found_iter, SUMMARY_COLUMN_PLAYER_NUM,
+				 player_num + 1);
+
+	if (result == FIND_NO_MATCH) {
 		scroll_to_end = TRUE;
 	} else {
 		GtkTreePath *path =
 		    gtk_tree_model_get_path(GTK_TREE_MODEL(summary_store),
-					    &summary_found_iter);
+					    &found_iter);
 		if (gtk_tree_path_prev(path))
 			gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW
 						     (summary_widget),
@@ -443,14 +553,14 @@ static void player_show_summary(gint player_num)
 		gtk_tree_path_free(path);
 	}
 
-	summary_found_flag = STORE_NO_MATCH;
-	gtk_tree_model_foreach(GTK_TREE_MODEL(summary_store),
-			       summary_locate_player,
-			       GINT_TO_POINTER(player_num));
-	if (summary_found_flag != STORE_NO_MATCH) {
+	result =
+	    find_integer_in_tree(GTK_TREE_MODEL(summary_store),
+				 &found_iter, SUMMARY_COLUMN_PLAYER_NUM,
+				 player_num);
+	if (result != FIND_NO_MATCH) {
 		GtkTreePath *path =
 		    gtk_tree_model_get_path(GTK_TREE_MODEL(summary_store),
-					    &summary_found_iter);
+					    &found_iter);
 		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(summary_widget),
 					     path, NULL, scroll_to_end,
 					     0.0, 0.0);
@@ -460,7 +570,7 @@ static void player_show_summary(gint player_num)
 	}
 }
 
-GtkWidget *player_build_summary()
+GtkWidget *player_build_summary(void)
 {
 	GtkWidget *vbox;
 	GtkWidget *label;
@@ -489,7 +599,8 @@ GtkWidget *player_build_summary()
 					   G_TYPE_STRING,	/* text */
 					   GDK_TYPE_COLOR,	/* text colour */
 					   G_TYPE_STRING,	/* score */
-					   G_TYPE_INT);	/* statistic */
+					   G_TYPE_INT,	/* statistic */
+					   G_TYPE_INT);	/* points */
 	summary_widget =
 	    gtk_tree_view_new_with_model(GTK_TREE_MODEL(summary_store));
 
@@ -594,7 +705,7 @@ static gint expose_turn_area_cb(GtkWidget * area,
 	return FALSE;
 }
 
-GtkWidget *player_build_turn_area()
+GtkWidget *player_build_turn_area(void)
 {
 	turn_area = gtk_drawing_area_new();
 	g_signal_connect(G_OBJECT(turn_area), "expose_event",
@@ -619,7 +730,7 @@ void player_show_current(gint player_num)
 	player_show_summary(player_num);
 }
 
-void player_clear_summary()
+void player_clear_summary(void)
 {
 	gtk_list_store_clear(GTK_LIST_STORE(summary_store));
 }
