@@ -3,7 +3,8 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
- * 
+ * Copyright (C) 2006 Roland Clobus <rclobus@bigfoot.com>
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -32,7 +33,8 @@
 typedef enum {
 	PARAM_STRING,
 	PARAM_INT,
-	PARAM_BOOL
+	PARAM_BOOL,
+	PARAM_INTLIST
 } ParamType;
 
 typedef struct {
@@ -68,12 +70,12 @@ static Param game_params[] = {
 	{PARAM_V(develop-library, PARAM_INT, num_develop_type[DEVEL_LIBRARY])},
 	{PARAM_V(develop-market, PARAM_INT, num_develop_type[DEVEL_MARKET])},
 	{PARAM_V(develop-soldier, PARAM_INT, num_develop_type[DEVEL_SOLDIER])},
-	{PARAM_V(tournament-time, PARAM_INT, tournament_time)},
-	{PARAM_V(use-pirate, PARAM_BOOL, use_pirate)}
+	{PARAM_V(use-pirate, PARAM_BOOL, use_pirate)},
+	{PARAM_V(island-discovery-bonus, PARAM_INTLIST, island_discovery_bonus)}
 };
 /* *INDENT-ON* */
 
-GameParams *params_new()
+GameParams *params_new(void)
 {
 	GameParams *params;
 
@@ -113,54 +115,71 @@ static gboolean match_word(gchar ** str, const gchar * word)
 	return FALSE;
 }
 
-static void build_int_list(GArray * array, char *str)
+/* Create a new array, filled with the list in str.
+ * If the array has length zero, NULL is returned.
+ */
+static GArray *build_int_list(const gchar * str)
 {
+	GArray *array = g_array_new(FALSE, FALSE, sizeof(gint));
 	while (*str != '\0') {
 		gint num;
+		gint sign = +1;
 
-		/* Skip leading space
-		 */
+		/* Skip leading space */
 		while (isspace(*str))
 			str++;
 		if (*str == '\0')
 			break;
-		/* Get the next number and add it to the array
-		 */
+		/* Get the next number and add it to the array */
 		num = 0;
+		if (*str == '-') {
+			sign = -1;
+			str++;
+		}
 		while (isdigit(*str))
 			num = num * 10 + *str++ - '0';
+		num *= sign;
 		g_array_append_val(array, num);
-		/* Get comma (if any)
-		 */
-		if (*str == ',')
+
+		/* Skip the non-digits */
+		while (!isdigit(*str) && *str != '-' && *str != '\0')
 			str++;
 	}
+	if (array->len == 0) {
+		g_array_free(array, FALSE);
+		array = NULL;
+	}
+	return array;
 }
 
-static void format_int_list(const gchar * name, GArray * array, char *str,
-			    int len)
+/* This function allocates a buffer and returns it.  It must be freed by the
+ * caller.  */
+static gchar *format_int_list(const gchar * name, GArray * array)
 {
+	gchar *old, *str = NULL;
 	int idx;
 
-	if (array->len == 0) {
-		strncpy(str, name, len);
-		return;
-	}
-	*str = '\0';
-	for (idx = 0; idx < array->len; idx++) {
-		gint num;
+	if (array == NULL)
+		return NULL;
 
-		if (idx == 0)
-			num = g_snprintf(str, len, "%s %d", name,
-					 g_array_index(array, gint, idx));
-		else
-			num = g_snprintf(str, len, ",%d",
-					 g_array_index(array, gint, idx));
-		if (num <= 0)
-			return;
-		str += num;
-		len -= num;
+	if (array->len == 0) {
+		str = g_strdup(name);
+		return str;
 	}
+	for (idx = 0; idx < array->len; idx++) {
+		old = str;
+		if (idx == 0)
+			str = g_strdup_printf("%s %d", name,
+					      g_array_index(array, gint,
+							    idx));
+		else
+			str = g_strdup_printf("%s,%d", str,
+					      g_array_index(array, gint,
+							    idx));
+		if (old)
+			g_free(old);
+	}
+	return str;
 }
 
 struct nosetup_t {
@@ -175,7 +194,7 @@ static gboolean find_no_setup(G_GNUC_UNUSED Map * map, Hex * hex,
 	for (idx = 0; idx < G_N_ELEMENTS(hex->nodes); ++idx) {
 		Node *node = hex->nodes[idx];
 		if (node->no_setup) {
-			gchar buff[512];
+			gchar buff[50];
 			if (node->x != hex->x || node->y != hex->y)
 				continue;
 			snprintf(buff, sizeof(buff), "nosetup %d %d %d",
@@ -191,7 +210,7 @@ void params_write_lines(GameParams * params, gboolean write_secrets,
 {
 	gint idx;
 	gint y;
-	gchar buff[512];
+	gchar *buff;
 	gchar *str;
 
 	switch (params->variant) {
@@ -215,33 +234,48 @@ void params_write_lines(GameParams * params, gboolean write_secrets,
 					    param->offset);
 			if (!str)
 				continue;
-			g_snprintf(buff, sizeof(buff), "%s %s",
-				   param->name, str);
+			buff = g_strdup_printf("%s %s", param->name, str);
 			func(user_data, buff);
+			g_free(buff);
 			break;
 		case PARAM_INT:
-			g_snprintf(buff, sizeof(buff), "%s %d",
-				   param->name,
-				   G_STRUCT_MEMBER(gint, params,
-						   param->offset));
+			buff = g_strdup_printf("%s %d", param->name,
+					       G_STRUCT_MEMBER(gint,
+							       params,
+							       param->
+							       offset));
 			func(user_data, buff);
+			g_free(buff);
 			break;
 		case PARAM_BOOL:
 			if (G_STRUCT_MEMBER
 			    (gboolean, params, param->offset)) {
-				g_snprintf(buff, sizeof(buff), "%s",
-					   param->name);
+				func(user_data, param->name);
+			}
+			break;
+		case PARAM_INTLIST:
+			buff =
+			    format_int_list(param->name,
+					    G_STRUCT_MEMBER(GArray *,
+							    params,
+							    param->
+							    offset));
+			/* Don't send empty intlists */
+			if (buff != NULL) {
 				func(user_data, buff);
+				g_free(buff);
 			}
 			break;
 		}
 	}
-	format_int_list("chits", params->map->chits, buff, sizeof(buff));
+	buff = format_int_list("chits", params->map->chits);
 	func(user_data, buff);
+	g_free(buff);
 	func(user_data, "map");
 	for (y = 0; y < params->map->y_size; y++) {
-		map_format_line(params->map, write_secrets, buff, y);
+		buff = map_format_line(params->map, write_secrets, y);
 		func(user_data, buff);
+		g_free(buff);
 	}
 	func(user_data, ".");
 	if (params->map) {
@@ -252,7 +286,7 @@ void params_write_lines(GameParams * params, gboolean write_secrets,
 	}
 }
 
-void params_load_line(GameParams * params, gchar * line)
+gboolean params_load_line(GameParams * params, gchar * line)
 {
 	gint idx;
 
@@ -261,16 +295,20 @@ void params_load_line(GameParams * params, gchar * line)
 	if (params->parsing_map) {
 		if (strcmp(line, ".") == 0) {
 			params->parsing_map = FALSE;
-			map_parse_finish(params->map);
+			if (!map_parse_finish(params->map)) {
+				map_free(params->map);
+				params->map = NULL;
+				return FALSE;
+			}
 		} else
-			map_parse_line(params->map, line);
-		return;
+			return map_parse_line(params->map, line);
+		return TRUE;
 	}
 	line = skip_space(line);
 	if (*line == '#')
-		return;
+		return TRUE;
 	if (*line == 0)
-		return;
+		return TRUE;
 	if (match_word(&line, "variant")) {
 		if (match_word(&line, "islands"))
 			params->variant = VAR_ISLANDS;
@@ -278,24 +316,21 @@ void params_load_line(GameParams * params, gchar * line)
 			params->variant = VAR_INTIMATE;
 		else
 			params->variant = VAR_DEFAULT;
-		return;
+		return TRUE;
 	}
 	if (match_word(&line, "map")) {
 		params->parsing_map = TRUE;
-		return;
+		return TRUE;
 	}
 	if (match_word(&line, "chits")) {
 		if (params->map->chits != NULL)
 			g_array_free(params->map->chits, TRUE);
-		params->map->chits =
-		    g_array_new(FALSE, FALSE, sizeof(gint));
-		build_int_list(params->map->chits, line);
-		if (params->map->chits->len == 0) {
+		params->map->chits = build_int_list(line);
+		if (params->map->chits == NULL) {
 			g_warning("Zero length chits array");
-			g_array_free(params->map->chits, FALSE);
-			params->map->chits = NULL;
+			return FALSE;
 		}
-		return;
+		return TRUE;
 	}
 	if (match_word(&line, "nosetup")) {
 		gint x = 0, y = 0, pos = 0;
@@ -307,16 +342,17 @@ void params_load_line(GameParams * params, gchar * line)
 		if (node) {
 			node->no_setup = TRUE;
 		} else {
-			g_warning(_
-				  ("Nosetup node %d %d %d is not in the map"),
-				  x, y, pos);
+			g_warning
+			    ("Nosetup node %d %d %d is not in the map", x,
+			     y, pos);
 		}
-		return;
+		return TRUE;
 	}
 
 	for (idx = 0; idx < G_N_ELEMENTS(game_params); idx++) {
 		Param *param = game_params + idx;
 		gchar *str;
+		GArray *array;
 
 		if (!match_word(&line, param->name))
 			continue;
@@ -330,24 +366,68 @@ void params_load_line(GameParams * params, gchar * line)
 			str = g_strchomp(g_strdup(line));
 			G_STRUCT_MEMBER(gchar *, params, param->offset) =
 			    str;
-			return;
+			return TRUE;
 		case PARAM_INT:
 			G_STRUCT_MEMBER(gint, params, param->offset) =
 			    atoi(line);
-			return;
+			return TRUE;
 		case PARAM_BOOL:
 			G_STRUCT_MEMBER(gboolean, params, param->offset) =
 			    TRUE;
-			return;
+			return TRUE;
+		case PARAM_INTLIST:
+			array =
+			    G_STRUCT_MEMBER(GArray *, params,
+					    param->offset);
+			if (array != NULL)
+				g_array_free(array, TRUE);
+			array = build_int_list(line);
+			if (array == NULL) {
+				g_warning("Zero length array for %s",
+					  param->name);
+			}
+			G_STRUCT_MEMBER(GArray *, params, param->offset) =
+			    array;
+			return array != NULL;
 		}
 	}
 	g_warning("Unknown keyword: %s", line);
+	return FALSE;
+}
+
+/* read a line from a file.  The memory needed is allocated.  The returned line
+ * is unbounded.  Returns FALSE if no (partial) line could be read */
+gboolean read_line_from_file(gchar ** line, FILE * f)
+{
+	gchar part[512];
+	gint len;
+
+	if (fgets(part, sizeof(part), f) == NULL)
+		return FALSE;
+
+	len = strlen(part);
+	g_assert(len > 0);
+	*line = g_strdup(part);
+	while ((*line)[len - 1] != '\n') {
+		gchar *oldline;
+		if (fgets(part, sizeof(part), f) == NULL)
+			break;
+		oldline = *line;
+		*line = g_strdup_printf("%s%s", *line, part);
+		g_free(oldline);
+		len = strlen(*line);
+	}
+	/* In case of error or EOF, just return the part we have.
+	 * Otherwise, strip the newline.  */
+	if ((*line)[len - 1] == '\n')
+		(*line)[len - 1] = '\0';
+	return TRUE;
 }
 
 GameParams *params_load_file(const gchar * fname)
 {
 	FILE *fp;
-	gchar line[512];
+	gchar *line;
 	GameParams *params;
 
 	if ((fp = fopen(fname, "r")) == NULL) {
@@ -356,15 +436,15 @@ GameParams *params_load_file(const gchar * fname)
 	}
 
 	params = params_new();
-	while (fgets(line, sizeof(line), fp) != NULL) {
-		gint len = strlen(line);
-
-		if (len > 0 && line[len - 1] == '\n')
-			line[len - 1] = '\0';
-		params_load_line(params, line);
+	while (read_line_from_file(&line, fp) && params) {
+		if (!params_load_line(params, line)) {
+			params_free(params);
+			params = NULL;
+		}
+		g_free(line);
 	}
 	fclose(fp);
-	if (!params_load_finish(params)) {
+	if (params && !params_load_finish(params)) {
 		params_free(params);
 		return NULL;
 	}
@@ -373,9 +453,19 @@ GameParams *params_load_file(const gchar * fname)
 
 GameParams *params_copy(const GameParams * params)
 {
+	/* Copy the const parameter to a non-const version, because
+	 * G_STRUCT_MEMBER doesn't want const values.  Note that this
+	 * variable does not own its pointers, that is, they don't have to
+	 * be freed when it goes out of scope. */
+	GameParams nonconst;
 	GameParams *copy;
 	gint idx;
+	gchar *buff;
 
+	if (params == NULL)
+		return NULL;
+
+	memcpy(&nonconst, params, sizeof(GameParams));
 	copy = params_new();
 	copy->map = map_copy(params->map);
 	copy->variant = params->variant;
@@ -388,21 +478,37 @@ GameParams *params_copy(const GameParams * params)
 			G_STRUCT_MEMBER(gchar *, copy, param->offset)
 			    =
 			    g_strdup(G_STRUCT_MEMBER
-				     (gchar *, params, param->offset));
+				     (gchar *, &nonconst, param->offset));
 			break;
 		case PARAM_INT:
 			G_STRUCT_MEMBER(gint, copy, param->offset)
-			    = G_STRUCT_MEMBER(gint, params, param->offset);
+			    = G_STRUCT_MEMBER(gint, &nonconst,
+					      param->offset);
 			break;
 		case PARAM_BOOL:
 			G_STRUCT_MEMBER(gboolean, copy, param->offset)
-			    = G_STRUCT_MEMBER(gboolean, params,
+			    = G_STRUCT_MEMBER(gboolean, &nonconst,
 					      param->offset);
 			break;
+		case PARAM_INTLIST:
+			buff =
+			    format_int_list("",
+					    G_STRUCT_MEMBER(GArray *,
+							    &nonconst,
+							    param->
+							    offset));
+			if (buff != NULL) {
+				G_STRUCT_MEMBER(GArray *, copy,
+						param->offset) =
+				    build_int_list(buff);
+				g_free(buff);
+			}
+			break;
 		}
 	}
 
 	copy->quit_when_done = params->quit_when_done;
+	copy->tournament_time = params->tournament_time;
 	return copy;
 }
 
@@ -455,3 +561,17 @@ gboolean params_write_file(GameParams * params, const gchar * fname)
 
 	return TRUE;
 }
+
+Points *points_new(gint id, const gchar * name, gint points)
+{
+	Points *p = g_malloc0(sizeof(Points));
+	p->id = id;
+	p->name = g_strdup(name);
+	p->points = points;
+	return p;
+}
+
+void points_free(Points * points)
+{
+	g_free(points->name);
+}
