@@ -3,7 +3,8 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
- * 
+ * Copyright (C) 2006 Roland Clobus <rclobus@bigfoot.com>
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -22,6 +23,7 @@
 #ifndef __game_h
 #define __game_h
 
+#include <stdio.h>
 #include "map.h"
 #include "driver.h"
 
@@ -65,9 +67,10 @@ typedef struct {
 	gint num_develop_type[NUM_DEVEL_TYPES];	/* number of each development */
 	Map *map;		/* the game map */
 	gboolean parsing_map;	/* currently parsing map? *//* Not in game_params[] */
-	gint tournament_time;	/* time to start tournament time in minutes */
+	gint tournament_time;	/* time to start tournament time in minutes *//* Not in game_params[] */
 	gboolean quit_when_done;	/* server quits after someone wins *//* Not in game_params[] */
 	gboolean use_pirate;	/* is there a pirate in this game? */
+	GArray *island_discovery_bonus;	/* list of VPs for discovering an island */
 } GameParams;
 
 typedef struct {
@@ -85,6 +88,11 @@ void params_free(GameParams * params);
 void params_write_lines(GameParams * params, gboolean write_secrets,
 			WriteLineFunc func, gpointer user_data);
 gboolean params_write_file(GameParams * params, const gchar * fname);
-void params_load_line(GameParams * params, gchar * line);
+gboolean params_load_line(GameParams * params, gchar * line);
 gboolean params_load_finish(GameParams * params);
+
+gboolean read_line_from_file(gchar ** line, FILE * f);
+
+Points *points_new(gint id, const gchar * name, gint points);
+void points_free(Points * points);
 #endif
