@@ -168,12 +168,10 @@ void map_move_pirate(Map * map, gint x, gint y);
 
 Map *map_new(void);
 Map *map_copy(Map * map);
-void map_format_line(Map * map, gboolean write_secrets, gchar * line,
-		     gint y);
-void map_parse_line(Map * map, char *line);
-void map_parse_finish(Map * map);
+gchar *map_format_line(Map * map, gboolean write_secrets, gint y);
+gboolean map_parse_line(Map * map, const gchar * line);
+gboolean map_parse_finish(Map * map);
 void map_free(Map * map);
-Map *map_load(char *name);
 Hex *map_add_hex(Map * map, gint x, gint y);
 
 /* map_query.c
@@ -226,6 +224,7 @@ gboolean map_bridge_connect_ok(Map * map, gint owner, gint x, gint y,
 			       gint pos);
 /* information gathering */
 void map_longest_road(Map * map, gint * lengths, gint num_players);
+gboolean map_is_island_discovered(Map * map, Node * node, gint owner);
 void map_maritime_info(Map * map, MaritimeInfo * info, gint owner);
 
 extern GRand *g_rand_ctx;
