@@ -2,8 +2,9 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
- * 
+ * Copyright (C) 2003, 2006 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2005,2006 Roland Clobus <rclobus@bigfoot.com>
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -47,6 +48,7 @@
 #include <stdlib.h>		/* For atoi */
 #endif				/* ndef HAVE_GETADDRINFO_ET_AL */
 
+#include <time.h>
 #include "config.h"
 #include "driver.h"
 #include "game.h"
@@ -62,20 +64,40 @@ typedef union {
 #endif				/* HAVE_GETADDRINFO_ET_AL */
 } sockaddr_t;
 
-#ifdef DEBUG
+static gboolean debug_enabled = FALSE;
+
+/* Number of seconds between pings
+ * (in the absence of other network activity).  */
+static const int PING_PERIOD = 30;
+
+void set_enable_debug(gboolean enabled)
+{
+	debug_enabled = enabled;
+}
+
 void debug(const gchar * fmt, ...)
 {
 	va_list ap;
-	gchar buff[16 * 1024];
-	gint len;
+	gchar *buff;
 	gint idx;
+	time_t t;
+	struct tm *alpha;
+
+	if (!debug_enabled)
+		return;
 
 	va_start(ap, fmt);
-	len = g_vsnprintf(buff, sizeof(buff), fmt, ap);
+	buff = g_strdup_vprintf(fmt, ap);
 	va_end(ap);
 
-	for (idx = 0; idx < len; idx++) {
-		if (isprint(buff[idx]) || idx == len - 1)
+	t = time(NULL);
+	alpha = localtime(&t);
+
+	g_print("%02d:%02d:%02d ", alpha->tm_hour,
+		alpha->tm_min, alpha->tm_sec);
+
+	for (idx = 0; buff[idx] != '\0'; idx++) {
+		if (isprint(buff[idx]))
 			g_print("%c", buff[idx]);
 		else
 			switch (buff[idx]) {
@@ -89,12 +111,12 @@ void debug(const gchar * fmt, ...)
 				g_print("\\t");
 				break;
 			default:
-				g_print("\\x%02x", buff[idx]);
+				g_print("\\x%02x", (buff[idx] & 0xff));
 				break;
 			}
 	}
+	g_print("\n");
 }
-#endif
 
 static void read_ready(Session * ses);
 static void write_ready(Session * ses);
@@ -132,10 +154,15 @@ static void notify(Session * ses, NetEvent event, gchar * line)
 
 void net_close(Session * ses)
 {
+	if (ses->timer_id != 0) {
+		g_source_remove(ses->timer_id);
+		ses->timer_id = 0;
+	}
+
 	if (ses->fd >= 0) {
 		listen_read(ses, FALSE);
 		listen_write(ses, FALSE);
-		close(ses->fd);
+		net_closesocket(ses->fd);
 		ses->fd = -1;
 
 		while (ses->write_queue != NULL) {
@@ -169,6 +196,36 @@ static void close_and_callback(Session * ses)
 	notify(ses, NET_CLOSE, NULL);
 }
 
+static gboolean ping_function(gpointer s)
+{
+	Session *ses = (Session *) s;
+	double interval = difftime(time(NULL), ses->last_response);
+	/* Ask for activity every PING_PERIOD seconds, but don't ask if there
+	 * was activity anyway.  */
+	if (interval >= 2 * PING_PERIOD) {
+		/* There was no response to the ping in time.  The connection
+		 * should be considered dead.  */
+		log_message(MSG_ERROR,
+			    "No activity and no response to ping.  Closing connection");
+		close_and_callback(ses);
+	} else if (interval >= PING_PERIOD) {
+		/* There was no activity.
+		 * Send a ping (but don't update activity time).  */
+		net_write(ses, "hello\n");
+		ses->timer_id =
+		    g_timeout_add(PING_PERIOD * 1000, ping_function, s);
+	} else {
+		/* Everything is fine.  Reschedule this check.  */
+		ses->timer_id =
+		    g_timeout_add((PING_PERIOD - interval) * 1000,
+				  ping_function, s);
+	}
+	/* Return FALSE to not reschedule this timeout.  If it needed to be
+	 * rescheduled, it has been done explicitly above (with a different
+	 * timeout).  */
+	return FALSE;
+}
+
 static void write_ready(Session * ses)
 {
 	if (!ses || ses->fd < 0)
@@ -211,10 +268,8 @@ static void write_ready(Session * ses)
 		int len = strlen(data);
 
 		num = send(ses->fd, data, len, 0);
-#ifdef DEBUG
-		debug("write_ready: write(%d, \"%.*s\", %d) = %d\n",
+		debug("write_ready: write(%d, \"%.*s\", %d) = %d",
 		      ses->fd, len, data, len, num);
-#endif
 		if (num < 0) {
 			if (errno == EAGAIN)
 				break;
@@ -261,13 +316,11 @@ void net_write(Session * ses, const gchar * data)
 
 		len = strlen(data);
 		num = send(ses->fd, data, len, 0);
-#ifdef DEBUG
 		if (num > 0)
-			debug("(%d) --> %s\n", ses->fd, data);
+			debug("(%d) --> %s", ses->fd, data);
 		else if (errno != EAGAIN)
-			debug("(%d) --- Error writing to socket.\n",
+			debug("(%d) --- Error writing to socket.",
 			      ses->fd);
-#endif
 		if (num < 0) {
 			if (errno != EAGAIN) {
 				log_message(MSG_ERROR,
@@ -289,14 +342,15 @@ void net_write(Session * ses, const gchar * data)
 
 void net_printf(Session * ses, const gchar * fmt, ...)
 {
-	char buff[4096];
+	char *buff;
 	va_list ap;
 
 	va_start(ap, fmt);
-	g_vsnprintf(buff, sizeof(buff), fmt, ap);
+	buff = g_strdup_vprintf(fmt, ap);
 	va_end(ap);
 
 	net_write(ses, buff);
+	g_free(buff);
 }
 
 static int find_line(char *buff, int len)
@@ -314,6 +368,9 @@ static void read_ready(Session * ses)
 	int num;
 	int offset;
 
+	/* There is data from this connection: record the time.  */
+	ses->last_response = time(NULL);
+
 	if (ses->read_len == sizeof(ses->read_buff)) {
 		/* We are in trouble now - the application has not
 		 * been processing the data we have been
@@ -358,9 +415,16 @@ static void read_ready(Session * ses)
 		line[len] = '\0';
 		offset += len + 1;
 
-#ifdef DEBUG
-		debug("(%d) <-- %s\n", ses->fd, line);
-#endif
+		debug("(%d) <-- %s", ses->fd, line);
+
+		if (!strcmp(line, "hello")) {
+			net_write(ses, "yes\n");
+			continue;
+		}
+		if (!strcmp(line, "yes")) {
+			continue;	/* Don't notify the program */
+		}
+
 		notify(ses, NET_READ, line);
 	}
 
@@ -392,9 +456,14 @@ Session *net_new(NetNotifyFunc notify_func, void *user_data)
 	return ses;
 }
 
-void net_use_fd(Session * ses, int fd)
+void net_use_fd(Session * ses, int fd, gboolean do_ping)
 {
 	ses->fd = fd;
+	if (do_ping) {
+		ses->last_response = time(NULL);
+		ses->timer_id =
+		    g_timeout_add(PING_PERIOD * 1000, ping_function, ses);
+	}
 	listen_read(ses, TRUE);
 }
 
@@ -495,7 +564,7 @@ gboolean net_connect(Session * ses, const gchar * host, const gchar * port)
 				    _
 				    ("Error setting socket close-on-exec: %s\n"),
 				    g_strerror(errno));
-			close(ses->fd);
+			net_closesocket(ses->fd);
 			ses->fd = -1;
 			continue;
 		}
@@ -505,7 +574,7 @@ gboolean net_connect(Session * ses, const gchar * host, const gchar * port)
 				    _
 				    ("Error setting socket non-blocking: %s\n"),
 				    g_strerror(errno));
-			close(ses->fd);
+			net_closesocket(ses->fd);
 			ses->fd = -1;
 			continue;
 		}
@@ -530,7 +599,7 @@ gboolean net_connect(Session * ses, const gchar * host, const gchar * port)
 					    _
 					    ("Error connecting to %s: %s\n"),
 					    host, g_strerror(errno));
-				close(ses->fd);
+				net_closesocket(ses->fd);
 				ses->fd = -1;
 				continue;
 			}
@@ -637,11 +706,11 @@ int net_open_listening_socket(const gchar * port, gchar ** error_message)
 		if (setsockopt
 		    (fd, SOL_SOCKET, SO_REUSEADDR, &yes,
 		     sizeof(yes)) < 0) {
-			close(fd);
+			net_closesocket(fd);
 			continue;
 		}
 		if (bind(fd, aip->ai_addr, aip->ai_addrlen) < 0) {
-			close(fd);
+			net_closesocket(fd);
 			continue;
 		}
 
@@ -664,7 +733,7 @@ int net_open_listening_socket(const gchar * port, gchar ** error_message)
 		    g_strdup_printf(_
 				    ("Error setting socket non-blocking: %s\n"),
 				    g_strerror(errno));
-		close(fd);
+		net_closesocket(fd);
 		return -1;
 	}
 
@@ -673,7 +742,7 @@ int net_open_listening_socket(const gchar * port, gchar ** error_message)
 		    g_strdup_printf(_
 				    ("Error during listen on socket: %s\n"),
 				    g_strerror(errno));
-		close(fd);
+		net_closesocket(fd);
 		return -1;
 	}
 	*error_message = NULL;
@@ -685,6 +754,15 @@ int net_open_listening_socket(const gchar * port, gchar ** error_message)
 #endif				/* HAVE_GETADDRINFO_ET_AL */
 }
 
+void net_closesocket(int fd)
+{
+#ifdef G_OS_WIN32
+	closesocket(fd);
+#else				/* G_OS_WIN32 */
+	close(fd);
+#endif				/* G_OS_WIN32 */
+}
+
 gboolean net_get_peer_name(gint fd, gchar ** hostname, gchar ** servname,
 			   gchar ** error_message)
 {
@@ -757,7 +835,7 @@ void net_init(void)
 	wVersionRequested = MAKEWORD(2, 2);
 
 	if (0 != WSAStartup(wVersionRequested, &wsaData)) {
-		g_error(_("No usable version of WinSock was found."));
+		g_error("No usable version of WinSock was found.");
 	}
 #else				/* G_OS_WIN32 */
 	/* Do nothing on unix like platforms */
