@@ -2,7 +2,7 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003, 2006 Bas Wijnen <shevek@fmf.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 #define __network_h
 
 #include <glib.h>
+#include <time.h>
 
 typedef enum {
 	NET_CONNECT,
@@ -37,6 +38,8 @@ typedef void (*NetNotifyFunc) (NetEvent event, void *user_data,
 typedef struct _Session Session;
 struct _Session {
 	int fd;
+	time_t last_response;	/* used for activity detection.  */
+	guint timer_id;
 	void *user_data;
 
 	gboolean connect_in_progress;
@@ -54,9 +57,8 @@ struct _Session {
 	NetNotifyFunc notify_func;
 };
 
-#ifdef DEBUG
+void set_enable_debug(gboolean enabled);
 void debug(const gchar * fmt, ...);
-#endif
 
 /** Initialize the network drivers */
 void net_init(void);
@@ -67,7 +69,7 @@ void net_finish(void);
 Session *net_new(NetNotifyFunc notify_func, void *user_data);
 void net_free(Session ** ses);
 
-void net_use_fd(Session * ses, int fd);
+void net_use_fd(Session * ses, int fd, gboolean do_ping);
 gboolean net_connect(Session * ses, const gchar * host,
 		     const gchar * port);
 gboolean net_connected(Session * ses);
@@ -80,6 +82,10 @@ gboolean net_connected(Session * ses);
  */
 int net_open_listening_socket(const gchar * port, gchar ** error_message);
 
+/** Close a socket
+ */
+void net_closesocket(int fd);
+
 /** Get peer name
  *  @param fd File descriptor to resolve
  *  @retval hostname The resolved hostname
