@@ -2,7 +2,7 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003, 2006 Bas Wijnen <shevek@fmf.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,8 @@
  * [fmt] format string is modelled on the printf format string.  It
  * understands the following types:
  *	%S - string from current position to end of line
- *		this takes two arguments: the gchar * and the buffer size
+ *		this takes a gchar ** argument, in which an allocated buffer
+ *		is returned.  It must be freed by the caller.
  *	%d - integer
  *	%B - build type;
  *		'road' = BUILD_ROAD,
@@ -93,9 +94,8 @@
  * 	Pop a number of states off the stack and set a new current state
  * 	accordingly.
  *
- * sm_pop_all()
- *	Clear the state stack, set the current state to the state that
- *	was on the bottom of the stack.
+ * sm_pop_all_and_goto(new_state)
+ *	Clear the state stack, set the current state to [new_state]
  *
  * sm_stack_inspect()
  *	Return the state at offset from the top of the stack.
@@ -108,7 +108,8 @@ typedef enum {
 
 	SM_ENTER,
 	SM_INIT,
-	SM_RECV
+	SM_RECV,
+	SM_FREE
 } EventType;
 
 typedef struct StateMachine StateMachine;
@@ -151,7 +152,7 @@ void sm_state_name(StateMachine * sm, const gchar * name);
 gboolean sm_recv(StateMachine * sm, const gchar * fmt, ...);
 gboolean sm_recv_prefix(StateMachine * sm, const gchar * fmt, ...);
 void sm_cancel_prefix(StateMachine * sm);
-void sm_vnformat(gchar * buff, gint len, const gchar * fmt, va_list ap);
+gchar *sm_vformat(const gchar * fmt, va_list ap);
 void sm_write(StateMachine * sm, const gchar * str);
 void sm_send(StateMachine * sm, const gchar * fmt, ...);
 /** Cache the messages that are sent.
@@ -174,7 +175,6 @@ void sm_push(StateMachine * sm, StateFunc new_state);
 void sm_push_noenter(StateMachine * sm, StateFunc new_state);
 void sm_pop(StateMachine * sm);
 void sm_multipop(StateMachine * sm, gint depth);
-void sm_pop_all(StateMachine * sm);
 void sm_pop_all_and_goto(StateMachine * sm, StateFunc new_state);
 StateFunc sm_current(StateMachine * sm);
 StateFunc sm_stack_inspect(const StateMachine * sm, guint offset);
@@ -184,7 +184,7 @@ void sm_unhandled_set(StateMachine * sm, StateFunc state);
 gboolean sm_is_connected(StateMachine * sm);
 gboolean sm_connect(StateMachine * sm, const gchar * host,
 		    const gchar * port);
-void sm_use_fd(StateMachine * sm, gint fd);
+void sm_use_fd(StateMachine * sm, gint fd, gboolean do_ping);
 void sm_dec_use_count(StateMachine * sm);
 void sm_inc_use_count(StateMachine * sm);
 /** Dump the stack */
