@@ -1,11 +1,29 @@
+/* Pioneers - Implementation of the excellent Settlers of Catan board game.
+ *   Go buy a copy.
+ *
+ * Copyright (C) 2005 Brian Wellington
+ * Copyright (C) 2005,2006 Roland Clobus
+ * Copyright (C) 2005,2006 Bas Wijnen
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 #include "config.h"
+#include "version.h"
 
 #include <string.h>
-
-#ifdef HAVE_LIBGNOME
-#include <libgnome/libgnome.h>
-#endif
-
+#include "authors.h"
 #include "aboutbox.h"
 #include "config-gnome.h"
 #include "game.h"
@@ -35,6 +53,7 @@ static GameBuildings *game_buildings;
 static GameResources *game_resources;
 static GtkWidget *terrain_menu;
 static GtkWidget *roll_menu;
+static GtkWidget *roll_numbers[12 + 1];
 static GtkCheckMenuItem *shuffle_tile;
 static GtkWidget *port_menu;
 static GtkWidget *port_directions[6];
@@ -44,25 +63,41 @@ static GuiMap *gmap;
 static Hex *current_hex;
 
 static const gchar *terrain_names[] = {
-	N_("Hill"),
-	N_("Field"),
-	N_("Mountain"),
-	N_("Pasture"),
-	N_("Forest"),
-	N_("Desert"),
-	N_("Sea"),
-	N_("Gold"),
-	N_("None")
+	/* Use an unique shortcut key for each resource */
+	N_("_Hill"),
+	/* Use an unique shortcut key for each resource */
+	N_("_Field"),
+	/* Use an unique shortcut key for each resource */
+	N_("_Mountain"),
+	/* Use an unique shortcut key for each resource */
+	N_("_Pasture"),
+	/* Use an unique shortcut key for each resource */
+	N_("F_orest"),
+	/* Use an unique shortcut key for each resource */
+	N_("_Desert"),
+	/* Use an unique shortcut key for each resource */
+	N_("_Sea"),
+	/* Use an unique shortcut key for each resource */
+	N_("_Gold"),
+	/* Use an unique shortcut key for each resource */
+	N_("_None")
 };
 
 static const gchar *port_names[] = {
-	N_("Brick (2:1)"),
-	N_("Grain (2:1)"),
-	N_("Ore (2:1)"),
-	N_("Wool (2:1)"),
-	N_("Lumber (2:1)"),
-	N_("None"),
-	N_("Any (3:1)")
+	/* Use an unique shortcut key for each port type */
+	N_("_Brick (2:1)"),
+	/* Use an unique shortcut key for each port type */
+	N_("_Grain (2:1)"),
+	/* Use an unique shortcut key for each port type */
+	N_("_Ore (2:1)"),
+	/* Use an unique shortcut key for each port type */
+	N_("_Wool (2:1)"),
+	/* Use an unique shortcut key for each port type */
+	N_("_Lumber (2:1)"),
+	/* Use an unique shortcut key for each port type */
+	N_("_None"),
+	/* Use an unique shortcut key for each port type */
+	N_("_Any (3:1)")
 };
 
 static const gchar *port_direction_names[] = {
@@ -83,11 +118,11 @@ static const gchar *port_direction_names[] = {
 static void error_dialog(const char *fmt, ...)
 {
 	GtkWidget *dialog;
-	gchar buf[1024];
+	gchar *buf;
 	va_list ap;
 
 	va_start(ap, fmt);
-	g_vsnprintf(buf, sizeof(buf), fmt, ap);
+	buf = g_strdup_vprintf(fmt, ap);
 	va_end(ap);
 
 	dialog = gtk_message_dialog_new(GTK_WINDOW(toplevel),
@@ -95,6 +130,7 @@ static void error_dialog(const char *fmt, ...)
 					GTK_DIALOG_DESTROY_WITH_PARENT,
 					GTK_MESSAGE_ERROR,
 					GTK_BUTTONS_CLOSE, "%s", buf);
+	g_free(buf);
 	gtk_dialog_run(GTK_DIALOG(dialog));
 	gtk_widget_destroy(dialog);
 }
@@ -129,8 +165,10 @@ static void canonicalize_map(Map * map)
 	GArray *chits;
 	Hex *hex;
 	gint x, y;
+	gint sequence_number;
 
 	chits = g_array_new(FALSE, FALSE, sizeof(gint));
+	sequence_number = 0;
 	for (y = 0; y < map->y_size; y++) {
 		for (x = 0; x < map->x_size; x++) {
 			hex = map->grid[y][x];
@@ -138,7 +176,9 @@ static void canonicalize_map(Map * map)
 				continue;
 			if (hex->roll > 0) {
 				g_array_append_val(chits, hex->roll);
-				hex->chit_pos = chits->len - 1;
+				hex->chit_pos = sequence_number++;
+			} else if (hex->terrain == DESERT_TERRAIN) {
+				hex->chit_pos = sequence_number++;
 			}
 		}
 	}
@@ -256,6 +296,12 @@ static gint button_press_map_cb(GtkWidget * area, GdkEventButton * event,
 		menu = terrain_menu;
 	else if (current_hex->roll > 0) {
 		menu = roll_menu;
+		for (i = 2; i <= 12; i++) {
+			if (roll_numbers[i] != NULL)
+				gtk_check_menu_item_set_active
+				    (GTK_CHECK_MENU_ITEM(roll_numbers[i]),
+				     current_hex->roll == i);
+		}
 		gtk_check_menu_item_set_active(shuffle_tile,
 					       current_hex->shuffle);
 	} else if (current_hex->terrain == SEA_TERRAIN) {
@@ -273,6 +319,12 @@ static gint button_press_map_cb(GtkWidget * area, GdkEventButton * event,
 			}
 			gtk_widget_set_sensitive(port_directions[i],
 						 port_ok);
+			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
+						       (port_directions
+							[i]),
+						       current_hex->
+						       facing == i
+						       && port_ok);
 		}
 
 		if (num_ports > 0)
@@ -385,6 +437,7 @@ static GtkWidget *build_map(void)
 	table = gtk_table_new(2, 2, FALSE);
 
 	gmap = guimap_new();
+	guimap_set_show_no_setup_nodes(gmap, TRUE);
 	area = guimap_build_drawingarea(gmap, MAP_WIDTH, MAP_HEIGHT);
 
 	GTK_WIDGET_SET_FLAGS(area, GTK_CAN_FOCUS);
@@ -462,8 +515,9 @@ static GtkWidget *build_terrain_menu(void)
 
 	for (i = 0; i <= LAST_TERRAIN; i++) {
 		item =
-		    gtk_image_menu_item_new_with_label(gettext
-						       (terrain_names[i]));
+		    gtk_image_menu_item_new_with_mnemonic(gettext
+							  (terrain_names
+							   [i]));
 
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		g_signal_connect(G_OBJECT(item), "activate",
@@ -483,20 +537,19 @@ static GtkWidget *build_terrain_menu(void)
 	return menu;
 }
 
-static gint select_roll_cb(G_GNUC_UNUSED GtkWidget * menu,
+static void select_roll_cb(GtkCheckMenuItem * menu_item,
 			   gpointer user_data)
 {
-	current_hex->roll = GPOINTER_TO_INT(user_data);
-	guimap_draw_hex(gmap, current_hex);
-	return TRUE;
+	if (gtk_check_menu_item_get_active(menu_item)) {
+		current_hex->roll = GPOINTER_TO_INT(user_data);
+		guimap_draw_hex(gmap, current_hex);
+	}
 }
 
-static gint select_shuffle_cb(G_GNUC_UNUSED GtkWidget * menu,
+static void select_shuffle_cb(GtkCheckMenuItem * menu_item,
 			      G_GNUC_UNUSED gpointer user_data)
 {
-	current_hex->shuffle =
-	    gtk_check_menu_item_get_active(shuffle_tile);
-	return TRUE;
+	current_hex->shuffle = gtk_check_menu_item_get_active(menu_item);
 }
 
 static GtkWidget *build_roll_menu(void)
@@ -522,15 +575,18 @@ static GtkWidget *build_roll_menu(void)
 			"<span foreground=\"#%04x%04x%04x\">%d</span>",
 			color->red, color->green, color->blue, i);
 
-		item = gtk_menu_item_new();
+		item = gtk_check_menu_item_new();
 		label = gtk_label_new("");
 		gtk_label_set_markup(GTK_LABEL(label), buffer);
 		gtk_container_add(GTK_CONTAINER(item), label);
 
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-		g_signal_connect(G_OBJECT(item), "activate",
+		g_signal_connect(G_OBJECT(item), "toggled",
 				 G_CALLBACK(select_roll_cb),
 				 GINT_TO_POINTER(i));
+		gtk_check_menu_item_set_draw_as_radio(GTK_CHECK_MENU_ITEM
+						      (item), TRUE);
+		roll_numbers[i] = item;
 	}
 
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu),
@@ -538,7 +594,7 @@ static GtkWidget *build_roll_menu(void)
 
 	item = gtk_check_menu_item_new_with_label(_("Shuffle"));
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-	g_signal_connect(G_OBJECT(item), "activate",
+	g_signal_connect(G_OBJECT(item), "toggled",
 			 G_CALLBACK(select_shuffle_cb), NULL);
 	shuffle_tile = GTK_CHECK_MENU_ITEM(item);
 
@@ -570,12 +626,13 @@ static gint select_port_resource_cb(G_GNUC_UNUSED GtkWidget * menu,
 	return TRUE;
 }
 
-static gint select_port_direction_cb(G_GNUC_UNUSED GtkWidget * menu,
+static void select_port_direction_cb(GtkCheckMenuItem * menu_item,
 				     gpointer user_data)
 {
-	current_hex->facing = GPOINTER_TO_INT(user_data);
-	guimap_draw_hex(gmap, current_hex);
-	return TRUE;
+	if (gtk_check_menu_item_get_active(menu_item)) {
+		current_hex->facing = GPOINTER_TO_INT(user_data);
+		guimap_draw_hex(gmap, current_hex);
+	}
 }
 
 static GtkWidget *build_port_menu(void)
@@ -591,8 +648,8 @@ static GtkWidget *build_port_menu(void)
 
 	for (i = 0; i <= ANY_RESOURCE; i++) {
 		item =
-		    gtk_image_menu_item_new_with_label(gettext
-						       (port_names[i]));
+		    gtk_image_menu_item_new_with_mnemonic(gettext
+							  (port_names[i]));
 
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		g_signal_connect(G_OBJECT(item), "activate",
@@ -611,12 +668,13 @@ static GtkWidget *build_port_menu(void)
 			      gtk_separator_menu_item_new());
 	for (i = 0; i < 6; i++) {
 		item =
-		    gtk_menu_item_new_with_label(Q_
-						 (port_direction_names
-						  [i]));
-
+		    gtk_check_menu_item_new_with_label(Q_
+						       (port_direction_names
+							[i]));
+		gtk_check_menu_item_set_draw_as_radio(GTK_CHECK_MENU_ITEM
+						      (item), TRUE);
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-		g_signal_connect(G_OBJECT(item), "activate",
+		g_signal_connect(G_OBJECT(item), "toggled",
 				 G_CALLBACK(select_port_direction_cb),
 				 GINT_TO_POINTER(i));
 		port_directions[i] = item;
@@ -679,17 +737,17 @@ static GtkWidget *build_settings(void)
 
 static void set_window_title(const gchar * title)
 {
-	gchar str[100];
+	gchar *str;
 	g_free(window_title);
 	if (title == NULL) {
 		title = "Untitled";
 		window_title = NULL;
 	} else
 		window_title = g_strdup(title);
-	g_snprintf(str, sizeof(str), "%s: %s", _("Pioneers Editor"),
-		   title);
+	str = g_strdup_printf("%s: %s", _("Pioneers Editor"), title);
 
 	gtk_window_set_title(GTK_WINDOW(toplevel), str);
+	g_free(str);
 }
 
 static void apply_params(GameParams * params)
@@ -792,6 +850,7 @@ static void load_game(const gchar * file, gboolean is_reload)
 		new_filename = NULL;
 	} else {
 		new_filename = g_strdup(file);
+		config_set_string("editor/last-game", new_filename);
 	}
 
 	guimap_reset(gmap);
@@ -816,6 +875,8 @@ static void save_game(const gchar * file)
 	canonicalize_map(gmap->map);
 	if (!params_write_file(params, file))
 		error_dialog(_("Failed to save to '%s'"), file);
+	else
+		config_set_string("editor/last-game", file);
 	fill_map(gmap->map);
 }
 
@@ -955,8 +1016,7 @@ static void contents_menu_cb(void)
 static void about_menu_cb(void)
 {
 	const gchar *authors[] = {
-		"Brian Wellington",
-		NULL
+		AUTHORLIST
 	};
 
 	aboutbox_display(_("Pioneers Game Editor"), authors);
@@ -1017,9 +1077,26 @@ static const char *ui_description =
 "</ui>";
 /* *INDENT-ON* */
 
+gchar **filenames;
+gboolean show_version = FALSE;
+
+static GOptionEntry commandline_entries[] = {
+	{G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY,
+	 &filenames,
+	 /* Long help for commandline option (editor): filename */
+	 N_("Open this file"),
+	 /* Commandline option for editor: filename */
+	 N_("filename")},
+	{"version", '\0', 0, G_OPTION_ARG_NONE, &show_version,
+	 /* Commandline option of editor: version */
+	 N_("Show version information"), NULL},
+	{NULL, '\0', 0, 0, NULL, NULL, NULL}
+};
+
 int main(int argc, char *argv[])
 {
-	const gchar *filename;
+	gchar *filename;
+	gboolean default_used;
 	GtkWidget *notebook;
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
@@ -1028,27 +1105,42 @@ int main(int argc, char *argv[])
 	GtkAccelGroup *accel_group;
 	GError *error = NULL;
 	gchar *icon_file;
-
-	if (argc > 1)
-		filename = argv[1];
-	else
-		filename = NULL;
+	GOptionContext *context;
 
 	default_game = g_build_filename(get_pioneers_dir(), "default.game",
 					NULL);
 
-#ifdef HAVE_LIBGNOME
-	gnome_program_init("pioneers-editor", VERSION, LIBGNOME_MODULE,
-			   argc, argv, GNOME_PARAM_POPT_TABLE, NULL, NULL);
-#endif
-
-	gtk_init(&argc, &argv);
-
 	/* Gtk+ handles the locale, we must bind the translations */
+	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 
+	/* Long description in the command line: --help */
+	context =
+	    g_option_context_new(_("- Editor for games of Pioneers"));
+	g_option_context_add_main_entries(context, commandline_entries,
+					  GETTEXT_PACKAGE);
+	g_option_context_add_group(context, gtk_get_option_group(TRUE));
+	g_option_context_parse(context, &argc, &argv, &error);
+	if (error != NULL) {
+		g_print("%s\n", error->message);
+		g_error_free(error);
+		return 1;
+	}
+	if (show_version) {
+		g_print(_("Pioneers version:"));
+		g_print(" ");
+		g_print(FULL_VERSION);
+		g_print("\n");
+		return 0;
+	}
+
+	if (filenames != NULL)
+		filename = g_strdup(filenames[0]);
+	else
+		filename = NULL;
+
 	toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	g_signal_connect(G_OBJECT(toplevel), "delete_event",
 			 G_CALLBACK(exit_cb), NULL);
@@ -1080,7 +1172,7 @@ int main(int argc, char *argv[])
 		gtk_window_set_default_icon_from_file(icon_file, NULL);
 	} else {
 		/* Missing pixmap, main icon file */
-		g_warning(_("Pixmap not found: %s\n"), icon_file);
+		g_warning("Pixmap not found: %s", icon_file);
 	}
 	g_free(icon_file);
 
@@ -1107,7 +1199,18 @@ int main(int argc, char *argv[])
 	gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
 
+	if (filename == NULL) {
+		filename =
+		    config_get_string("editor/last-game", &default_used);
+		if (default_used
+		    || !g_file_test(filename, G_FILE_TEST_EXISTS)) {
+			g_free(filename);
+			filename = NULL;
+		}
+	}
+
 	load_game(filename, FALSE);
+	g_free(filename);
 	if (params == NULL)
 		return 1;
 
@@ -1116,5 +1219,7 @@ int main(int argc, char *argv[])
 	gtk_main();
 
 	config_finish();
+
+	g_option_context_free(context);
 	return 0;
 }
