@@ -2,7 +2,7 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2003, 2006 Bas Wijnen <shevek@fmf.nl>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,9 +26,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif
 #include <glib.h>
 #include <signal.h>
 
@@ -56,6 +53,7 @@ void admin_run_command(Session * admin_session, const gchar * line)
 	gint value_int;
 	static gchar *server_port = NULL;
 	static gboolean register_server = TRUE;
+	static GameParams *params = NULL;
 
 	/* parse the line down into command and value */
 	sscanf(line, "admin %99s %99s", command, value_str);
@@ -73,12 +71,14 @@ void admin_run_command(Session * admin_session, const gchar * line)
 
 		/* start the server */
 	} else if (!strcmp(command, "start-server")) {
+		gchar *meta_server_name = get_meta_server_name(TRUE);
 		if (server_is_running())
 			server_stop();
 		if (!server_port)
 			server_port = g_strdup(PIONEERS_DEFAULT_GAME_PORT);
-		start_server(get_server_name(), server_port,
-			     register_server);
+		start_server(params, get_server_name(), server_port,
+			     register_server, meta_server_name, TRUE);
+		g_free(meta_server_name);
 
 	} else if (!strcmp(command, "stop-server")) {
 		server_stop();
@@ -96,7 +96,7 @@ void admin_run_command(Session * admin_session, const gchar * line)
 		if (value_int) {
 			if (server_is_running())
 				server_stop();
-			cfg_set_num_players(value_int);
+			cfg_set_num_players(params, value_int);
 		}
 
 		/* set the sevens rule */
@@ -104,7 +104,7 @@ void admin_run_command(Session * admin_session, const gchar * line)
 		if (value_int) {
 			if (server_is_running())
 				server_stop();
-			cfg_set_sevens_rule(value_int);
+			cfg_set_sevens_rule(params, value_int);
 		}
 
 		/* set the victory points */
@@ -112,7 +112,7 @@ void admin_run_command(Session * admin_session, const gchar * line)
 		if (value_int) {
 			if (server_is_running())
 				server_stop();
-			cfg_set_victory_points(value_int);
+			cfg_set_victory_points(params, value_int);
 		}
 
 		/* set whether to use random terrain */
@@ -120,7 +120,7 @@ void admin_run_command(Session * admin_session, const gchar * line)
 		if (value_int) {
 			if (server_is_running())
 				server_stop();
-			cfg_set_terrain_type(value_int);
+			cfg_set_terrain_type(params, value_int);
 		}
 
 		/* set the game type (by name) */
@@ -128,7 +128,9 @@ void admin_run_command(Session * admin_session, const gchar * line)
 		if (value_str) {
 			if (server_is_running())
 				server_stop();
-			cfg_set_game(value_str);
+			if (params)
+				params_free(params);
+			params = cfg_set_game(value_str);
 		}
 
 		/* request to close the connection */
@@ -200,6 +202,7 @@ void admin_connect(comm_info * admin_info)
 {
 	Session *admin_session;
 	gint new_fd;
+	gchar *location;
 
 	/* somebody connected to the administration port, so we... */
 
@@ -212,21 +215,29 @@ void admin_connect(comm_info * admin_info)
 	admin_session->user_data = admin_session;
 
 	/* (3) accept the connection into a new file descriptor */
-	new_fd = accept_connection(admin_info->fd, NULL);
+	new_fd = accept_connection(admin_info->fd, &location);
 
-	/* (4) tie the new file descriptor to the session we created earlier */
-	net_use_fd(admin_session, new_fd);
+	/* (4) tie the new file descriptor to the session we created earlier.
+	 * Don't use keepalive pings on this connection.  */
+	net_use_fd(admin_session, new_fd, FALSE);
 }
 
 /* set up the administration port */
 void admin_listen(const gchar * port)
 {
+	gchar *error_message;
+
 	if (!_accept_info) {
 		_accept_info = g_malloc0(sizeof(comm_info));
 	}
 
 	/* open up a socket on which to listen for connections */
-	_accept_info->fd = open_listen_socket(port);
+	_accept_info->fd = net_open_listening_socket(port, &error_message);
+	if (_accept_info->fd == -1) {
+		log_message(MSG_ERROR, "%s\n", error_message);
+		g_free(error_message);
+		return;
+	}
 #ifdef PRINT_INFO
 	g_print("admin_listen: fd = %d\n", _accept_info->fd);
 #endif
