@@ -3,7 +3,7 @@
  *
  * Copyright (C) 1999 Dave Cole
  * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
- * Copyright (C) 2004-2005 Roland Clobus <rclobus@bigfoot.com>
+ * Copyright (C) 2004-2006 Roland Clobus <rclobus@bigfoot.com>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,13 +21,12 @@
  */
 
 #include "config.h"
-#ifdef HAVE_LIBGNOME
-#include <libgnome/libgnome.h>
-#endif
+#include "version.h"
 #include <ctype.h>
 #include <gtk/gtk.h>
 #include <string.h>
 
+#include "authors.h"
 #include "aboutbox.h"
 #include "game.h"
 #include "common_gtk.h"
@@ -40,14 +39,16 @@
 
 #define MAINICON_FILE	"pioneers-server.png"
 
+static GtkWidget *settings_notebook;	/* relevant settings */
 static GtkWidget *game_frame;	/* the frame containing all settings regarding the game */
 static GtkWidget *select_game;	/* select game type */
 static GtkWidget *game_settings;	/* the settings of the game */
 static GtkWidget *server_frame;	/* the frame containing all settings regarding the server */
+static GtkWidget *ai_frame;	/* the frame containing all settings regarding the ai */
 static GtkWidget *register_toggle;	/* register with meta server? */
 static GtkWidget *chat_toggle;	/* disable AI chatting? */
 static GtkWidget *meta_entry;	/* name of meta server */
-static GtkWidget *hostname_entry;	/* name of server (allows masquerading) */
+static GtkWidget *overridden_hostname_entry;	/* name of server (allows masquerading) */
 static GtkWidget *port_entry;	/* server port */
 static GtkWidget *random_toggle;	/* randomize seating order? */
 static GtkWidget *addcomputer_btn;	/* button to add computer players */
@@ -56,13 +57,17 @@ static GtkWidget *start_btn;	/* start/stop the server */
 static GtkTooltips *tooltips;	/* tooltips */
 
 static GtkListStore *store;	/* shows player connection status */
+static gboolean is_running;	/* current server status */
 
 
-static gboolean ui_enabled;	/* is the ui accessible? */
-static gchar *hostname;		/* reported hostname */
+static gchar *overridden_hostname;	/* override reported hostname */
 static gchar *server_port = NULL;	/* port of the game */
 static gboolean register_server = TRUE;	/* Register at the meta server */
+static const gchar *meta_server_name = NULL;	/* hostname of the meta server */
 static gboolean want_ai_chat = TRUE;
+static gboolean random_order = TRUE;	/* random seating order */
+static gboolean enable_debug = FALSE;
+static gboolean show_version = FALSE;
 
 /* Local function prototypes */
 static void add_game_to_list(gpointer name, gpointer user_data);
@@ -122,7 +127,8 @@ static void register_toggle_cb(GtkToggleButton * toggle,
 	gtk_label_set_text(GTK_LABEL(label),
 			   register_server ? _("Yes") : _("No"));
 	gtk_widget_set_sensitive(meta_entry, register_server);
-	gtk_widget_set_sensitive(hostname_entry, register_server);
+	gtk_widget_set_sensitive(overridden_hostname_entry,
+				 register_server);
 }
 
 static void random_toggle_cb(GtkToggleButton * toggle,
@@ -158,9 +164,9 @@ static void game_settings_change_cb(GameSettings *gs, G_GNUC_UNUSED gpointer use
 }
 */
 
-static void update_game_settings(void)
+static void update_game_settings(const GameParams * params)
 {
-	g_assert(params != NULL);
+	g_return_if_fail(params != NULL);
 
 	/* Update the UI */
 	game_settings_set_terrain(GAMESETTINGS(game_settings),
@@ -171,79 +177,91 @@ static void update_game_settings(void)
 					 params->victory_points);
 	game_settings_set_sevens_rule(GAMESETTINGS(game_settings),
 				      params->sevens_rule);
-
-	/* Update the global structure */
-	cfg_set_terrain_type(params->random_terrain);
-	cfg_set_num_players(params->num_players);
-	cfg_set_victory_points(params->victory_points);
-	cfg_set_sevens_rule(params->sevens_rule);
 }
 
 static void game_activate(GtkWidget * widget,
 			  G_GNUC_UNUSED gpointer user_data)
 {
 	const gchar *title;
+	const GameParams *params;
 
 	title = select_game_get_active(SELECTGAME(widget));
 	params = game_list_find_item(title);
-	update_game_settings();
+	update_game_settings(params);
 }
 
-static void gui_ui_enable(gboolean sensitive)
+static void gui_set_server_state(gboolean running)
 {
-	gboolean addcomputer_btn_enabled = !sensitive;
+	gboolean ai_settings_enabled = TRUE;
+	gchar *fullname;
 
-	ui_enabled = sensitive;
+	is_running = running;
 
-	gtk_widget_set_sensitive(game_frame, sensitive);
-	gtk_widget_set_sensitive(server_frame, sensitive);
-	gtk_button_set_label(GTK_BUTTON(start_btn), ui_enabled ?
-			     _("Start server") : _("Stop server"));
-	gtk_tooltips_set_tip(tooltips, start_btn, ui_enabled ?
-			     _("Start the server") : _("Stop the server"),
-			     NULL);
+	gtk_widget_set_sensitive(gtk_notebook_get_nth_page
+				 (GTK_NOTEBOOK(settings_notebook), 0),
+				 !running);
+	if (running)
+		gtk_widget_show(gtk_notebook_get_nth_page
+				(GTK_NOTEBOOK(settings_notebook), 1));
+	else
+		gtk_widget_hide(gtk_notebook_get_nth_page
+				(GTK_NOTEBOOK(settings_notebook), 1));
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(settings_notebook),
+				      running ? 1 : 0);
+	gtk_button_set_label(GTK_BUTTON(start_btn),
+			     running ? _("Stop server") :
+			     _("Start server"));
+	gtk_tooltips_set_tip(tooltips, start_btn,
+			     running ? _("Stop the server") :
+			     _("Start the server"), NULL);
 
-	if (addcomputer_btn_enabled) {
-		gchar *fullname = g_find_program_in_path(PIONEERS_AI_PATH);
-		if (fullname) {
-			g_free(fullname);
-		} else {
-			addcomputer_btn_enabled = FALSE;
-		}
+	fullname = g_find_program_in_path(PIONEERS_AI_PATH);
+	if (fullname) {
+		g_free(fullname);
+	} else {
+		ai_settings_enabled = FALSE;
 	}
-	gtk_widget_set_sensitive(addcomputer_btn, addcomputer_btn_enabled);
+	gtk_widget_set_sensitive(ai_frame, ai_settings_enabled);
 }
 
 static void start_clicked_cb(G_GNUC_UNUSED GtkButton * start_btn,
 			     G_GNUC_UNUSED gpointer user_data)
 {
-	if (ui_enabled) {
+	if (is_running) {
+		if (server_stop())
+			gui_set_server_state(FALSE);
+	} else {		/* not running */
 		const gchar *title;
+		GameParams *params;
+
 		title = select_game_get_active(SELECTGAME(select_game));
-		params = game_list_find_item(title);
-		params->random_terrain =
-		    game_settings_get_terrain(GAMESETTINGS(game_settings));
-		params->num_players =
-		    game_settings_get_players(GAMESETTINGS(game_settings));
-		params->victory_points =
-		    game_settings_get_victory_points(GAMESETTINGS
-						     (game_settings));
-		params->sevens_rule =
-		    game_settings_get_sevens_rule(GAMESETTINGS
-						  (game_settings));
-		update_game_settings();
+		params = params_copy(game_list_find_item(title));
+		cfg_set_terrain_type(params,
+				     game_settings_get_terrain(GAMESETTINGS
+							       (game_settings)));
+		cfg_set_num_players(params,
+				    game_settings_get_players(GAMESETTINGS
+							      (game_settings)));
+		cfg_set_victory_points(params,
+				       game_settings_get_victory_points
+				       (GAMESETTINGS(game_settings)));
+		cfg_set_sevens_rule(params,
+				    game_settings_get_sevens_rule
+				    (GAMESETTINGS(game_settings)));
+		update_game_settings(params);
 		g_assert(server_port != NULL);
-		if (start_server(hostname, server_port, register_server)) {
-			gui_ui_enable(FALSE);
+		if (start_server
+		    (params, overridden_hostname, server_port,
+		     register_server, meta_server_name, random_order)) {
+			gui_set_server_state(TRUE);
 			config_set_string("server/meta-server",
 					  meta_server_name);
 			config_set_string("server/port", server_port);
 			config_set_int("server/register", register_server);
-			config_set_string("server/hostname", hostname);
+			config_set_string("server/overridden-hostname",
+					  overridden_hostname);
 			config_set_int("server/random-seating-order",
 				       random_order);
-			config_set_int("server/enable-ai-chat",
-				       want_ai_chat);
 
 			config_set_string("game/name", params->title);
 			config_set_int("game/random-terrain",
@@ -255,9 +273,7 @@ static void start_clicked_cb(G_GNUC_UNUSED GtkButton * start_btn,
 			config_set_int("game/sevens-rule",
 				       params->sevens_rule);
 		}
-	} else {		/* UI was not enabled */
-		if (server_stop())
-			gui_ui_enable(TRUE);
+		params_free(params);
 	}
 }
 
@@ -266,6 +282,7 @@ static void addcomputer_clicked_cb(G_GNUC_UNUSED GtkButton * start_btn,
 {
 	g_assert(server_port != NULL);
 	new_computer_player(NULL, server_port, want_ai_chat);
+	config_set_int("ai/enable-chat", want_ai_chat);
 }
 
 
@@ -293,7 +310,7 @@ static void gui_player_rename(void *data)
 static gboolean everybody_left(G_GNUC_UNUSED gpointer data)
 {
 	if (server_stop())
-		gui_ui_enable(TRUE);
+		gui_set_server_state(FALSE);
 	return FALSE;
 }
 
@@ -337,17 +354,18 @@ static void add_game_to_list(gpointer name,
 	select_game_add(SELECTGAME(select_game), a->title);
 }
 
-static void hostname_changed_cb(GtkEntry * widget,
-				G_GNUC_UNUSED gpointer user_data)
+static void overridden_hostname_changed_cb(GtkEntry * widget,
+					   G_GNUC_UNUSED gpointer
+					   user_data)
 {
 	const gchar *text;
 
 	text = gtk_entry_get_text(widget);
 	while (*text != '\0' && isspace(*text))
 		text++;
-	if (hostname)
-		g_free(hostname);
-	hostname = g_strdup(text);
+	if (overridden_hostname)
+		g_free(overridden_hostname);
+	overridden_hostname = g_strdup(text);
 }
 
 static void meta_server_changed_cb(GtkWidget * widget,
@@ -368,6 +386,7 @@ static GtkWidget *build_game_settings(GtkWidget * parent)
 	gboolean default_returned;
 	gchar *gamename;
 	gint temp;
+	GameParams *params;
 
 	tooltips = gtk_tooltips_new();
 
@@ -398,6 +417,10 @@ static GtkWidget *build_game_settings(GtkWidget * parent)
 	/* Fill the GUI with the saved settings */
 	gamename =
 	    config_get_string("game/name=Default", &default_returned);
+	load_game_types(get_pioneers_dir());
+	params = cfg_set_game(gamename);
+	if (params == NULL)
+		params = cfg_set_game("Default");
 	select_game_set_default(SELECTGAME(select_game), gamename);
 	game_list_foreach(add_game_to_list, NULL);
 	g_free(gamename);
@@ -407,18 +430,19 @@ static GtkWidget *build_game_settings(GtkWidget * parent)
 	g_assert(params != NULL);
 	temp = config_get_int("game/random-terrain", &default_returned);
 	if (!default_returned)
-		params->random_terrain = temp;
+		cfg_set_terrain_type(params, temp);
 	temp = config_get_int("game/num-players", &default_returned);
 	if (!default_returned)
-		params->num_players = temp;
+		cfg_set_num_players(params, temp);
 	temp = config_get_int("game/victory-points", &default_returned);
 	if (!default_returned)
-		params->victory_points = temp;
+		cfg_set_victory_points(params, temp);
 	temp = config_get_int("game/sevens-rule", &default_returned);
 	if (!default_returned)
-		params->sevens_rule = temp;
+		cfg_set_sevens_rule(params, temp);
 
-	update_game_settings();
+	update_game_settings(params);
+	params_free(params);
 	return frame;
 }
 
@@ -444,7 +468,8 @@ static GtkWidget *build_interface(void)
 {
 	GtkWidget *vbox;
 	GtkWidget *vbox_settings;
-	GtkWidget *hbox;
+	GtkWidget *vbox_ai;
+	GtkWidget *hbox_ai;
 	GtkWidget *frame;
 	GtkWidget *table;
 	GtkWidget *label;
@@ -458,17 +483,22 @@ static GtkWidget *build_interface(void)
 
 	tooltips = gtk_tooltips_new();
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_vbox_new(FALSE, 5);
 	gtk_widget_show(vbox);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 
-	hbox = gtk_hbox_new(FALSE, 5);
-	gtk_widget_show(hbox);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
+	settings_notebook = gtk_notebook_new();
+	gtk_widget_show(settings_notebook);
+	gtk_notebook_set_show_border(GTK_NOTEBOOK(settings_notebook),
+				     FALSE);
+	gtk_box_pack_start(GTK_BOX(vbox), settings_notebook, FALSE, TRUE,
+			   0);
 
 	vbox_settings = gtk_vbox_new(FALSE, 5);
 	gtk_widget_show(vbox_settings);
-	gtk_box_pack_start(GTK_BOX(hbox), vbox_settings, FALSE, TRUE, 0);
+	gtk_notebook_append_page(GTK_NOTEBOOK(settings_notebook),
+				 vbox_settings,
+				 gtk_label_new(_("Game Settings")));
 
 	/* Game settings frame */
 	game_frame = build_game_settings(vbox_settings);
@@ -537,13 +567,13 @@ static GtkWidget *build_interface(void)
 			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
-	hostname_entry = gtk_entry_new();
-	g_signal_connect(G_OBJECT(hostname_entry), "changed",
-			 G_CALLBACK(hostname_changed_cb), NULL);
-	gtk_widget_show(hostname_entry);
-	gtk_table_attach(GTK_TABLE(table), hostname_entry, 1, 2, 3, 4,
-			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-	gtk_tooltips_set_tip(tooltips, hostname_entry,
+	overridden_hostname_entry = gtk_entry_new();
+	g_signal_connect(G_OBJECT(overridden_hostname_entry), "changed",
+			 G_CALLBACK(overridden_hostname_changed_cb), NULL);
+	gtk_widget_show(overridden_hostname_entry);
+	gtk_table_attach(GTK_TABLE(table), overridden_hostname_entry, 1, 2,
+			 3, 4, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	gtk_tooltips_set_tip(tooltips, overridden_hostname_entry,
 			     _
 			     ("The public name of this computer (needed when playing behind a firewall)"),
 			     NULL);
@@ -563,21 +593,6 @@ static GtkWidget *build_interface(void)
 	gtk_tooltips_set_tip(tooltips, random_toggle,
 			     _("Randomize turn order"), NULL);
 
-	label = gtk_label_new(_("Enable AI Chat"));
-	gtk_widget_show(label);
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
-			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-
-	chat_toggle = gtk_toggle_button_new_with_label(_("No"));
-	gtk_widget_show(chat_toggle);
-	gtk_table_attach(GTK_TABLE(table), chat_toggle, 1, 2, 5, 6,
-			 GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-	g_signal_connect(G_OBJECT(chat_toggle), "toggled",
-			 G_CALLBACK(chat_toggle_cb), NULL);
-	gtk_tooltips_set_tip(tooltips, chat_toggle,
-			     _("Enable AI chat messages"), NULL);
-
 	/* Initialize server-settings */
 	server_port = config_get_string("server/port="
 					PIONEERS_DEFAULT_GAME_PORT,
@@ -593,10 +608,12 @@ static GtkWidget *build_interface(void)
 	gtk_entry_set_text(GTK_ENTRY(meta_entry), meta_server_name);
 
 	novar = 0;
-	hostname = config_get_string("server/hostname", &novar);
-	if (novar || !strlen(hostname))
-		hostname = get_server_name();
-	gtk_entry_set_text(GTK_ENTRY(hostname_entry), hostname);
+	overridden_hostname =
+	    config_get_string("server/overridden-hostname", &novar);
+	if (novar)
+		overridden_hostname = g_strdup("");
+	gtk_entry_set_text(GTK_ENTRY(overridden_hostname_entry),
+			   overridden_hostname);
 
 	register_server =
 	    config_get_int_with_default("server/register", TRUE);
@@ -610,33 +627,14 @@ static GtkWidget *build_interface(void)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(random_toggle),
 				     random_order);
 
-	want_ai_chat = config_get_int_with_default("server/enable-ai-chat",
-						   TRUE);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chat_toggle),
-				     want_ai_chat);
-
-	start_btn = gtk_button_new();
-	gtk_widget_show(start_btn);
-
-	gtk_box_pack_start(GTK_BOX(vbox_settings), start_btn, FALSE, FALSE,
-			   0);
-	g_signal_connect(G_OBJECT(start_btn), "clicked",
-			 G_CALLBACK(start_clicked_cb), NULL);
-
-	addcomputer_btn =
-	    gtk_button_new_with_label(_("Add Computer Player"));
-	gtk_widget_show(addcomputer_btn);
-	gtk_box_pack_start(GTK_BOX(vbox_settings), addcomputer_btn, FALSE,
-			   FALSE, 0);
-	g_signal_connect(G_OBJECT(addcomputer_btn), "clicked",
-			 G_CALLBACK(addcomputer_clicked_cb), NULL);
-	gtk_widget_set_sensitive(addcomputer_btn, FALSE);
-	gtk_tooltips_set_tip(tooltips, addcomputer_btn,
-			     _("Add a computer player to the game"), NULL);
+	vbox_settings = gtk_vbox_new(FALSE, 5);
+	gtk_notebook_append_page(GTK_NOTEBOOK(settings_notebook),
+				 vbox_settings,
+				 gtk_label_new(_("Running Game")));
 
 	frame = gtk_frame_new(_("Players Connected"));
 	gtk_widget_show(frame);
-	gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox_settings), frame, TRUE, TRUE, 0);
 
 	scroll_win = gtk_scrolled_window_new(NULL, NULL);
 	gtk_widget_show(scroll_win);
@@ -721,6 +719,52 @@ static GtkWidget *build_interface(void)
 
 	gtk_widget_show(label);
 
+	ai_frame = gtk_frame_new(_("Computer Players"));
+	gtk_widget_show(ai_frame);
+	gtk_box_pack_start(GTK_BOX(vbox_settings), ai_frame, FALSE, FALSE,
+			   0);
+	vbox_ai = gtk_vbox_new(FALSE, 5);
+	gtk_widget_show(vbox_ai);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox_ai), 5);
+	gtk_container_add(GTK_CONTAINER(ai_frame), vbox_ai);
+	hbox_ai = gtk_hbox_new(FALSE, 5);
+	gtk_widget_show(hbox_ai);
+	gtk_box_pack_start(GTK_BOX(vbox_ai), hbox_ai, FALSE, FALSE, 0);
+
+	label = gtk_label_new(_("Enable Chat"));
+	gtk_widget_show(label);
+	gtk_box_pack_start(GTK_BOX(hbox_ai), label, FALSE, FALSE, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+
+	chat_toggle = gtk_toggle_button_new_with_label(_("No"));
+	gtk_widget_show(chat_toggle);
+	gtk_box_pack_start(GTK_BOX(hbox_ai), chat_toggle, TRUE, TRUE, 0);
+	g_signal_connect(G_OBJECT(chat_toggle), "toggled",
+			 G_CALLBACK(chat_toggle_cb), NULL);
+	gtk_tooltips_set_tip(tooltips, chat_toggle,
+			     _("Enable chat messages"), NULL);
+
+	want_ai_chat = config_get_int_with_default("ai/enable-chat", TRUE);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chat_toggle),
+				     want_ai_chat);
+
+	addcomputer_btn =
+	    gtk_button_new_with_label(_("Add Computer Player"));
+	gtk_widget_show(addcomputer_btn);
+	gtk_box_pack_start(GTK_BOX(vbox_ai), addcomputer_btn, FALSE, FALSE,
+			   0);
+	g_signal_connect(G_OBJECT(addcomputer_btn), "clicked",
+			 G_CALLBACK(addcomputer_clicked_cb), NULL);
+	gtk_tooltips_set_tip(tooltips, addcomputer_btn,
+			     _("Add a computer player to the game"), NULL);
+
+	start_btn = gtk_button_new();
+	gtk_widget_show(start_btn);
+
+	gtk_box_pack_start(GTK_BOX(vbox), start_btn, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(start_btn), "clicked",
+			 G_CALLBACK(start_clicked_cb), NULL);
+
 	frame = gtk_frame_new(_("Messages"));
 	gtk_widget_show(frame);
 	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
@@ -742,7 +786,7 @@ static GtkWidget *build_interface(void)
 			     _("Messages from the server"), NULL);
 	message_window_set_text(message_text);
 
-	gui_ui_enable(TRUE);
+	gui_set_server_state(FALSE);
 	return vbox;
 }
 
@@ -754,8 +798,7 @@ static void quit_cb(void)
 static void help_about_cb(void)
 {
 	const gchar *authors[] = {
-		"Dave Cole",
-		NULL
+		AUTHORLIST
 	};
 	aboutbox_display(_("The Pioneers Game Server"), authors);
 }
@@ -768,6 +811,16 @@ void game_is_over(G_GNUC_UNUSED Game * game)
 	log_message(MSG_INFO, _("The game is over.\n"));
 }
 
+static GOptionEntry commandline_entries[] = {
+	{"debug", '\0', 0, G_OPTION_ARG_NONE, &enable_debug,
+	 /* Commandline option of server-gtk: enable debug logging */
+	 N_("Enable debug messages"), NULL},
+	{"version", '\0', 0, G_OPTION_ARG_NONE, &show_version,
+	 /* Commandline option of server-gtk: version */
+	 N_("Show version information"), NULL},
+	{NULL, '\0', 0, 0, NULL, NULL, NULL}
+};
+
 int main(int argc, char *argv[])
 {
 	gchar *icon_file;
@@ -778,6 +831,7 @@ int main(int argc, char *argv[])
 	GtkUIManager *ui_manager;
 	GtkAccelGroup *accel_group;
 	GError *error = NULL;
+	GOptionContext *context;
 
 	net_init();
 
@@ -793,27 +847,35 @@ int main(int argc, char *argv[])
 	/* Initialize frontend inspecific things */
 	server_init();
 
-#ifdef HAVE_LIBGNOME
-	/* @todo RC 2005-04-10 If the client does not need libgnomeui
-	 * anymore, perhaps the gnome_program_init call could be moved
-	 * to config-gnome.c, which would allow a GNOME-free application,
-	 * for architectures that don't have GNOME libraries.
-	 */
-	gnome_program_init("pioneers-server", VERSION,
-			   LIBGNOME_MODULE,
-			   argc, argv, GNOME_PARAM_POPT_TABLE, NULL, NULL);
-#endif
-
-	/* Initialize the widget set */
-	gtk_init(&argc, &argv);
-
 #ifdef ENABLE_NLS
+	setlocale(LC_ALL, "");
 	/* Gtk+ handles the locale, we must bind the translations */
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
 #endif
 
+	/* Long description in the commandline for server-gtk: help */
+	context = g_option_context_new(_("- Host a game of Pioneers"));
+	g_option_context_add_main_entries(context, commandline_entries,
+					  PACKAGE);
+	g_option_context_add_group(context, gtk_get_option_group(TRUE));
+	g_option_context_parse(context, &argc, &argv, &error);
+	if (error != NULL) {
+		g_print("%s\n", error->message);
+		g_error_free(error);
+		return 1;
+	}
+	if (show_version) {
+		g_print(_("Pioneers version:"));
+		g_print(" ");
+		g_print(FULL_VERSION);
+		g_print("\n");
+		return 0;
+	}
+
+	set_enable_debug(enable_debug);
+
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	/* Name in the titlebar of the server */
 	gtk_window_set_title(GTK_WINDOW(window), _("Pioneers Server"));
@@ -847,7 +909,7 @@ int main(int argc, char *argv[])
 	if (g_file_test(icon_file, G_FILE_TEST_EXISTS)) {
 		gtk_window_set_default_icon_from_file(icon_file, NULL);
 	} else {
-		g_warning(_("Pixmap not found: %s\n"), icon_file);
+		g_warning("Pixmap not found: %s", icon_file);
 	}
 	g_free(icon_file);
 
@@ -858,6 +920,7 @@ int main(int argc, char *argv[])
 			   0);
 
 	gtk_widget_show_all(window);
+	gui_set_server_state(FALSE);
 	g_signal_connect(G_OBJECT(window), "delete_event",
 			 G_CALLBACK(quit_cb), NULL);
 
@@ -869,5 +932,6 @@ int main(int argc, char *argv[])
 
 	config_finish();
 	net_finish();
+	g_option_context_free(context);
 	return 0;
 }
