@@ -2,8 +2,9 @@
  *   Go buy a copy.
  *
  * Copyright (C) 1999 Dave Cole
- * Copyright (C) 2003 Bas Wijnen <shevek@fmf.nl>
- * 
+ * Copyright (C) 2003, 2006 Bas Wijnen <shevek@fmf.nl>
+ * Copyright (C) 2005,2006 Roland Clobus <rclobus@bigfoot.com>
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -29,7 +30,6 @@
 static gboolean mode_check_version(Player * player, gint event);
 static gboolean mode_check_status(Player * player, gint event);
 static gboolean mode_bad_version(Player * player, gint event);
-static gboolean mode_game_is_over(Player * player, gint event);
 static gboolean mode_global(Player * player, gint event);
 static gboolean mode_unhandled(Player * player, gint event);
 static void player_setup(Player * player, int playernum,
@@ -90,9 +90,27 @@ static gboolean mode_global(Player * player, gint event)
 {
 	StateMachine *sm = player->sm;
 	Game *game = player->game;
-	gchar text[512];
+	gchar *text;
 
 	switch (event) {
+	case SM_FREE:
+		if (player->name != NULL)
+			g_free(player->name);
+		if (player->location != NULL)
+			g_free(player->location);
+		if (player->client_version != NULL)
+			g_free(player->client_version);
+		if (player->devel != NULL)
+			deck_free(player->devel);
+		if (player->num >= 0
+		    && !player_is_viewer(game, player->num)) {
+			game->num_players--;
+			meta_report_num_players(game->num_players);
+		}
+		g_list_free(player->build_list);
+		g_list_free(player->special_points);
+		g_free(player);
+		return TRUE;
 	case SM_NET_CLOSE:
 		player_remove(player);
 		if (player->num >= 0) {
@@ -104,23 +122,25 @@ static gboolean mode_global(Player * player, gint event)
 		driver->player_change(game);
 		return TRUE;
 	case SM_RECV:
-		if (sm_recv(sm, "chat %S", text, sizeof(text))) {
+		if (sm_recv(sm, "chat %S", &text)) {
 			if (strlen(text) > MAX_CHAT)
 				sm_send(sm, "ERR %s\n",
 					_("chat too long"));
 			else
 				player_broadcast(player, PB_ALL,
 						 "chat %s\n", text);
+			g_free(text);
 			return TRUE;
 		}
-		if (sm_recv(sm, "name %S", text, sizeof(text))) {
-			if (text[0] == 0)
+		if (sm_recv(sm, "name %S", &text)) {
+			if (text[0] == '\0')
 				sm_send(sm, "ERR invalid-name\n");
 			else if (strlen(text) > MAX_NAME_LENGTH)
 				sm_send(sm, "ERR %s\n",
 					_("name too long"));
 			else
 				player_set_name(player, text);
+			g_free(text);
 			return TRUE;
 		}
 		break;
@@ -133,15 +153,17 @@ static gboolean mode_global(Player * player, gint event)
 static gboolean mode_unhandled(Player * player, gint event)
 {
 	StateMachine *sm = player->sm;
-	gchar text[512];
+	gchar *text;
 
 	switch (event) {
 	case SM_RECV:
-		if (sm_recv(sm, "extension %S", text, sizeof(text))) {
-			sm_send(sm, "NOTE ignoring unknown extension\n");
+		if (sm_recv(sm, "extension %S", &text)) {
+			sm_send(sm, "NOTE %s\n",
+				N_("ignoring unknown extension"));
 			log_message(MSG_INFO,
 				    "ignoring unknown extension from %s: %s\n",
 				    player->name, text);
+			g_free(text);
 			return TRUE;
 		}
 		break;
@@ -165,8 +187,8 @@ static gboolean tournament_start_cb(gpointer data)
 	if (game->num_players == game->params->num_players)
 		return FALSE;
 
-	player_broadcast(player, PB_SILENT,
-			 "NOTE Game starts, adding computer players\n");
+	player_broadcast(player, PB_SILENT, "NOTE %s\n",
+			 N_("Game starts, adding computer players"));
 	/* add computer players to start game */
 	for (i = game->num_players; i < game->params->num_players; i++) {
 		new_computer_player(NULL, game->server_port, TRUE);
@@ -183,15 +205,20 @@ static gboolean talk_about_tournament_cb(gpointer data)
 {
 	Player *player = (Player *) data;
 	Game *game = player->game;
+	const gchar *message;
 
 	/* if game already started */
 	if (game->num_players == game->params->num_players)
 		return FALSE;
 
+	/* FIXME: Use ngettext here? */
+	message = game->params->tournament_time != 1 ?
+	    N_("The game starts in %s minutes.") :
+	    N_("The game starts in %s minute.");
+
 	player_broadcast(player, PB_SILENT,
-			 "NOTE Game starts in %d minute%s\n",
-			 game->params->tournament_time,
-			 game->params->tournament_time != 1 ? "s" : "");
+			 "NOTE1 %d|%s\n",
+			 game->params->tournament_time, message);
 	game->params->tournament_time--;
 
 	if (game->params->tournament_time > 0)
@@ -218,8 +245,27 @@ Player *player_new(Game * game, int fd, gchar * location)
 	}
 	if (i == G_N_ELEMENTS(name) - 1) {
 		/* there are too many pending connections */
-		write(fd, "Too many connections\n", 21);
-		close(fd);
+		write(fd, "ERR Too many connections\n", 25);
+		net_closesocket(fd);
+		return NULL;
+	}
+
+	if (game->is_game_over) {
+		/* The game is over, don't accept new players */
+		Session *ses = net_new(NULL, NULL);
+		gchar *message;
+		net_use_fd(ses, fd, FALSE);
+		/* Message to send to the client when the game is already over
+		 * when a connection is made. */
+		message =
+		    g_strdup_printf("NOTE %s\n",
+				    N_("Sorry, game is over."));
+		net_write(ses, message);
+		log_message(MSG_INFO,
+			    _("Player from %s is refused: game is over\n"),
+			    location);
+		net_close_when_flushed(ses);
+		g_free(message);
 		return NULL;
 	}
 
@@ -228,18 +274,7 @@ Player *player_new(Game * game, int fd, gchar * location)
 
 	sm_global_set(sm, (StateFunc) mode_global);
 	sm_unhandled_set(sm, (StateFunc) mode_unhandled);
-	sm_use_fd(sm, fd);
-
-	if (game->is_game_over) {
-		/* The game is over, don't accept new players */
-		sm_goto(sm, (StateFunc) mode_game_is_over);
-		log_message(MSG_INFO,
-			    _("Player from %s is refused: game is over\n"),
-			    location);
-		sm_free(sm);
-		close(fd);
-		return NULL;
-	}
+	sm_use_fd(sm, fd, TRUE);
 
 	/* Cache messages of the game in progress until all intial 
 	 * messages have been sent
@@ -256,15 +291,24 @@ Player *player_new(Game * game, int fd, gchar * location)
 	player->gov_played = 0;
 	player->libr_played = 0;
 	player->market_played = 0;
+	player->islands_discovered = 0;
 	player->disconnected = FALSE;
 	player->name = g_strdup(name);
+	player->special_points = NULL;
+	player->special_points_next_id = 0;
 
-	/* if first player in and this is a tournament start the timer */
-	if ((game->num_players == 0)
-	    && (game->params->tournament_time > 0)) {
-		g_timeout_add(game->params->tournament_time * 60 * 1000 +
-			      500, &tournament_start_cb, player);
-		g_timeout_add(1000, &talk_about_tournament_cb, player);
+	if (game->params->tournament_time > 0) {
+		/* if first player in and this is a tournament start the timer */
+		if (game->num_players == 0) {
+			g_timeout_add(game->params->tournament_time * 60 *
+				      1000 + 500, &tournament_start_cb,
+				      player);
+			g_timeout_add(1000, &talk_about_tournament_cb,
+				      player);
+		} else {
+			sm_send_uncached(sm, "NOTE %s\n",
+					 N_("This game will start soon."));
+		}
 	}
 
 	sm_goto(sm, (StateFunc) mode_check_version);
@@ -288,7 +332,8 @@ static void player_set_name_real(Player * player, gchar * name,
 		/* make it a note, not an error, so nothing bad happens
 		 * (on error the AI would disconnect) */
 		sm_send(sm, "NOTE %s\n",
-			_("Name not changed: new name is already in use"));
+			N_
+			("Name not changed: new name is already in use"));
 		return;
 	}
 
@@ -398,21 +443,6 @@ void player_free(Player * player)
 	driver->player_change(game);
 
 	sm_free(player->sm);
-	if (player->name != NULL)
-		g_free(player->name);
-	if (player->location != NULL)
-		g_free(player->location);
-	if (player->client_version != NULL)
-		g_free(player->client_version);
-	if (player->devel != NULL)
-		deck_free(player->devel);
-	if (player->num >= 0 && !player_is_viewer(game, player->num)) {
-		game->num_players--;
-		meta_report_num_players(game->num_players);
-	}
-	g_list_free(player->build_list);
-	g_list_free(player->points);
-	g_free(player);
 }
 
 void player_archive(Player * player)
@@ -430,8 +460,10 @@ void player_archive(Player * player)
 	   machine and inform others as necessary */
 	state = sm_current(player->sm);
 	if (state == (StateFunc) mode_wait_quote_exit) {
+		/* Fake the acknowledgement */
 		sm_pop(player->sm);
 	} else if (state == (StateFunc) mode_domestic_quote) {
+		/* Retract all quotes */
 		for (;;) {
 			QuoteInfo *quote;
 			quote = quotelist_find_domestic(game->quotes,
@@ -439,12 +471,12 @@ void player_archive(Player * player)
 			if (quote == NULL)
 				break;
 			quotelist_delete(game->quotes, quote);
+			player_broadcast(player, PB_RESPOND,
+					 "domestic-quote delete %d\n",
+					 quote->var.d.quote_num);
 		}
-		player_broadcast(player, PB_RESPOND,
-				 "domestic-quote finish\n");
-
-		sm_pop(player->sm);
 	} else if (state == (StateFunc) mode_domestic_initiate) {
+		/* End the trade */
 		trade_finish_domestic(player);
 	}
 
@@ -464,6 +496,7 @@ void player_revive(Player * newp, char *name)
 	GList *current = NULL;
 	Player *p = NULL;
 	gboolean reviving_player_in_setup;
+	gchar *safe_name;
 
 	/* first see if a player with the given name exists */
 	if (name) {
@@ -523,9 +556,9 @@ void player_revive(Player * newp, char *name)
 	newp->devel = p->devel;
 	p->devel = NULL;
 
-	g_assert(newp->points == NULL);
-	newp->points = p->points;
-	p->points = NULL;	/* prevent deletion */
+	g_assert(newp->special_points == NULL);
+	newp->special_points = p->special_points;
+	p->special_points = NULL;	/* prevent deletion */
 
 	newp->discard_num = p->discard_num;
 	newp->num_roads = p->num_roads;
@@ -573,9 +606,13 @@ void player_revive(Player * newp, char *name)
 
 	player_free(p);
 
-	player_broadcast(newp, PB_SILENT,
-			 _("NOTE player %d (%s) has reconnected.\n"),
-			 newp->num, newp->name);
+	/* Make sure the name in the broadcast doesn't contain the separator */
+	safe_name = g_strdup(newp->name);
+	g_strdelimit(safe_name, "|", '_');
+	player_broadcast(newp, PB_SILENT, "NOTE1 %s|%s\n", safe_name,
+			 /* %s is the name of the reconnecting player */
+			 N_("%s has reconnected."));
+	g_free(safe_name);
 	return;
 }
 
@@ -630,19 +667,7 @@ static gboolean mode_bad_version(Player * player, gint event)
 	switch (event) {
 	case SM_ENTER:
 		sm_send_uncached(sm, "ERR sorry, version conflict\n");
-		break;
-	}
-	return FALSE;
-}
-
-static gboolean mode_game_is_over(Player * player, gint event)
-{
-	StateMachine *sm = player->sm;
-
-	sm_state_name(sm, "mode_game_is_over");
-	switch (event) {
-	case SM_ENTER:
-		sm_send(sm, "NOTE %s\n", _("Sorry, game is over."));
+		player_free(player);
 		break;
 	}
 	return FALSE;
@@ -656,7 +681,6 @@ static gboolean check_versions(gchar * client_version)
 static gboolean mode_check_version(Player * player, gint event)
 {
 	StateMachine *sm = player->sm;
-	gchar version[512];
 
 	sm_state_name(sm, "mode_check_version");
 	switch (event) {
@@ -665,24 +689,25 @@ static gboolean mode_check_version(Player * player, gint event)
 		break;
 
 	case SM_RECV:
-		if (sm_recv(sm, "version %S", version, sizeof(version))) {
-			player->client_version = g_strdup(version);
-			if (check_versions(version)) {
+		if (sm_recv(sm, "version %S", &player->client_version)) {
+			if (check_versions(player->client_version)) {
 				sm_goto(sm, (StateFunc) mode_check_status);
-				return TRUE;
 			} else {
-				/* Version mismatch. 
-				 * %1 = expected version, 
-				 * %2 = version of the client. 
-				 * Don't translate the NOTE at 
-				 * the beginning of the text */
-				sm_send(sm,
-					_
-					("NOTE Expecting version %s, not %s\n"),
-					PROTOCOL_VERSION, version);
+				gchar *mismatch =
+				    g_strdup_printf("%s <-> %s",
+						    PROTOCOL_VERSION,
+						    player->
+						    client_version);
+				/* Make sure the argument does not contain the separator */
+				g_strdelimit(mismatch, "|", '_');
+				sm_send_uncached(sm, "NOTE1 %s|%s\n",
+						 mismatch,
+						 N_
+						 ("Version mismatch: %s"));
+				g_free(mismatch);
 				sm_goto(sm, (StateFunc) mode_bad_version);
-				return TRUE;
 			}
+			return TRUE;
 		}
 		break;
 	default:
@@ -694,7 +719,7 @@ static gboolean mode_check_version(Player * player, gint event)
 static gboolean mode_check_status(Player * player, gint event)
 {
 	StateMachine *sm = player->sm;
-	gchar playername[512];
+	gchar *playername;
 
 	sm_state_name(sm, "mode_check_status");
 	switch (event) {
@@ -707,19 +732,19 @@ static gboolean mode_check_status(Player * player, gint event)
 			player_setup(player, -1, NULL, FALSE);
 			return TRUE;
 		}
-		if (sm_recv(sm, "status reconnect %S", playername,
-			    sizeof(playername))) {
+		if (sm_recv(sm, "status reconnect %S", &playername)) {
 			/* if possible, try to revive the player */
 			player_revive(player, playername);
+			g_free(playername);
 			return TRUE;
 		}
 		if (sm_recv(sm, "status newviewer")) {
 			player_setup(player, -1, NULL, TRUE);
 			return TRUE;
 		}
-		if (sm_recv(sm, "status viewer %S", playername,
-			    sizeof(playername))) {
+		if (sm_recv(sm, "status viewer %S", &playername)) {
 			player_setup(player, -1, playername, TRUE);
+			g_free(playername);
 			return TRUE;
 		}
 		break;
@@ -846,12 +871,12 @@ void player_broadcast(Player * player, BroadcastType type, const char *fmt,
 		      ...)
 {
 	Game *game = player->game;
-	char buff[4096];
+	gchar *buff;
 	GList *list;
 	va_list ap;
 
 	va_start(ap, fmt);
-	sm_vnformat(buff, sizeof(buff), fmt, ap);
+	buff = sm_vformat(fmt, ap);
 	va_end(ap);
 
 	playerlist_inc_use_count(game);
@@ -868,6 +893,7 @@ void player_broadcast(Player * player, BroadcastType type, const char *fmt,
 				buff);
 	}
 	playerlist_dec_use_count(game);
+	g_free(buff);
 }
 
 void player_set_name(Player * player, gchar * name)
