@@ -24,6 +24,7 @@
 #include <string.h>
 #include "buildrec.h"
 #include "server.h"
+#include "version.h"
 
 static void build_add(Player * player, BuildType type, gint x, gint y,
 		      gint pos)
@@ -119,7 +120,7 @@ static void build_add(Player * player, BuildType type, gint x, gint y,
 		sm_send(sm, "ERR bad-pos\n");
 		return;
 	}
-	node_add(player, BUILD_SETTLEMENT, x, y, pos, FALSE);
+	node_add(player, BUILD_SETTLEMENT, x, y, pos, FALSE, NULL);
 }
 
 static void build_remove(Player * player)
@@ -350,6 +351,8 @@ static void send_player_list(StateMachine * sm, Player * player)
 		if (player == scan || scan->num < 0)
 			continue;
 		sm_send(sm, "player %d is %s\n", scan->num, scan->name);
+		if (scan->disconnected)
+			sm_send(sm, "player %d has quit\n", scan->num);
 	}
 	sm_send(sm, ".\n");
 }
@@ -424,12 +427,11 @@ gboolean send_gameinfo(Map * map, Hex * hex, StateMachine * sm)
  */
 gboolean mode_pre_game(Player * player, gint event)
 {
-	static gboolean old_player_disconnected;
 	StateMachine *sm = player->sm;
 	Game *game = player->game;
 	Map *map = game->params->map;
 	StateFunc state;
-	gchar prevstate[40];
+	const gchar *prevstate;
 	gint i;
 	GList *next;
 	gint longestroadpnum = -1;
@@ -450,12 +452,12 @@ gboolean mode_pre_game(Player * player, gint event)
 	case SM_ENTER:
 		sm_send(sm,
 			"player %d of %d, welcome to pioneers server %s\n",
-			player->num, game->params->num_players, VERSION);
+			player->num, game->params->num_players,
+			FULL_VERSION);
 		/* Tell the player that he exists.  This is not done in
 		 * player_set_name, because at that point the client doesn't
 		 * know how many players are in the game, and therefore if
-		 * he is a player of a viewer. */
-		old_player_disconnected = player->disconnected;
+		 * he is a player or a viewer. */
 		/* Tell the other players about this player */
 		player_broadcast(player, PB_OTHERS, "is %s\n",
 				 player->name);
@@ -506,7 +508,7 @@ gboolean mode_pre_game(Player * player, gint event)
 			if (game->bought_develop) {
 				sm_send(sm, "bought develop\n");
 			}
-			if (old_player_disconnected) {
+			if (player->disconnected) {
 				sm_send(sm, "player disconnected\n");
 			}
 			stack_offset = 1;
@@ -521,36 +523,38 @@ gboolean mode_pre_game(Player * player, gint event)
 			}
 
 			if (state == (StateFunc) mode_idle)
-				strcpy(prevstate, "IDLE");
+				prevstate = "IDLE";
 			else if (state == (StateFunc) mode_turn)
-				strcpy(prevstate, "TURN");
+				prevstate = "TURN";
 			else if (state ==
 				 (StateFunc) mode_discard_resources)
-				strcpy(prevstate, "DISCARD");
+				prevstate = "DISCARD";
 			else if (state == (StateFunc)
 				 mode_wait_for_other_discarding_players)
-				strcpy(prevstate, "DISCARD");
+				prevstate = "DISCARD";
 			else if (state == (StateFunc) mode_place_robber)
-				strcpy(prevstate, "YOUAREROBBER");
+				prevstate = "YOUAREROBBER";
 			else if (state == (StateFunc) mode_road_building)
-				strcpy(prevstate, "ROADBUILDING");
+				prevstate = "ROADBUILDING";
 			else if (state == (StateFunc) mode_monopoly)
-				strcpy(prevstate, "MONOPOLY");
+				prevstate = "MONOPOLY";
 			else if (state ==
 				 (StateFunc) mode_plenty_resources) {
 				recover_from_plenty = TRUE;
-				strcpy(prevstate, "PLENTY");
+				prevstate = "PLENTY";
 			} else if (state == (StateFunc) mode_setup) {
 				if (game->double_setup)
-					strcpy(prevstate, "SETUPDOUBLE");
+					prevstate = "SETUPDOUBLE";
+				else if (game->reverse_setup)
+					prevstate = "RSETUP";
 				else
-					strcpy(prevstate, "SETUP");
+					prevstate = "SETUP";
 				/* If player is selecting gold, the state 
 				 * should be IDLE instead */
 				if (stack_offset != 1)
-					strcpy(prevstate, "IDLE");
+					prevstate = "IDLE";
 			} else
-				strcpy(prevstate, "PREGAME");
+				prevstate = "PREGAME";
 
 			sm_send(sm, "state %s\n", prevstate);
 
@@ -572,6 +576,8 @@ gboolean mode_pre_game(Player * player, gint event)
 			   who has longest road/army */
 			/* Only send this when the player is not a viewer */
 			if (!player_is_viewer(game, player->num)) {
+				GList *list;
+
 				sm_send(sm, "playerinfo: resources: %R\n",
 					player->assets);
 				sm_send(sm,
@@ -602,11 +608,24 @@ gboolean mode_pre_game(Player * player, gint event)
 					player->market_played,
 					(player->num == longestroadpnum),
 					(player->num == largestarmypnum));
+
+				/* Send special points */
+				list = player->special_points;
+				while (list) {
+					Points *points = list->data;
+					sm_send(sm,
+						"get-point %d %d %d %s\n",
+						player->num, points->id,
+						points->points,
+						points->name);
+					list = g_list_next(list);
+				}
 			}
 			/* send info about other players */
 			for (next = player_first_real(game); next != NULL;
 			     next = player_next_real(next)) {
 				Player *p = (Player *) next->data;
+				GList *list;
 				gint numassets = 0;
 				if (p->num == player->num)
 					continue;
@@ -622,6 +641,18 @@ gboolean mode_pre_game(Player * player, gint event)
 					p->libr_played, p->market_played,
 					(p->num == longestroadpnum),
 					(p->num == largestarmypnum));
+
+				/* Send special points */
+				list = p->special_points;
+				while (list) {
+					Points *points = list->data;
+					sm_send(sm,
+						"get-point %d %d %d %s\n",
+						p->num, points->id,
+						points->points,
+						points->name);
+					list = g_list_next(list);
+				}
 			}
 
 			/* send build info for the current player
@@ -643,10 +674,20 @@ gboolean mode_pre_game(Player * player, gint event)
 			sm_send(sm, "OK\n");
 
 			/* Some player was in the setup phase */
-			if (game->setup_player != NULL)
-				sm_send(sm, "player %d setup\n",
-					((Player *) (game->setup_player->
-						     data))->num);
+			if (game->setup_player != NULL
+			    && (Player *) game->setup_player->data !=
+			    player) {
+				gint num =
+				    ((Player *) (game->setup_player->
+						 data))->num;
+				if (game->double_setup)
+					sm_send(sm,
+						"player %d setup-double\n",
+						num);
+				else
+					sm_send(sm, "player %d setup %d\n",
+						num, game->reverse_setup);
+			}
 
 			if (recover_from_plenty) {
 				sm_send(sm, "plenty %R\n",
@@ -681,7 +722,51 @@ gboolean mode_pre_game(Player * player, gint event)
 					player->gold, limited_bank);
 			}
 
-			player->disconnected = old_player_disconnected;
+			/* Trade was in progress */
+			if (game->curr_player != -1 &&
+			    (StateFunc) mode_domestic_initiate ==
+			    sm_stack_inspect(player_by_num
+					     (game, game->curr_player)->sm,
+					     0)) {
+				QuoteInfo *quote =
+				    quotelist_first(game->quotes);
+
+				sm_send(sm,
+					"player %d domestic-trade call supply %R receive %R\n",
+					game->curr_player,
+					game->quote_supply,
+					game->quote_receive);
+				while (quote) {
+					if (quote->is_domestic) {
+						sm_send(sm,
+							"player %d domestic-quote quote %d supply %R receive %R\n",
+							quote->var.d.
+							player_num,
+							quote->var.d.
+							quote_num,
+							quote->var.d.
+							supply,
+							quote->var.d.
+							receive);
+					}
+					quote = quotelist_next(quote);
+				}
+				/* The player already rejected all quotes,
+				 * send reject again */
+				if (state !=
+				    (StateFunc) mode_domestic_quote) {
+					sm_send(sm,
+						"player %d domestic-quote finish\n",
+						player->num);
+					/* Restore the stack */
+					sm_goto_noenter(player->sm,
+							(StateFunc)
+							mode_wait_quote_exit);
+					sm_push_noenter(player->sm,
+							(StateFunc)
+							mode_pre_game);
+				}
+			}
 			g_free(sm);
 			sm = player->sm;
 			sm_set_use_cache(sm, FALSE);
