@@ -46,7 +46,8 @@ typedef struct {
 	gint assets[NO_RESOURCE];	/* our resources */
 	gint gold;		/* how much gold will we recieve? */
 	DevelDeck *devel;	/* development cards we own */
-	GList *points;		/* points from special actions */
+	GList *special_points;	/* points from special actions */
+	gint special_points_next_id;	/* Next id for the special points */
 	gint discard_num;	/* number of resources we must discard */
 
 	gint num_roads;		/* number of roads available */
@@ -63,6 +64,7 @@ typedef struct {
 	gint gov_played;	/* number of Governors cards played */
 	gint libr_played;	/* number of Library cards played */
 	gint market_played;	/* number of Market cards played */
+	gint islands_discovered;	/* number of islands discovered */
 	gboolean disconnected;
 } Player;
 
@@ -87,6 +89,8 @@ struct Game {
 	Player *largest_army;	/* who has largest army */
 
 	QuoteList *quotes;	/* domestic trade quotes */
+	gint quote_supply[NO_RESOURCE];	/* only valid when trading */
+	gint quote_receive[NO_RESOURCE];	/* only valid when trading */
 
 	gint curr_player;	/* whose turn is it? */
 	gint curr_turn;		/* current turn number */
@@ -100,23 +104,18 @@ struct Game {
 	gint *develop_deck;	/* development cards */
 	gint develop_next;	/* next development card to deal */
 
-	gboolean register_server;	/* register game with meta-server? */
 	gchar *server_port;	/* port to run game on */
 	gboolean random_order;	/* is turn order randomized? */
 };
 
 /**** global variables ****/
-extern gboolean random_order;
-
-extern GameParams *params;
-
-
 /* buildutil.c */
 void check_longest_road(Game * game, gboolean can_cut);
 void node_add(Player * player,
-	      BuildType type, int x, int y, int pos, gboolean paid_for);
-void edge_add(Player * player,
-	      BuildType type, int x, int y, int pos, gboolean paid_for);
+	      BuildType type, int x, int y, int pos, gboolean paid_for,
+	      Points * special_points);
+void edge_add(Player * player, BuildType type, int x, int y, int pos,
+	      gboolean paid_for);
 gboolean perform_undo(Player * player);
 
 /* develop.c */
@@ -134,9 +133,9 @@ gboolean mode_wait_others_place_robber(Player * player, gint event);
 gboolean mode_discard_resources_place_robber(Player * player, gint event);
 
 /* meta.c */
-extern const gchar *meta_server_name;
 gchar *get_server_name(void);
 void meta_register(const gchar * server, const gchar * port, Game * game);
+void meta_unregister(void);
 void meta_start_game(void);
 void meta_report_num_players(gint num_players);
 void meta_send_details(Game * game);
@@ -193,37 +192,40 @@ void timed_out(int signum);
 void start_timeout(void);
 void stop_timeout(void);
 gint get_rand(gint range);
-Game *game_new(GameParams * params);
+Game *game_new(const GameParams * params);
 void game_free(Game * game);
 gint new_computer_player(const gchar * server, const gchar * port,
 			 gboolean want_chat);
-gboolean server_startup(GameParams * params, const gchar * hostname,
-			const gchar * port, gboolean meta,
+gboolean server_startup(const GameParams * params, const gchar * hostname,
+			const gchar * port, gboolean meta_register,
+			const gchar * meta_name_server,
 			gboolean random_order);
 gboolean server_stop(void);
 gboolean server_is_running(void);
 gint accept_connection(gint in_fd, gchar ** location);
-gint open_listen_socket(const gchar * port);
 void server_cleanup_static_data(void);
 
 /**** game list control functions ****/
-GameParams *game_list_find_item(const gchar * title);
+const GameParams *game_list_find_item(const gchar * title);
 void game_list_foreach(GFunc func, gpointer user_data);
 void load_game_types(const gchar * path);
 
 /**** callbacks to set parameters ****/
-void cfg_set_num_players(gint num_players);
-void cfg_set_sevens_rule(gint sevens_rule);
-void cfg_set_victory_points(gint victory_points);
-void cfg_set_game(const gchar * game);
-void cfg_set_terrain_type(gint terrain_type);
-void cfg_set_tournament_time(gint tournament_time);
-void cfg_set_quit(gboolean quitdone);
+GameParams *cfg_set_game(const gchar * game);
+GameParams *cfg_set_game_file(const gchar * game_filename);
+void cfg_set_num_players(GameParams * params, gint num_players);
+void cfg_set_sevens_rule(GameParams * params, gint sevens_rule);
+void cfg_set_victory_points(GameParams * params, gint victory_points);
+void cfg_set_terrain_type(GameParams * params, gint terrain_type);
+void cfg_set_tournament_time(GameParams * params, gint tournament_time);
+void cfg_set_quit(GameParams * params, gboolean quitdone);
 void cfg_set_timeout(gint to);
 
 /* callbacks related to server starting / stopping */
-gboolean start_server(const gchar * hostname,
-		      const gchar * port, gboolean register_server);
+gboolean start_server(const GameParams * params, const gchar * hostname,
+		      const gchar * port, gboolean register_server,
+		      const gchar * meta_server_name,
+		      gboolean random_order);
 
 /* initialize the server */
 void server_init(void);
