@@ -113,27 +113,6 @@ gboolean mode_wait_quote_exit(Player * player, gint event)
 	return FALSE;
 }
 
-/** The player has rejected/ended domestic trade.
- *  Remove all quotes from that player
- */
-static void end_quote(Player * player)
-{
-	for (;;) {
-		QuoteInfo *quote;
-		quote = quotelist_find_domestic(player->game->quotes,
-						player->num, -1);
-		if (quote == NULL)
-			break;
-		player_broadcast(player, PB_ALL,
-				 "domestic-quote delete %d\n",
-				 quote->var.d.quote_num);
-		quotelist_delete(player->game->quotes, quote);
-	}
-	player_broadcast(player, PB_RESPOND, "domestic-quote finish\n");
-
-	sm_goto(player->sm, (StateFunc) mode_wait_quote_exit);
-}
-
 gboolean mode_domestic_quote(Player * player, gint event)
 {
 	StateMachine *sm = player->sm;
@@ -147,7 +126,24 @@ gboolean mode_domestic_quote(Player * player, gint event)
 		return FALSE;
 
 	if (sm_recv(sm, "domestic-quote finish")) {
-		end_quote(player);
+		/* Player has rejected domestic trade - remove all
+		 * quotes from that player
+		 */
+		for (;;) {
+			QuoteInfo *quote;
+			quote = quotelist_find_domestic(game->quotes,
+							player->num, -1);
+			if (quote == NULL)
+				break;
+			player_broadcast(player, PB_ALL,
+					 "domestic-quote delete %d\n",
+					 quote->var.d.quote_num);
+			quotelist_delete(game->quotes, quote);
+		}
+		player_broadcast(player, PB_RESPOND,
+				 "domestic-quote finish\n");
+
+		sm_goto(sm, (StateFunc) mode_wait_quote_exit);
 		return TRUE;
 	}
 
@@ -208,7 +204,8 @@ void trade_finish_domestic(Player * player)
 	     list != NULL; list = player_next_real(list)) {
 		Player *scan = list->data;
 		if (scan != player && !player_is_viewer(game, scan->num))
-			end_quote(scan);
+			sm_goto(scan->sm,
+				(StateFunc) mode_wait_quote_exit);
 	}
 	quotelist_free(&game->quotes);
 }
@@ -287,6 +284,12 @@ static void process_call_domestic(Player * player, gint * supply,
 {
 	Game *game = player->game;
 	GList *list;
+	gint i;
+
+	for (i = 0; i < NO_RESOURCE; i++) {
+		game->quote_supply[i] = supply[i];
+		game->quote_receive[i] = receive[i];
+	}
 
 	player_broadcast(player, PB_RESPOND,
 			 "domestic-trade call supply %R receive %R\n",
